/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jmx.rmi;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.RMISocketFactory;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Logger;

import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * This is the Fractal JMX adaptor for use with MX4J tools.
 * @author ddesjardins - EBM WebSourcing
 */
@FractalComponent
public class RMIConnector {

    /**
     * AdminAttributes
     */
    @Requires(name = "adminAtt", signature = org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    protected AdminAttributes adminAttributes;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * JMX server
     */
    protected JMXConnectorServer cs;

    /**
     * Build the JMX Service URL.
     * 
     * @param host
     *            the JMX/RMI host
     * @param jmxRmiPort
     *            the RMI port
     * @return the URL in String form
     */
    public static final String getJmxServiceURL(String host, int jmxRmiPort) {
        // The RMI server definition
        StringBuffer rmiServer = new StringBuffer("rmi://");
        rmiServer.append(host).append(":").append(jmxRmiPort);
        // The JMX Service over RMI URL
        StringBuffer jmxServiceURL = new StringBuffer("service:jmx:");
        jmxServiceURL.append(rmiServer.toString()).append("/jndi/").append(rmiServer.toString())
                .append("/jmxRmiConnector");

        return jmxServiceURL.toString();
    }

    /**
     * Start the JMX RMI Server
     * 
     * @throws IOException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException {
        this.log = new LoggingUtil(Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        try {
            // Retrieve the port and host
            ContainerConfiguration containerConfiguration = this.configurationService
                    .getContainerConfiguration();
            final String rmiHost = containerConfiguration.getHost();
            final int rmiPort = containerConfiguration.getJmxRMIConnectorPort();
            final long tcpConnectionTimeout = containerConfiguration.getTCPConnectionTimeout();

            System.setProperty("sun.rmi.transport.tcp.responseTimeout",
                    Long.toString(tcpConnectionTimeout));
            System.setProperty("java.rmi.server.disableHttp", "true");

            // Create a custom RMI Direct Socket Factory to handle the
            // connection timeout properly
            RMISocketFactory rmiSocketFactory = new TimedRMIDirectConnectorFactory(
                    (int) tcpConnectionTimeout, rmiHost);
            RMISocketFactory.setSocketFactory(rmiSocketFactory);

            // create an RMI connector
            Registry registry = LocateRegistry.getRegistry(rmiPort);
            try {
                registry.list();
            } catch (Exception e) {
                LocateRegistry.createRegistry(rmiPort);
            }

            JMXServiceURL jmxUrl = new JMXServiceURL(getJmxServiceURL(rmiHost, rmiPort));

            HashMap<String, Object> env = this.setupJMXSecurity(containerConfiguration);

            this.cs = JMXConnectorServerFactory.newJMXConnectorServer(jmxUrl, env,
                    this.adminAttributes.getRawMBeanServer());
            this.cs.start();
        } catch (IOException e) {
            this.log.error("Failed to start JMX RMI connector service", e);
            throw e;
        }

        this.log.info("JMX RMI server started at : " + "service:jmx:rmi://"
                + this.cs.getAddress().getURLPath());

        this.log.end();
    }

    /**
     * Stop the RMI Server
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.start();
        if (this.cs != null) {
            try {
                this.cs.stop();
            } catch (Exception e) {
                this.log.error("Failed to stop JMX RMI connector service", e);
            }
        }
        this.log.end();
    }

    /**
     * Setup JMX security : <br>
     * create a $PETALS_HOME/work/jmx/password.properties file containing jmx
     * user/pwd values <br>
     * create a $PETALS_HOME/work/jmx/access.properties file containing jmx user
     * and readwrite <br>
     * return a hashmap referencing these files
     * 
     * @param containerConfiguration
     * @return the env hashmap
     */
    private HashMap<String, Object> setupJMXSecurity(ContainerConfiguration containerConfiguration) {
        Properties pwdProperties = new Properties();
        pwdProperties.put(containerConfiguration.getUser(), containerConfiguration.getPassword());

        Properties accessProperties = new Properties();
        accessProperties.put(containerConfiguration.getUser(), "readwrite");

        File workDirectory = new File(containerConfiguration.getWorkDirectoryPath());

        File passwordFile = new File(workDirectory, "password.properties");
        File accessFile = new File(workDirectory, "access.properties");

        try {
            pwdProperties.store(new FileOutputStream(passwordFile),
                    "role and password for JMX remote connection");
        } catch (IOException e) {
            this.log.error("Can not create " + passwordFile.getAbsolutePath()
                    + " file to store JMX password");
        }
        try {
            accessProperties.store(new FileOutputStream(accessFile),
                    "role and right for JMX remote connection");
        } catch (IOException e) {
            this.log.error("Can not create " + accessFile.getAbsolutePath()
                    + " file to store JMX rights");
        }

        HashMap<String, Object> env = new HashMap<String, Object>();
        env.put("jmx.remote.x.password.file", passwordFile.getAbsolutePath());
        env.put("jmx.remote.x.access.file", accessFile.getAbsolutePath());

        return env;

    }

}
