/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.agent;

import java.io.Serializable;
import java.util.Map;

import javax.naming.NamingException;
import javax.naming.NotContextException;
import javax.naming.ServiceUnavailableException;

import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.microkernel.communication.jndi.agent.util.RegistryUtil;

/**
 * The abstract implementation of the PEtALS JNDI agent {@link JNDIAgentService}.
 * @author Adrien Louis - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public abstract class AbstractJNDIAgentServiceImpl implements JNDIAgentService {

    public static final String INITIAL_CONTEXT = "/";

    /**
     * The domain configuration
     */
    protected DomainConfiguration domainConfiguration;

    /**
     * Logging utility
     */
    protected LoggingUtil log;

    /**
     * Data stored in this distributed registry
     */
    protected Map<String, Map<String, Serializable>> data;

    /**
     * Registry utility
     */
    protected RegistryUtil registryUtil;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.communication.jndi.agent.JNDIAgentService#processRequest
     * (org.ow2.petals.communication.jndi.agent.msg.request.RegistryRequest)
     */
    public Object processRequest(RegistryRequest request) throws NamingException {
        this.log.start(request.getType());

        Object output = null;
        switch (request.getType()) {
            /*
             * BIND
             */
            case bind:
                this.registryUtil.bind(request.getArg1(), request.getArg2(), request.getArg3());
                break;
            /*
             * CREATE SUBCONTEXT
             */
            case createSubcontext:
                output = this.registryUtil.createSubcontext(request.getArg1(), request.getArg2());
                break;
            /*
             * DESTROY SUBCONTEXT
             */
            case destroySubcontext:
                this.registryUtil.destroySubcontext(request.getArg1(), request.getArg2());
                break;
            /*
             * LIST
             */
            case list:
                output = this.registryUtil.list(request.getArg1(), request.getArg2());
                break;
            /*
             * LIST BINDINGS
             */
            case listBindings:
                output = this.registryUtil.listBindings(request.getArg1(), request.getArg2(),
                        request.isLocalConnection());
                break;
            /*
             * LOOKUP
             */
            case lookup:
                output = this.registryUtil.lookup(request.getArg1(), request.getArg2(), request
                        .isLocalConnection());
                break;
            /*
             * LOOKUP LINK
             */
            case lookupLink:
                output = this.registryUtil.lookupLink(request.getArg1(), request.getArg2(), request
                        .isLocalConnection());
                break;
            /*
             * REBIND
             */
            case rebind:
                this.registryUtil.rebind(request.getArg1(), request.getArg2(), request.getArg3());
                break;
            /*
             * RENAME
             */
            case rename:
                this.registryUtil.rename(request.getArg1(), request.getArg2(), request.getArg3(),
                        request.isLocalConnection());
                break;
            /*
             * UNBIND
             */
            case unbind:
                this.registryUtil.unbind(request.getArg1(), request.getArg2());
                break;

            default:
                break;
        }

        this.log.end();
        return output;
    }

    public LoggingUtil getLog() {
        return this.log;
    }

    public DomainConfiguration getDomainConfiguration() {
        return this.domainConfiguration;
    }

    /**
     * Check if a naming Context matching the given key exits.
     * 
     * @param key
     * @return {@code true} if it exists
     */
    public abstract boolean containsContext(String key) throws ServiceUnavailableException;

    /**
     * Get the Naming Context matching the given key. Create a new Naming
     * Context if it doesn't exist.
     * 
     * @param key
     *      The key bound to the Naming Context
     * @return The matching Naming Context map
     * 
     * @throws NotContextException
     */
    public abstract Map<String, Serializable> getContext(String key)
            throws ServiceUnavailableException;

    /**
     * Remove the naming Context matching the given key.
     * 
     * @param key
     * @return
     * @throws ServiceUnavailableException
     */
    public abstract void removeContext(String key) throws ServiceUnavailableException;
}
