/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.client.naming;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.URI;

import javax.naming.NamingException;
import javax.naming.ServiceUnavailableException;

import org.ow2.petals.microkernel.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.RegistryResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.RegistryResponse.ResponseType;

/**
 * Connection to the PEtALS JNDI agent.
 * @author ddesjardins - EBM WebSourcing
 */
public class JNDIConnection implements NamingConnection, Serializable {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 815676782982350588L;

    /**
     * Port of the PEtALS JNDI Agent
     */
    protected int port;

    /**
     * Host of the PEtALS JNDI Agent
     */
    protected String host;

    /**
     * Creates a new instance of {@link JNDIConnection}
     * 
     * @param port
     * @param host
     */
    public JNDIConnection(URI jndiURI) {
        super();
        this.port = jndiURI.getPort();
        this.host = jndiURI.getHost();
    }

    /**
     * Send a request to a Registry server
     * 
     * @param request
     *            request to send
     * @return return of the request
     * @throws NamingException
     *             if a NamingException is thrown on the server
     */
    public synchronized Object send(RegistryRequest request) throws NamingException {
        Object out = null;
        Socket socket = null;
        try {
            socket = new Socket();
            socket.connect(new InetSocketAddress(host, port), 1000);
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(socket.getOutputStream());
            objectOutputStream.writeObject(request);
            objectOutputStream.flush();

            ObjectInputStream objectInputStream = new ObjectInputStream(socket.getInputStream());
            RegistryResponse response = (RegistryResponse) objectInputStream.readObject();

            if (response.getType() == ResponseType.exception) {
                throw (NamingException) response.getArg1();
            }
            out = response.getArg1();
        } catch (IOException e) {
            throw new ServiceUnavailableException("Problem while sending request to server " + host
                    + ":" + port);
        } catch (ClassNotFoundException e) {
            throw new ServiceUnavailableException("Problem while sending request to server " + host
                    + ":" + port);
        } finally {
            // close the socket and its streams
            if (socket != null) {
                try {
                    socket.getInputStream().close();
                } catch (IOException e) {
                }
                try {
                    socket.getOutputStream().close();
                } catch (IOException e) {
                }
                try {
                    socket.close();
                } catch (IOException e) {
                }
            }
        }
        return out;
    }
}
