/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.topology;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.clientserverapi.topology.exception.TopologyServiceException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.communication.topology.exception.AddSubDomainException;
import org.ow2.petals.microkernel.api.communication.topology.exception.RemoveContainerException;
import org.ow2.petals.microkernel.api.communication.topology.exception.RemoveSubDomainException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.communication.RemoteCheckerClient;

/**
 * The default implementation of the topology service.
 * @author Christophe Hamerling - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = TopologyService.class))
public class TopologyServiceImpl implements TopologyService {

    /**
     * Inner class to ping the remote container.
     * @author Roland Naudin - EBM WebSourcing
     */
    private class TopologyPinger extends TimerTask {

        @Override
        public void run() {
            for (String containerName : containerStates.keySet()) {
                if (TopologyServiceImpl.this.stopped) {
                    break;
                }
                if (!containerName.equals(TopologyServiceImpl.this.configurationService
                        .getContainerConfiguration().getName()))
                    if (TopologyServiceImpl.this.remoteChecker.ping(containerName)) {
                        TopologyServiceImpl.this.setContainerState(containerName,
                                ContainerState.STARTED);
                        TopologyServiceImpl.this.log.debug("Container '" + containerName
                                + "' started");
                    } else {
                        TopologyServiceImpl.this.setContainerState(containerName,
                                ContainerState.STOPPED);

                        TopologyServiceImpl.this.log.debug("Container '" + containerName
                                + "' stopped");
                    }
            }
        }

    }

    private static final long TIMER_PERIOD = 5000;

    private static final long TIMER_START = 2000;

    /**
     * Logger wrapper.
     */
    protected LoggingUtil log;
    
    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    private Map<String, ContainerState> containerStates = new ConcurrentHashMap<String, ContainerState>();

    /**
     * Flag to know if the fractal component is stopping
     */
    private boolean stopped;

    /**
     * The Ping service<br>
     * TODO: It must be a ping via the transporter instead of JMX
     */
    @Requires(name = "remote", signature = RemoteCheckerClient.class)
    private RemoteCheckerClient remoteChecker;

    /**
     * The timer to ping the remote containers
     */
    private Timer topologyTimer;

    /**
     * {@inheritDoc}
     */
    public void addContainerConfiguration(ContainerConfiguration addedContainer)
            throws AddContainerException {
        if (addedContainer == null) {
            throw new AddContainerException(null);
        }

        try {
            this.configurationService.addContainerConfiguration(addedContainer);
        } catch (final ConfigurationException e) {
            throw new AddContainerException(addedContainer.getName(), e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void addContainerConfiguration(final ContainerConfiguration addedContainer,
            final String subdomainName) throws AddContainerException {
        this.addContainerConfiguration(addedContainer, subdomainName, true);
    }

    /**
     * {@inheritDoc}
     */
    public void addContainerConfiguration(final ContainerConfiguration addedContainer,
            final String subdomainName, final boolean validateTopology)
            throws AddContainerException {

        if (!this.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new AddContainerException(
                    "No dynamic configuration found on the target container");
        }

        if (addedContainer == null) {
            final StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't add a null container to the topology");
            throw new AddContainerException(errorSB.toString());
        }

        // Check if the subdomain has been already registered...
        boolean existingSubdomain = false;
        final Set<SubDomainConfiguration> subdomainsConfigurations = this
                .getSubDomainsConfiguration();
        for (final SubDomainConfiguration subdomainConfiguration : subdomainsConfigurations) {
            if (subdomainConfiguration.getName().equals(subdomainName)) {
                existingSubdomain = true;
                break;
            }
        }

        if (!existingSubdomain) {
            final StringBuilder errorSB = new StringBuilder();
            errorSB.append("You must register a new subdomain with name : " + subdomainName);
            throw new AddContainerException(errorSB.toString());
        }

        this.addContainerConfiguration(addedContainer);
    }

    /**
     * {@inheritDoc}
     */
    public void addSubdomainConfiguration(SubDomainConfiguration addedSubdomain)
            throws AddSubDomainException {
        if (addedSubdomain == null) {
            throw new AddSubDomainException(null);
        }

        try {
            this.configurationService.addSubdomainConfiguration(addedSubdomain);
        } catch (final ConfigurationException e) {
            throw new AddSubDomainException(addedSubdomain.getName(), e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public ContainerConfiguration getContainerConfiguration(String containerName) {
        // FIXME : Default get the container configuration from the
        // configuration service. This container information is not updated at
        // runtime!
        return this.configurationService.getContainerConfiguration(containerName);
    }

    /**
     * {@inheritDoc}
     */
    public Set<ContainerConfiguration> getContainersConfiguration(ContainerState state) {
        Set<ContainerConfiguration> result = null;

        final Set<ContainerConfiguration> set = this.configurationService
                .getContainersConfiguration();
        if (state == null) {
            result = set;
        } else {
            result = new HashSet<ContainerConfiguration>(set.size());
            for (ContainerConfiguration containerConfiguration : set) {
                if ((containerConfiguration.getState() != null)
                        && containerConfiguration.getState().equals(state)) {
                    result.add(containerConfiguration);
                }
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public Set<ContainerConfiguration> getContainersConfigurationsForLocalSubdomain() {
        final Set<ContainerConfiguration> containersConfigurationForLocalSubdomain = new HashSet<ContainerConfiguration>();

        final String localSubdomainName = this.retrieveLocalSubdomainName();

        for (ContainerConfiguration containerConfiguration : this
                .retrieveAllContainersConfigurations()) {
            if (containerConfiguration.getSubdomainName().equals(localSubdomainName)) {
                containersConfigurationForLocalSubdomain.add(containerConfiguration);
            }
        }

        return containersConfigurationForLocalSubdomain;
    }

    /**
     * {@inheritDoc}
     */
    public DomainConfiguration getDomainConfiguration() {
        return this.configurationService.getDomainConfiguration();
    }

    /**
     * {@inheritDoc}
     */
    public Set<SubDomainConfiguration> getSubDomainsConfiguration() {
        return this.configurationService.getSubDomainsConfiguration();
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasValidLocalContainerDynamicTopologyConfiguration() {
        if (this.configurationService.getDomainConfiguration().getMode() != DomainMode.DYNAMIC) {
            return false;
        }

        if (this.configurationService.getContainerConfiguration().isSlave()
                || this.configurationService.getContainerConfiguration().isMaster()) {
            return true;
        }

        return false;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isContainerStarted(String containerName) {
        return ContainerState.STARTED.equals(this.containerStates.get(containerName));
    }

    // Register the current node to the master
    public void registerLocalContainerOnMaster() throws TopologyServiceException {
        try {
            if (!this.hasValidLocalContainerDynamicTopologyConfiguration()) {
                return;
            }

            final ContainerConfiguration localContainerConfiguration = this.configurationService
                    .getContainerConfiguration();
            if (localContainerConfiguration.isMaster()) {
                return;
            }
            
            final ContainerConfiguration masterContainerConfiguration = retrieveMasterContainerConfiguration();
            
            final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(
                    masterContainerConfiguration.getHost(),
                    masterContainerConfiguration.getJmxRMIConnectorPort(),
                    masterContainerConfiguration.getUser(),
                    masterContainerConfiguration.getPassword());

            final String localSubdomainName = this.retrieveLocalSubdomainName();

            jmxClient.getTopologyServiceClient().addContainerConfiguration(
                    localContainerConfiguration, localSubdomainName);

        } catch (final ConnectionErrorException e) {
            throw new TopologyServiceException(e);
        } catch (final DuplicatedServiceException e) {
            throw new TopologyServiceException(e);
        } catch (final MissingServiceException e) {
            throw new TopologyServiceException(e);
        } catch (final TopologyServiceErrorException e) {
            throw new TopologyServiceException(e);
        } catch (final TopologyServiceDoesNotExistException e) {
            throw new TopologyServiceException(e);
        } catch (final PetalsException e) {
            throw new TopologyServiceException(e);
        }

    }

    /**
     * {@inheritDoc}
     */
    public void removeContainerConfiguration(ContainerConfiguration removedContainer)
            throws RemoveContainerException {
        try {
            this.configurationService.removeContainerConfiguration(removedContainer);
        } catch (final ConfigurationException e) {
            throw new RemoveContainerException(removedContainer.getName(), e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void removeSubdomainConfiguration(String removedSubdomain)
            throws RemoveSubDomainException {
        try {
            this.configurationService.removeSubdomainConfiguration(removedSubdomain);
        } catch (final ConfigurationException e) {
            throw new RemoveSubDomainException(removedSubdomain, e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setContainerState(String containerName, ContainerState state) {
        this.containerStates.put(containerName, state);
    }

    /**
     * {@inheritDoc}
     */
    public void validateDynamicTopology() throws TopologyServiceException {
        try {
            this.configurationService.validateDynamicTopology();
        } catch (final ConfigurationException e) {
            throw new TopologyServiceException("Error validating the dynamic topology", e);
        }
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        final Set<ContainerConfiguration> containers = this.configurationService
                .getContainersConfiguration();

        for (ContainerConfiguration containerConfiguration : containers) {
            this.containerStates.put(containerConfiguration.getName(), ContainerState.UNKNOWN);
        }

        this.stopped = false;
        this.topologyTimer = new Timer("Topology Pinger");
        this.topologyTimer.schedule(new TopologyPinger(), TIMER_START, TIMER_PERIOD);

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.start();

        this.stopped = true;
        this.topologyTimer.cancel();

        this.log.end();
    }

    private Set<ContainerConfiguration> retrieveAllContainersConfigurations() {
        final Set<ContainerConfiguration> containersConfigurationsSet = this.configurationService
                .getContainersConfiguration();

        return containersConfigurationsSet;
    }

    private ContainerConfiguration retrieveLocalContainerConfiguration() {
        final ContainerConfiguration localContainerConfiguration = this.configurationService
                .getContainerConfiguration();

        return localContainerConfiguration;
    }

    private String retrieveLocalSubdomainName() {
        final String localContainerName = this.retrieveLocalContainerConfiguration().getName();
        String subdomainName = null;

        for (ContainerConfiguration containerConfiguration : this
                .retrieveAllContainersConfigurations()) {
            if (containerConfiguration.getName().equals(localContainerName)) {
                subdomainName = containerConfiguration.getSubdomainName();
                break;
            }
        }

        return subdomainName;
    }

    private ContainerConfiguration retrieveMasterContainerConfiguration() {
        ContainerConfiguration masterContainerConfiguration = null;

        final ContainerConfiguration localContainerConfiguration = this
                .retrieveLocalContainerConfiguration();

        if (localContainerConfiguration.isMaster()) {
            masterContainerConfiguration = localContainerConfiguration;
        } else {
            final Set<ContainerConfiguration> localSubdomainContainersConfigurations = this
                    .getContainersConfigurationsForLocalSubdomain();
            for (ContainerConfiguration containerConfiguration : localSubdomainContainersConfigurations) {
                if (containerConfiguration.isMaster()) {
                    masterContainerConfiguration = containerConfiguration;
                    break;
                }
            }
        }

        return masterContainerConfiguration;
    }
}
