/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.container.lifecycle;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.jbi.JBIException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * This class manages the life cycle of deployed service assemblies.
 * @author Olivier Fabre - EBM WebSourcing
 * @author ddesjardins - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ServiceAssemblyLifeCycle.class))
public class ServiceAssemblyLifeCycleImpl extends AbstractLifeCycle implements
        ServiceAssemblyLifeCycle {

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate")
    private SystemStateService systemState;

    /**
     * JBI messaging component :: Router Service
     */
    @Requires(name = "router")
    private RouterService routerService;

    /**
     * The SA JBI descriptor
     */
    private ServiceAssembly serviceAssembly;

    /**
     * List of SU life-cycles
     */
    private List<ServiceUnitLifeCycle> serviceUnitsLifeCycles;

    public ServiceAssemblyLifeCycleImpl() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    public void init(ServiceAssembly serviceAssembly) throws JBIException {
        this.serviceAssembly = serviceAssembly;
        this.name = serviceAssembly.getIdentification().getName();

        // set the state to SHUTDOWN, as the JBI specifications specifies
        this.setState(SHUTDOWN);
    }

    /**
     * Do nothing
     */
    @Override
    public void doInit() throws JBIException {
        // Do nothing
    }

    @Override
    public void doShutdown() throws JBIException {
        final List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                this.serviceUnitsLifeCycles);
        final List<ServiceUnitLifeCycle> reversedSuLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                suLifeCycles);
        Collections.reverse(reversedSuLifeCycles);

        for (final ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            try {
                if (STOPPED.equals(suLifeCycle.getCurrentState())) {
                    suLifeCycle.shutDown();
                } else if (!this.forcedState) {
                    throw new JBIException(
                            "Unable to shut down a service unit which is not stopped");
                }
                if (UNKNOWN.equals(suLifeCycle.getCurrentState()) && !this.forcedState) {
                    throw new JBIException("Unable to shutdown a service unit");
                }
            } catch (final JBIException e) {
                if (!this.forcedState) {
                    throw e;
                }
                else {
                    // continue, ignoring the error
                    this.log.warning("An ignored error occurs shutdowing the service unit.", e);
                }
            }
        }

    }

    @Override
    public void doStart() throws JBIException {
        // The JBI specifications force to initialize all the service units
        // before starting all the service units
        for (final ServiceUnitLifeCycle suLifeCycle : this.serviceUnitsLifeCycles) {
            if (SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.init();
            }
            if (UNKNOWN.equals(suLifeCycle.getCurrentState())) {
                throw new JBIException("Unable to initialize a service unit");
            }
        }
        for (final ServiceUnitLifeCycle suLifeCycle : this.serviceUnitsLifeCycles) {
            if (STOPPED.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.start();
            }
            if (UNKNOWN.equals(suLifeCycle.getCurrentState())) {
                throw new JBIException("Unable to start a service unit");
            }
        }
    }

    @Override
    public void doStop() throws JBIException {
        final List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                this.serviceUnitsLifeCycles);
        final List<ServiceUnitLifeCycle> reversedSuLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                suLifeCycles);
        Collections.reverse(suLifeCycles);
        for (final ServiceUnitLifeCycle suLifeCycle : reversedSuLifeCycles) {
            try {
                if (STARTED.equals(suLifeCycle.getCurrentState())) {
                    suLifeCycle.stop();
                } else if (!this.forcedState) {
                    throw new JBIException("Unable to stop a service unit which is not started");
                }
                if (UNKNOWN.equals(suLifeCycle.getCurrentState()) && !this.forcedState) {
                    throw new JBIException("Unable to stop a service unit");
                }
            } catch (final JBIException e) {
                if (!this.forcedState) {
                    throw e;
                }
                else {
                    // continue, ignoring the error
                    this.log.warning("An ignored error occurs stopping the service unit.", e);
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public ServiceAssembly getServiceAssembly() {
        return this.serviceAssembly;
    }

    /**
     * @param serviceAssembly
     *            the serviceAssembly to set
     */
    public void setServiceAssembly(ServiceAssembly serviceAssembly) {
        this.serviceAssembly = serviceAssembly;
    }

    /**
     * {@inheritDoc}
     */
    public List<ServiceUnitLifeCycle> getServiceUnitLifeCycles() {
        return this.serviceUnitsLifeCycles;
    }

    /**
     * {@inheritDoc}
     */
    public void registerSU(ServiceUnitLifeCycle serviceUnitLifeCycle) throws JBIException {
        if (!this.serviceUnitsLifeCycles.contains(serviceUnitLifeCycle.getSuName())) {
            this.serviceUnitsLifeCycles.add(serviceUnitLifeCycle);
        } else {
            throw new JBIException("Duplicate service unit : same name");
        }
    }

    /**
     * {@inheritDoc}
     */
    public void unregisterSU(ServiceUnitLifeCycle serviceUnitLifeCycle) {
        this.serviceUnitsLifeCycles.remove(serviceUnitLifeCycle);
    }

    @Override
    public void setState(String state) throws JBIException {
        super.setState(state);

        this.routerService.modifiedSALifeCycle(this.serviceUnitsLifeCycles);

        try {
            this.systemState.updateServiceAssemblyState(this.name, state);
        } catch (final Exception e) {
            throw new JBIException("Failed to persist the state of the Service Assembly '"
                    + this.name + "'", e);
        }
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void startComponentLifeCycle() {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(ServiceAssemblyLifeCycle.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.serviceUnitsLifeCycles = new ArrayList<ServiceUnitLifeCycle>();

        this.log.end();
    }

    /**
     * Stop of the petals component
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stopComponentLifeCycle() throws JBIException {
        this.log.start();

        if (STARTED.equals(this.getCurrentState())) {
            try {
                this.doStop();
                this.doShutdown();
            } catch (final JBIException e) {
                this.log.error("Failed to stop the Service Assembly '" + this.name + "'", e);
            }
        }

        if (STOPPED.equals(this.getCurrentState())) {
            try {
                this.doShutdown();
            } catch (final JBIException e) {
                this.log.error("Failed to shut down the Service Assembly '" + this.name + "'", e);
            }
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.LifeCycleAbstract#getJBIArtifactType()
     */
    @Override
    protected String getJBIArtifactType() {
        return "Service Assembly";
    }
}
