/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.admin;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.JBIException;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.InstallerMBean;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.descriptor.original.generated.ComponentType;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.microkernel.api.communication.JMXService;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.MBeanNameBuilder;
import org.ow2.petals.microkernel.server.MBeanHelper;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;

/**
 * TODO AdminServiceImpl : check the exceptions catch.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = AdminService.class))
public class AdminServiceImpl implements AdminService {

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * The JMX Fractal service
     */
    @Requires(name = "jmx")
    private JMXService jmxService;

    private Map<String, Installer> installersByName;

    private Map<ObjectName, ComponentLifeCycleMBean> bindingLifeCyclesByMBeanName;

    private Map<ObjectName, ComponentLifeCycleMBean> engineLifeCyclesByMBeanName;

    private Map<String, ComponentLifeCycleMBean> bindingLifeCyclesByName;

    private Map<String, ComponentLifeCycleMBean> engineLifeCyclesByName;

    private Map<String, SharedLibraryLifeCycle> sharedLibraryLifeCycles;

    private Map<String, ServiceAssemblyLifeCycle> serviceAssemblyLifeCycles;

    @Requires(name = "mbean-name-builder")
    private MBeanNameBuilder mBeanNames;

    private MBeanServer mbeanServer;

    /**
     * Default constructor
     */
    public AdminServiceImpl() {
        super();
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents() {
        this.log.call();

        ObjectName[] names = new ObjectName[0];
        names = this.bindingLifeCyclesByMBeanName.keySet().toArray(names);

        return names;
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    public ObjectName[] getEngineComponents() {
        this.log.call();

        ObjectName[] names = new ObjectName[0];
        names = this.engineLifeCyclesByMBeanName.keySet().toArray(names);

        return names;
    }

    /*
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.
     * String)
     */
    public ObjectName getComponentByName(String name) {
        this.log.call();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(name, "name cannot be null", this.log);

        ObjectName result = null;

        for (ObjectName mbeanName : this.bindingLifeCyclesByMBeanName.keySet()) {
            if (name.equals(this.mBeanNames.getNameFromMBeanName(mbeanName))) {
                result = mbeanName;
                break;
            }
        }
        if (result == null) {
            for (ObjectName mbeanName : this.engineLifeCyclesByMBeanName.keySet()) {
                if (name.equals(this.mBeanNames.getNameFromMBeanName(mbeanName))) {
                    result = mbeanName;
                    break;
                }
            }
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo() {
        return new String("Petals JBI Container "
                + AdminServiceImpl.class.getPackage().getImplementationVersion() + "\n"
                + System.getProperty("java.runtime.name") + " "
                + System.getProperty("java.vm.version") + " "
                + System.getProperty("java.vm.vendor") + "\n" + System.getProperty("os.name") + " "
                + System.getProperty("os.version") + " " + System.getProperty("os.arch"));
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName) {
        this.log.call();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceName,
                "serviceName cannot be null", this.log);

        ObjectName result = null;
        ObjectName[] systemServices = this.getSystemServices();

        for (ObjectName objectName : systemServices) {
            if (objectName.getCanonicalName().equals(serviceName)) {
                result = objectName;
                break;
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    public ObjectName[] getSystemServices() {
        this.log.call();

        ObjectName[] names = new ObjectName[3];

        try {
            names[0] = MBeanHelper.retrieveServiceMBean(
                    AdminServiceClient.ADMIN_SERVICE_MBEAN_NAME, this.mbeanServer);
            names[1] = MBeanHelper.retrieveServiceMBean(
                    DeploymentServiceClient.DEPLOYMENT_SERVICE_MBEAN_NAME,
                    this.mbeanServer);
            names[2] = MBeanHelper.retrieveServiceMBean(
                    InstallationServiceClient.INSTALLATION_SERVICE_MBEAN_NAME,
                    this.mbeanServer);
        } catch (MalformedObjectNameException e) {
            this.log.warning("A system service name can't be retrieved !", e);
        }

        return names;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(componentName,
                "componentName cannot be null", this.log);

        if (this.bindingLifeCyclesByName.containsKey(componentName)) {
            return true;
        } else {
            final Installer installer = this.installersByName.get(componentName);
            if (installer == null) {
                return false;
            } else {
                return installer.getJBIDescriptor().getComponent().getType() == ComponentType.BINDING_COMPONENT;
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(componentName,
                "componentName cannot be null", this.log);

        if (this.engineLifeCyclesByName.containsKey(componentName)) {
            return true;
        } else {
            final Installer installer = this.installersByName.get(componentName);
            if (installer == null) {
                return false;
            } else {
                return installer.getJBIDescriptor().getComponent().getType() == ComponentType.SERVICE_ENGINE;
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminServiceMBean#
     * isExchangeWithConsumerOkayForComponent(java.lang.String,
     * javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        this.log.start();

        Installer installer = this.installersByName.get(componentName);
        if (installer == null) {
            throw new JBIException("Cannot find a component '" + componentName
                    + "' in the container");
        }

        boolean result = installer.getComponent().isExchangeWithConsumerOkay(internalEndpoint,
                exchange);

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminServiceMBean#
     * isExchangeWithProviderOkayForComponent(java.lang.String,
     * javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        this.log.start();

        Installer installer = this.installersByName.get(componentName);
        if (installer == null) {
            throw new JBIException("Cannot find a component '" + componentName
                    + "' in the container");
        }

        boolean result = installer.getComponent().isExchangeWithProviderOkay(internalEndpoint,
                exchange);

        this.log.end();
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public MBeanNames getMBeanNames() {
        return this.mBeanNames;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminService#getMBeanServer()
     */
    public MBeanServer getMBeanServer() {
        return this.mbeanServer;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getBindingComponentLifeCycleByName(java.lang.String)
     */
    public ComponentLifeCycleMBean getBindingComponentLifeCycleByName(String name) {
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(name, "name cannot be null or empty",
                this.log);
        this.log.call();

        return this.bindingLifeCyclesByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getEngineComponentLifeCycleByName(java.lang.String)
     */
    public ComponentLifeCycleMBean getEngineComponentLifeCycleByName(String name) {
        this.log.call();

        return this.engineLifeCyclesByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getInstallerByName(java
     * .lang.String)
     */
    public Installer getInstallerByName(String name) {
        return this.installersByName.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminService#getInstallers()
     */
    public Installer[] getInstallers() {
        return this.installersByName.values().toArray(new Installer[this.installersByName.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName) {
        return this.serviceAssemblyLifeCycles.get(saName);
    }

    /**
     * {@inheritDoc}
     */
    public Map<String, ServiceAssemblyLifeCycle> getServiceAssemblies() {
        return this.serviceAssemblyLifeCycles;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getSharedLibraryByName
     * (java.lang.String)
     */
    public SharedLibraryLifeCycle getSharedLibraryByName(String slName) {
        return this.sharedLibraryLifeCycles.get(slName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#getSharedLibraries()
     */
    public Map<String, SharedLibraryLifeCycle> getSharedLibraries() {
        return this.sharedLibraryLifeCycles;
    }

    public Map<String, ComponentLifeCycleMBean> getBindingComponentsLifeCycles() {
        return this.bindingLifeCyclesByName;
    }

    /**
     * {@inheritDoc}
     */
    public Map<String, ComponentLifeCycleMBean> getServiceEnginesLifeCycles() {
        return this.engineLifeCyclesByName;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.admin.AdminService#
     * getServiceUnitsLifeCyclesForComponent(java.lang.String)
     */
    public List<ServiceUnitLifeCycle> getServiceUnitsLifeCyclesForComponent(String component) {
        this.log.call();

        List<ServiceUnitLifeCycle> serviceUnitLifeCycles = new ArrayList<ServiceUnitLifeCycle>();

        for (final ServiceAssemblyLifeCycle serviceAssemblyLifeCycle : this.serviceAssemblyLifeCycles
                .values()) {
            for (ServiceUnitLifeCycle serviceUnitLifeCycle : serviceAssemblyLifeCycle
                    .getServiceUnitLifeCycles()) {
                if (serviceUnitLifeCycle.getTargetComponentName().equals(component)) {
                    serviceUnitLifeCycles.add(serviceUnitLifeCycle);
                }
            }
        }

        return serviceUnitLifeCycles;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerBindingComponent
     * (javax.jbi.management.ComponentLifeCycleMBean, java.lang.String)
     */
    public ObjectName registerBindingComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = this.mBeanNames.createBindingComponentMBeanName(name);
        try {
            this.mbeanServer.registerMBean(
                    MBeanHelper.createCustomMBean(lifeCycle, ComponentLifeCycleMBean.class),
                    mbeanName);
            this.bindingLifeCyclesByMBeanName.put(mbeanName, lifeCycle);
            this.bindingLifeCyclesByName.put(name, lifeCycle);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerEngineComponent
     * (javax.jbi.management.ComponentLifeCycleMBean, java.lang.String)
     */
    public ObjectName registerEngineComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = this.mBeanNames.createEngineComponentMBeanName(name);
        try {
            this.mbeanServer.registerMBean(
                    MBeanHelper.createCustomMBean(lifeCycle, ComponentLifeCycleMBean.class),
                    mbeanName);
            this.engineLifeCyclesByMBeanName.put(mbeanName, lifeCycle);
            this.engineLifeCyclesByName.put(name, lifeCycle);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerInstaller(org
     * .ow2.petals.container.lifecycle.Installer, java.lang.String)
     */
    public ObjectName registerInstaller(Installer installer, String name)
            throws ManagementException {
        this.log.call();

        ObjectName mbeanName = this.mBeanNames.createInstallerMBeanName(name);
        try {
            this.mbeanServer.registerMBean(
                    MBeanHelper.createCustomMBean(installer, InstallerMBean.class), mbeanName);
            this.installersByName.put(name, installer);
        } catch (Exception e) {
            throw new ManagementException(e);
        }

        return mbeanName;
    }

    /**
     * {@inheritDoc}
     */
    public void registerServiceAssembly(final ServiceAssemblyLifeCycle serviceAssemblyLifeCycle,
            String name) {
        this.log.call();

        this.serviceAssemblyLifeCycles.put(name, serviceAssemblyLifeCycle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#registerSharedLibrary
     * (org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle,
     * java.lang.String)
     */
    public void registerSharedLibrary(SharedLibraryLifeCycle sharedLibraryLifeCycle, String name) {
        this.sharedLibraryLifeCycles.put(name, sharedLibraryLifeCycle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterBindingComponent
     * (java.lang.String)
     */
    public void unregisterBindingComponent(String name) throws ManagementException {
        this.log.call();

        ObjectName objectName = this.mBeanNames.createBindingComponentMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.bindingLifeCyclesByMBeanName.remove(objectName);
        this.bindingLifeCyclesByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterEngineComponent
     * (java.lang.String)
     */
    public void unregisterEngineComponent(String name) throws ManagementException {
        this.log.call();

        ObjectName objectName = this.mBeanNames.createEngineComponentMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.engineLifeCyclesByMBeanName.remove(objectName);
        this.engineLifeCyclesByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterInstaller(
     * java.lang.String)
     */
    public void unregisterInstaller(String name) throws ManagementException {
        ObjectName objectName = this.mBeanNames.createInstallerMBeanName(name);
        try {
            this.mbeanServer.unregisterMBean(objectName);
        } catch (InstanceNotFoundException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        } catch (MBeanRegistrationException e) {
            throw new ManagementException("Failed to unregistered the binding Component '" + name
                    + "'", e);
        }

        this.installersByName.remove(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterServiceAssembly
     * (java.lang.String)
     */
    public void unregisterServiceAssembly(String name) throws ManagementException {
        if (this.serviceAssemblyLifeCycles.remove(name) == null) {
            throw new ManagementException("Failed to unregistered the Service Assembly '" + name
                    + "'");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.admin.AdminService#unregisterSharedLibrary
     * (java.lang.String)
     */
    public void unregisterSharedLibrary(String name) throws ManagementException {
        if (this.sharedLibraryLifeCycles.remove(name) == null) {
            throw new ManagementException("Failed to unregistered the Shared Library '" + name
                    + "'");
        }
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IllegalLifeCycleException {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(AdminService.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.mbeanServer = this.jmxService.getLocalJMXServer();

        this.bindingLifeCyclesByMBeanName = new ConcurrentHashMap<ObjectName, ComponentLifeCycleMBean>();
        this.engineLifeCyclesByMBeanName = new ConcurrentHashMap<ObjectName, ComponentLifeCycleMBean>();

        this.bindingLifeCyclesByName = new ConcurrentHashMap<String, ComponentLifeCycleMBean>();
        this.engineLifeCyclesByName = new ConcurrentHashMap<String, ComponentLifeCycleMBean>();

        this.serviceAssemblyLifeCycles = new ConcurrentHashMap<String, ServiceAssemblyLifeCycle>();

        this.installersByName = new ConcurrentHashMap<String, Installer>();

        this.sharedLibraryLifeCycles = new ConcurrentHashMap<String, SharedLibraryLifeCycle>();

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws IllegalLifeCycleException {
        this.log.call();

    }

}
