/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.deployment;

import static org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE;

import java.io.File;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.DeploymentException;
import javax.jbi.management.LifeCycleMBean;

import org.apache.commons.io.IOUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.commons.exception.ExceptionUtil;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.DeploymentService;
import org.ow2.petals.microkernel.api.jbi.management.TaskProcessor;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult.MessageType;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult.TaskResult;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.extension.InstallationExtensionsManager;
import org.ow2.petals.microkernel.jbi.management.task.deployment.DeploymentTaskFactory;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;
import org.ow2.petals.systemstate.generated.ServiceAssembly;

/**
 * JBI deployment service implementation.
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author ddesjardins - EBM WebSourcing
 * @author wjoseph - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = DeploymentService.class))
public class DeploymentServiceImpl implements DeploymentService {

    /**
     * Container :: Container Service
     */
    @Requires(name = "container")
    protected ContainerService container;

    /**
     * JBI management :: Admin service
     */
    @Requires(name = "admin")
    private AdminService adminService;

    /**
     * Configuration Service
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    private DeploymentTaskFactory deploymentTaskFactory;

    /**
     * JBI messaging :: Endpoint Service
     */
    @Requires(name = "endpoint")
    private EndpointRegistry endpointRegistry;

    @Requires(name = "installationExtensionManager")
    private InstallationExtensionsManager installationExtensionManager;

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    /**
     * Platform :: Repository Service
     */
    @Requires(name = "repository")
    private RepositoryService repositoryService;

    /**
     * Platform :: SystemState Service
     */
    @Requires(name = "systemstate")
    private SystemStateService systemStateService;

    /**
     * Default constructor.
     */
    public DeploymentServiceImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java
     * .lang.String)
     */
    public boolean canDeployToComponent(String componentName) {
        this.log.call();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        boolean result = false;
        Installer installer = this.adminService.getInstallerByName(componentName);
        if (installer.isInstalled() && (installer.getComponent().getServiceUnitManager() != null)) {
            result = true;
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
     */
    public String deploy(String serviceAssemblyZipUrl) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl, "serviceAssemblyZipUrl",
                this.log);

        String result = null;
        try {
            URL archiveURL;
            try {
                archiveURL = new URL(serviceAssemblyZipUrl);
            } catch (MalformedURLException e) {
                archiveURL = new File(serviceAssemblyZipUrl).toURI().toURL();
            }

            // Create deployment execution context
            Context deploymentContext = this.createDeploymentContext(archiveURL);

            // Create deployment processor and fill it with tasks
            TaskProcessor processor = this.createDeploymentProcessor(deploymentContext, archiveURL);

            // Launch the process
            final Throwable processResult = processor.process();
            if (processResult != null) {
                throw new DeploymentException("The deployment of the Service Assembly '"
                        + serviceAssemblyZipUrl + "' failed.", processResult);
            }

            // get the result string
            result = deploymentContext.getXmlResult().getString();
            if (result.contains("<task-result>FAILED</task-result>")) {
                this.log.warning("Service Assembly '" + deploymentContext.getEntityName()
                        + "' deployed with some SU deployment in failure");
            } else {
                this.log.info("Service Assembly '" + deploymentContext.getEntityName()
                        + "' deployed");
            }
        } catch (Exception e) {
            this.log.error("Unable to deploy the service assembly at '" + serviceAssemblyZipUrl
                    + "'", e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new Exception(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.end();
        return result;
    }

    public boolean forceUndeploy(String serviceAssemblyName) {
        this.log.start();

        boolean removed = false;
        final ServiceAssemblyLifeCycle saLifeCycle = this.adminService.getServiceAssemblies().get(
                serviceAssemblyName);
        if (saLifeCycle != null) {
            saLifeCycle.setForcedState(true);
        } else {
            this.log.warning("Cannot retrieve Service Assembly with the specified name : "
                    + serviceAssemblyName);
        }
        try {
            if (LifeCycleMBean.STARTED.equals(saLifeCycle.getCurrentState())
                    || LifeCycleMBean.UNKNOWN.equals(saLifeCycle.getCurrentState())) {
                this.stop(serviceAssemblyName, true);
            }
        } catch (Exception e) {
            // do nothing
        }
        try {
            if (LifeCycleMBean.STOPPED.equals(saLifeCycle.getCurrentState())) {
                this.shutDown(serviceAssemblyName, true);
            }
        } catch (Exception e) {
            // do nothing
        }
        try {
            this.undeploy(serviceAssemblyName);
            removed = true;
        } catch (Exception e) {
            // do nothing
        }

        this.log.end();
        return removed;
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.management.DeploymentServiceMBean#
     * getComponentsForDeployedServiceAssembly(java.lang.String)
     */
    public String[] getComponentsForDeployedServiceAssembly(String serviceAssemblyName)
            throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        List<String> result = new ArrayList<String>();
        final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            List<ServiceUnit> sus = saLifeCycle.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                result.add(unit.getTarget().getComponentName());
            }
        } else {
            throw new Exception("Cannot retrieve service assembly with the specified name: "
                    + serviceAssemblyName);
        }

        this.log.end();
        return result.toArray(new String[result.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies
     * ()
     */
    public String[] getDeployedServiceAssemblies() throws Exception {
        this.log.call();

        Set<String> sas = this.adminService.getServiceAssemblies().keySet();
        return sas.toArray(new String[sas.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.management.DeploymentServiceMBean#
     * getDeployedServiceAssembliesForComponent(java.lang.String)
     */
    public String[] getDeployedServiceAssembliesForComponent(String componentName) throws Exception {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblies();
        for (final ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTarget().getComponentName())) {
                    result.add(salc.getServiceAssembly().getIdentification().getName());
                }
            }
        }

        this.log.end();
        return result.toArray(new String[result.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList
     * (java.lang.String)
     */
    public String[] getDeployedServiceUnitList(String componentName) throws Exception {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        Set<String> result = new HashSet<String>();
        final Map<String, ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
                .getServiceAssemblies();
        for (final ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTarget().getComponentName())) {
                    result.add(unit.getIdentification().getName());
                }
            }
        }

        this.log.end();
        return result.toArray(new String[result.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#getServiceAssemblyDescriptor
     * (java.lang.String)
     */
    public String getServiceAssemblyDescriptor(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String result = "Service Assembly doesn't exist :" + serviceAssemblyName;

        File serviceAssemblyInstallFile = this.repositoryService
                .getServiceAssemblyInstallDirectory(serviceAssemblyName);
        if (serviceAssemblyInstallFile != null) {
            File jbiFile = new File(serviceAssemblyInstallFile, JBI_DESCRIPTOR_RESOURCE);
            result = IOUtils.toString(new FileInputStream(jbiFile));
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.deployment.DeploymentServiceMBean#
     * getServiceUnitForServiceAssembly(java.lang.String)
     */
    public String[] getServiceUnitForServiceAssembly(String saName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(saName, "saName", this.log);

        List<String> result = new ArrayList<String>();
        final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(saName);

        if (saLifeCycle != null) {
            List<ServiceUnit> serviceUnits = saLifeCycle.getServiceAssembly().getServiceUnit();
            for (ServiceUnit serviceUnit : serviceUnits) {
                result.add(serviceUnit.getIdentification().getName());
            }

        } else {
            String msg = "Cannot retrieve service assembly with the name '" + saName + '"';
            this.log.error(msg);
            throw new Exception(msg);
        }

        this.log.end();
        return result.toArray(new String[result.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
     */
    public String getState(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String result = null;
        String msg;
        final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            result = saLifeCycle.getCurrentState();
        } else {
            msg = "Cannot retrieve service assembly with the specified name : "
                    + serviceAssemblyName;
            throw new Exception(msg);
        }
        if (!(LifeCycleMBean.SHUTDOWN.equals(result) || LifeCycleMBean.STARTED.equals(result)
                || LifeCycleMBean.STOPPED.equals(result) || LifeCycleMBean.UNKNOWN.equals(result))) {
            msg = "Service Assembly state isn't correctly defined";
            throw new Exception(msg);
        }

        this.log.end();
        return result;
    }

    public boolean isDeployedServiceUnit(String componentName, String serviceUnitName) {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceUnitName, "serviceUnitName",
                this.log);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        boolean result = false;
        List<ServiceUnitLifeCycle> suLifeCycles = this.adminService
                .getServiceUnitsLifeCyclesForComponent(componentName);
        for (ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            if (suLifeCycle.getSuName().equals(serviceUnitName)) {
                result = true;
                break;
            }
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.PetalsLifeCycle#shutdown()
     */
    public void shutdown() throws Exception {
        this.log.call();
        // Undeploy all service assemblies
        this.undeployAllServiceAssemblies(true);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
     */
    public String shutDown(String serviceAssemblyName) throws Exception {
        return this.shutDown(serviceAssemblyName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.deployment.DeploymentServiceMBean#
     * shutdownAllServiceAssemblies()
     */
    public String[] shutdownAllServiceAssemblies() {
        this.log.start();

        List<String> shutdownSAs = new ArrayList<String>();
        // Try to shutdown all service assemblies. If a SA can't be shutdown,
        // it's skipped and the shutting down process ins't aborted.
        List<ServiceAssembly> serviceAssemblies = this.systemStateService
                .getServiceAssemblyStateHolders();
        Collections.reverse(serviceAssemblies);
        for (ServiceAssembly serviceAssembly : serviceAssemblies) {
            try {
                this.shutDown(serviceAssembly.getName());
                shutdownSAs.add(serviceAssembly.getName());
            } catch (Throwable e) {
                // continue
            }
        }

        this.log.end();
        return shutdownSAs.toArray(new String[shutdownSAs.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     */
    public String start(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            XMLResult xmlResult = new XMLResult("start");
            final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.start();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed");
            }
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully started service assembly : {1}",
                    new String[] { serviceAssemblyName }, null);
            output = xmlResult.getString();
        } catch (Exception e) {
            this.log.error("Unable to start the service assembly '" + serviceAssemblyName + "'", e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new Exception(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' started");

        this.log.end();
        return output;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.deployment.DeploymentServiceMBean#
     * startAllServiceAssemblies()
     */
    public String[] startAllServiceAssemblies() {
        this.log.start();

        List<String> startedSAs = new ArrayList<String>();
        // Try to start all service assemblies, if a SA can't be started, it's
        // skipped and the starting process ins't aborted.
        for (ServiceAssembly serviceAssembly : this.systemStateService
                .getServiceAssemblyStateHolders()) {
            try {
                this.start(serviceAssembly.getName());
                startedSAs.add(serviceAssembly.getName());
            } catch (Throwable e) {
                // continue
            }
        }

        this.log.end();
        return startedSAs.toArray(new String[startedSAs.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
     */
    public String stop(String serviceAssemblyName) throws Exception {
        return this.stop(serviceAssemblyName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.deployment.DeploymentServiceMBean#
     * stopAllServiceAssemblies()
     */
    public String[] stopAllServiceAssemblies() {
        this.log.start();

        List<String> stoppedSAs = new ArrayList<String>();

        // Try to stop all service assemblies. If a SA can't be stopped, it's
        // skipped and the stopping process ins't aborted.
        List<ServiceAssembly> serviceAssemblies = this.systemStateService
                .getServiceAssemblyStateHolders();
        Collections.reverse(serviceAssemblies);
        for (ServiceAssembly serviceAssembly : serviceAssemblies) {
            try {
                this.stop(serviceAssembly.getName());
                stoppedSAs.add(serviceAssembly.getName());
            } catch (Throwable e) {
                // continue
            }
        }

        this.log.end();
        return stoppedSAs.toArray(new String[stoppedSAs.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
     */
    public String undeploy(String serviceAssemblyName) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName, "serviceAssemblyZipUrl",
                this.log);

        String output = null;
        try {
            // Check if the SA exists
            ServiceAssembly serviceAssemblyStateHolder = this.systemStateService
                    .getServiceAssemblyStateHolder(serviceAssemblyName);
            if (serviceAssemblyStateHolder == null) {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed");
            } else if (!SHUTDOWN.equals(serviceAssemblyStateHolder.getLifecycleState())) {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not at the state SHUTDOWN");
            }

            // Create undeployment execution context
            Context undeploymentContext = this.createUndeploymentContext(serviceAssemblyName);
            // Create undeployment processor and fill with tasks
            TaskProcessor processor = this.createUndeploymentProcessor(undeploymentContext);
            // Launch the process
            final Throwable processResult = processor.process();
            if (processResult != null) {
                throw new DeploymentException("The undeployment of the Service Assembly '"
                        + serviceAssemblyName + "' failed.", processResult);
            }
            // Create the result string
            output = undeploymentContext.getXmlResult().getString();
        } catch (Exception e) {
            this.log.error("Unable to undeploy the service assembly '" + serviceAssemblyName + "'",
                    e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new Exception(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' undeployed");

        this.log.end();
        return output;
    }

    public String[] undeployAllServiceAssemblies(boolean force) {
        this.log.start();

        List<String> undeployedSAs = new ArrayList<String>();
        /*
         * Try to undeploy all service assemblies. If a SA can't be undeployed,
         * it's skipped and the undeploying process ins't aborted.
         */
        List<ServiceAssembly> serviceAssemblies = this.systemStateService
                .getServiceAssemblyStateHolders();
        Collections.reverse(serviceAssemblies);
        for (ServiceAssembly serviceAssembly : serviceAssemblies) {
            if (force) {
                if (this.forceUndeploy(serviceAssembly.getName())) {
                    undeployedSAs.add(serviceAssembly.getName());
                }
            } else if (SHUTDOWN.equals(serviceAssembly.getLifecycleState())) {
                try {
                    this.undeploy(serviceAssembly.getName());
                    undeployedSAs.add(serviceAssembly.getName());
                } catch (Throwable e) {
                    // continue
                }
            }
        }

        this.log.end();
        return undeployedSAs.toArray(new String[undeployedSAs.size()]);
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(DeploymentService.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.deploymentTaskFactory = new DeploymentTaskFactory(this.endpointRegistry, this,
                this.systemStateService, this.repositoryService, this.log, this.adminService,
                this.container, this.configurationService.getContainerConfiguration(),
                this.installationExtensionManager.getExtensions());

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Create the deployment context
     * 
     * @param archiveURI
     *            URI of the Service Assembly installation package file; must be
     *            non-null, non-empty, and a legal URI.
     * @return the Deployment Context HashMap
     */
    private Context createDeploymentContext(URL archiveURL) {
        this.log.call();

        Context deploymentContext = new Context();
        deploymentContext.setArchiveUrl(archiveURL);
        return deploymentContext;
    }

    /**
     * Create the deployment processor and add all required task for the
     * complete deployment of a service assembly
     * 
     * @param deploymentContext
     *            the deployment context
     * @param archiveURI
     *            the sl archive URI
     * @return the filled deployment processor
     */
    private TaskProcessor createDeploymentProcessor(Context deploymentContext, URL archiveURL) {
        this.log.call();
        TaskProcessor processor = new TaskProcessor(deploymentContext, this.log);
        String scheme = archiveURL.getProtocol();
        if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
            processor.addTask(this.deploymentTaskFactory.getDownloadTask());
        } else if (scheme.equalsIgnoreCase("file")) {
            processor.addTask(this.deploymentTaskFactory.getCopyArchiveToWorkDirTask());
        } else {
            throw new IllegalArgumentException("URL protocol not supported:" + scheme);
        }
        processor.addTask(this.deploymentTaskFactory.getExtractTask());
        processor.addTask(this.deploymentTaskFactory.getSaCheckPackageTask());
        processor.addTask(this.deploymentTaskFactory.getCopySAPackageToRepositoryTask());
        processor.addTask(this.deploymentTaskFactory.getCreateSAStateHolderTask());
        processor.addTask(this.deploymentTaskFactory.getCreateXMLDeploymentResultTask());
        processor.addTask(this.deploymentTaskFactory.getExtractSUsTask());
        processor.addTask(this.deploymentTaskFactory.getRegisterSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getDeployAllSUTask());
        processor.addTask(this.deploymentTaskFactory.getRegisterAllConnectionTask());
        processor.addTask(this.deploymentTaskFactory.getCopyArchiveToInstalledDirTask());

        return processor;
    }

    /**
     * Create the undeployment context
     * 
     * @param serviceAssemblyName
     *            the service assembly name
     * @return the undeployment context
     * @throws PetalsException
     *             if error occured during JBI descriptor loading
     */
    private Context createUndeploymentContext(String serviceAssemblyName) {
        this.log.call();

        Context undeploymentContext = new Context();
        ServiceAssembly serviceAssemblyStateHolder = this.systemStateService
                .getServiceAssemblyStateHolder(serviceAssemblyName);
        undeploymentContext.setEntityName(serviceAssemblyName);
        try {
            undeploymentContext.setArchiveUrl(new URL(serviceAssemblyStateHolder.getInstallURL()));
        } catch (MalformedURLException e) {
            // not possible
        }

        return undeploymentContext;
    }

    /**
     * Create the undeployment processor and add all required task for the
     * complete undeployment of a service assembly
     * 
     * @param undeploymentContext
     *            the deployment context
     * @return filled undeployment processor
     */
    private TaskProcessor createUndeploymentProcessor(Context undeploymentContext) {
        this.log.call();
        TaskProcessor processor = new TaskProcessor(undeploymentContext, this.log);
        processor.addTask(this.deploymentTaskFactory.getCreateXMLUndeploymentResultTask());
        processor.addTask(this.deploymentTaskFactory.getRetrieveSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getUndeployAllSUTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveAllConnectionsTask());
        processor.addTask(this.deploymentTaskFactory.getUnregisterSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveSAStateHolderTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveSAFolderTask());
        processor.addTask(this.deploymentTaskFactory.getRemoveSAArchiveTask());

        return processor;
    }

    /**
     * Shut down the given service assembly. if {@code forceState=true}, the SA
     * would be in the {@code SHUTDOWN} state even errors occur in starting of
     * service units, otherwise the SA get back to its previous state.
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     * 
     * @param serviceAssemblyName
     * @param forcedState
     * @return
     * @throws Exception
     */
    private String shutDown(String serviceAssemblyName, boolean forcedState) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            XMLResult xmlResult = new XMLResult("shutdown");

            final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.setForcedState(forcedState);
                saLifeCycle.shutDown();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed");
            }
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully shutdown service assembly : {1}",
                    new String[] { serviceAssemblyName }, null);
            output = xmlResult.getString();
        } catch (Exception e) {
            this.log.error("Unable to shutdown the service assembly '" + serviceAssemblyName + "'",
                    e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new Exception(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' shut down");

        this.log.end();
        return output;
    }

    private String stop(String serviceAssemblyName, boolean force) throws Exception {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(serviceAssemblyName,
                "serviceAssemblyName", this.log);

        String output = null;
        try {
            final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                    .getServiceAssemblyByName(serviceAssemblyName);
            if (saLifeCycle != null) {
                saLifeCycle.setForcedState(force);
                saLifeCycle.stop();
            } else {
                throw new DeploymentException("The service assembly '" + serviceAssemblyName
                        + "' is not deployed");
            }
            XMLResult xmlResult = new XMLResult("stop");
            xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                    "Successfully stopped service assembly : {1}",
                    new String[] { serviceAssemblyName }, null);
            output = xmlResult.getString();

        } catch (Exception e) {
            this.log.error("Unable to stop the service assembly '" + serviceAssemblyName + "'", e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new Exception(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.info("Service Assembly '" + serviceAssemblyName + "' stopped");

        this.log.end();
        return output;
    }

}
