/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.installation;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.management.ObjectName;
import javax.naming.InitialContext;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.commons.exception.ExceptionUtil;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.DeploymentService;
import org.ow2.petals.microkernel.api.jbi.management.InstallationService;
import org.ow2.petals.microkernel.api.jbi.management.TaskProcessor;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.container.lifecycle.ComponentLifeCycleImpl;
import org.ow2.petals.microkernel.extension.InstallationExtensionsManager;
import org.ow2.petals.microkernel.jbi.management.MBeanNameBuilder;
import org.ow2.petals.microkernel.jbi.management.task.installation.InstallationTaskFactory;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.systemstate.generated.SharedLibrary;

/**
 * JBI installation service implementation.
 * @author Rafael Marins - Fossil E-Commerce
 * @author Olivier Fabre - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = InstallationService.class))
public class InstallationServiceImpl implements InstallationService {

    /**
     * Communication :: JMX Agent Admin.
     */
    @Requires(name = "admin")
    protected AdminService adminService;

    protected InitialContext componentInitialContext;

    /**
     * Container :: Container Service.
     */
    @Requires(name = "container")
    protected ContainerService container;

    protected InstallationTaskFactory installationTaskFactory;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Platform :: Repository Service.
     */
    @Requires(name = "repository")
    protected RepositoryService repositoryService;

    /**
     * JBI messaging :: Router Service.
     */
    @Requires(name = "router")
    protected RouterService router;

    /**
     * JBI management :: Deployment Service.
     */
    @Requires(name = "deployment")
    protected DeploymentService deploymentService;

    /**
     * Platform :: SystemState Service.
     */
    @Requires(name = "systemstate")
    protected SystemStateService systemStateService;

    /**
     * Configuration Service
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    @Requires(name = "installationExtensionManager")
    private InstallationExtensionsManager installationExtensionManager;

    @Requires(name = "mbean-name-builder")
    private MBeanNameBuilder mbeanNameBuilder;

    /**
     * Default constructor.
     */
    public InstallationServiceImpl() {

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * forceUnloadComponentInstaller(java.lang.String)
     */
    public boolean forceUnloadInstaller(String componentName) {
        this.log.start("Component Name : " + componentName);

        boolean removed = false;
        try {
            this.forceShutdownComponent(componentName);
        } catch (final JBIException e) {
            // do nothing
        }
        try {
            this.uninstallComponent(componentName);
        } catch (final JBIException e) {
            // do nothing
        }
        removed = this.unloadInstaller(componentName, true);

        this.log.end();
        return removed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * getInstalledComponentsForSharedlibrary(java.lang.String)
     */
    public String[] getInstalledComponentsForSharedLibrary(String slName) throws Exception {
        this.log.call();

        String[] componentNames = new String[0];
        final SharedLibraryLifeCycle sharedLibraryLifeCycle = this.adminService
                .getSharedLibraryByName(slName);
        if (sharedLibraryLifeCycle == null) {
            throw new Exception("Shared Library '" + slName + "' is not installed");
        } else {
            componentNames = sharedLibraryLifeCycle.getRegisteredComponents().toArray(
                    componentNames);
        }
        return componentNames;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.service.InstallationServiceMBean#
     * getinstalledSharedLibrary()
     */
    public String[] getInstalledSharedLibraries() {
        this.log.call();

        final Set<String> sls = this.adminService.getSharedLibraries().keySet();
        return sls.toArray(new String[sls.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * getInstallers()
     */
    public String[] getInstallers() {
        this.log.call();
        
        final List<String> result = new ArrayList<String>();
        final  Installer[] installers = this.adminService.getInstallers();

        for (Installer installer : installers) {
            result.add(installer.getComponentName());
        }

        return result.toArray(new String[result.size()]);
    }

    public Logger getLogger() {
        return this.log.getLogger();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.InstallationServiceMBean#installSharedLibrary(java
     * .lang.String)
     */
    public String installSharedLibrary(String slZipURL) {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(slZipURL, "slZipURL", this.log);

        Context installationContext = null;
        try {
            URL archiveURL;
            try {
                archiveURL = new URL(slZipURL);
            } catch (final MalformedURLException e) {
                archiveURL = new File(slZipURL).toURI().toURL();
            }

            // Create installation execution context
            installationContext = this.createInstallationContext(archiveURL);

            // Create installation processor and fill it with tasks
            final TaskProcessor processor = this.createSLInstallationProcessor(installationContext,
                    archiveURL);

            // Launch the process
            final Throwable processResult = processor.process();
            if (processResult != null) {
                throw new Exception("The installation of the Shared Library '" + slZipURL
                        + "' failed.", processResult);
            }
        } catch (Exception e) {
            this.log
                    .error("Unable to install the shared library at location '" + slZipURL + "'", e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new RuntimeException(ExceptionUtil.getExtendedMessage(e));
        }
        this.log.info("Shared Library '" + installationContext.getEntityName() + "' with version '"
                + installationContext.getEntityVersion() + "' installed");

        this.log.end();
        return installationContext.getEntityName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.InstallationServiceMBean#loadInstaller(java.lang
     * .String)
     */
    public ObjectName loadInstaller(String componentName) {
        this.log.start();

        try {

            ParameterCheckHelper.isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

            // Call init on bootstrap
            final Installer installer = this.adminService.getInstallerByName(componentName);
            if (installer != null) {
                try {
                    installer.initBootstrap(false);
                } catch (final Exception e) {
                    final StringBuilder msg = new StringBuilder("Failed to initialize Component Bootstrap");
                    this.log.error(msg.toString(), e);
                    msg.append("\nCaused by: ").append(e.getMessage());
                    Throwable ex = e.getCause();
                    while (ex != null) {
                        if (ex.getMessage() != null) {
                            msg.append("\nCaused by: " + ex.getMessage());
                        }
                        ex = ex.getCause();
                    }
                    throw new RuntimeException(msg.toString());
                }

                this.log.info("Installer of the component '" + componentName + "' loaded");

                return this.mbeanNameBuilder.createInstallerMBeanName(componentName);
            }
            else {
                return null;
            }
        }
        finally {
            this.log.end();
        }
    }

    /**
     * @see javax.jbi.management.InstallationServiceMBean#loadNewInstaller(java.lang
     *      .String) Return a specific ObjectName if component can't be
     *      installed correctly. It's an implementation choice !
     * @throws RuntimeException
     *             if an error occurs during installation process. We have
     *             chosen this type of exception because, in the specification,
     *             this method doesn't return an exception
     */
    public ObjectName loadNewInstaller(String installZipURL) {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(installZipURL, "installZipURL", this.log);

        Context installationContext = null;
        try {
            // Transform URL to URI
            URL archiveURL;
            try {
                archiveURL = new URL(installZipURL);
            } catch (final MalformedURLException e) {
                archiveURL = new File(installZipURL).toURI().toURL();
            }
            // Create installation execution context
            installationContext = this.createInstallationContext(archiveURL);
            // Create installation processor and fill it with tasks
            final TaskProcessor processor = this.createComponentInstallationProcessor(
                    installationContext, archiveURL);
            // Launch the installation processor
            final Throwable processResult = processor.process();
            if (processResult != null) {
                throw new Exception(processResult);
            }
        } catch (Exception e) {
            this.log.error("Unable to load a new installer at location '" + installZipURL + "'", e);
            // Redefine the exception to prevent exotic exception cause type,
            // and then unserialization problem for external class-loaders (JMX
            // client)
            throw new RuntimeException(ExceptionUtil.getExtendedMessage(e));
        }

        this.log.info("A new installer for the component '" + installationContext.getEntityName()
                + "' is loaded");

        this.log.end();
        return installationContext.getInstallerObjectName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.PetalsLifeCycle#shutdown()
     */
    public void shutdown() throws Exception {
        this.log.call();

        this.unloadAllInstallers(true);
        this.uninstallAllSharedLibraries();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * shutdownAllComponents()
     */
    public String[] shutdownAllComponents() {
        this.log.call();

        final List<String> shutdownComponents = new ArrayList<String>();
        for (final Component componentStateHolder : this.systemStateService
                .getComponentStateHolders()) {
            try {
                this.shutdownComponent(componentStateHolder.getName());
                shutdownComponents.add(componentStateHolder.getName());
            } catch (final JBIException e) {
                // continue
            }
        }
        return shutdownComponents.toArray(new String[shutdownComponents.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public String[] shutdownAllNoMoreUsedComponents() {
        this.log.call();
        
        final List<String> shutdownComponents = new ArrayList<String>();
        final  Installer[] installers = this.adminService.getInstallers();
        if (installers != null && installers.length > 0) {
            for (final Installer installer : installers) {
                final String componentName = installer.getComponentName();
                try {
                    final String[] deployedServiceAssemblies = this.deploymentService
                            .getDeployedServiceAssembliesForComponent(componentName);
                    if (deployedServiceAssemblies != null && deployedServiceAssemblies.length == 0) {
                        this.shutdownComponent(componentName);
                        shutdownComponents.add(componentName);
                    }
                } catch (final Exception e) {
                    // continue
                }
            }
        }

        return shutdownComponents.toArray(new String[shutdownComponents.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * shutdownComponent(java.lang.String)
     */
    public boolean shutdownComponent(String componentName) throws JBIException {
        return this.shutdownComponent(componentName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * startAllComponents()
     */
    public String[] startAllComponents() {
        this.log.call();

        final List<String> startedComponents = new ArrayList<String>();
        for (final Component componentStateHolder : this.systemStateService
                .getComponentStateHolders()) {
            try {
                this.startComponent(componentStateHolder.getName());
                startedComponents.add(componentStateHolder.getName());
            } catch (final JBIException e) {
                // continue
            }
        }

        return startedComponents.toArray(new String[startedComponents.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * startComponent(java.lang.String)
     */
    public boolean startComponent(final String componentName) throws JBIException {
        this.log.start("Component Name : " + componentName);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        ComponentLifeCycleMBean componentLifeCycle = null;
        if (this.adminService.isEngine(componentName)) {
            componentLifeCycle = this.adminService.getEngineComponentLifeCycleByName(componentName);
        } else {
            componentLifeCycle = this.adminService
                    .getBindingComponentLifeCycleByName(componentName);
        }

        try {
            if (componentLifeCycle != null) {
                componentLifeCycle.start();
            } else {
                throw new JBIException("The component '" + componentName + "' is not installed");
            }
        } catch (final JBIException e) {
            this.log.error("Unable to start the component '" + componentName + "'", e);
            throw e;
        }

        this.log.info("Component '" + componentName + "' started");

        this.log.end();
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * stopAllComponents()
     */
    public String[] stopAllComponents() {
        this.log.call();

        final List<String> stoppedComponents = new ArrayList<String>();
        for (final Component componentStateHolder : this.systemStateService
                .getComponentStateHolders()) {
            try {
                this.stopComponent(componentStateHolder.getName());
                stoppedComponents.add(componentStateHolder.getName());
            } catch (final JBIException e) {
                // continue
            }
        }
        return stoppedComponents.toArray(new String[stoppedComponents.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * stopComponent(java.lang.String)
     */
    public boolean stopComponent(String componentName) throws JBIException {
        return this.stopComponent(componentName, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * uninstallAllComponents()
     */
    public String[] uninstallAllComponents() {
        this.log.call();

        final List<String> uninstalledComponents = new ArrayList<String>();
        for (final Component componentStateHolder : this.systemStateService
                .getComponentStateHolders()) {
            try {
                this.uninstallComponent(componentStateHolder.getName());
                uninstalledComponents.add(componentStateHolder.getName());
            } catch (final JBIException e) {
                // continue
            }
        }
        return uninstalledComponents.toArray(new String[uninstalledComponents.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public String[] uninstallAllNoMoreUsedComponents() {
        this.log.call();

        final List<String> uninstalledComponents = new ArrayList<String>();
        final Installer[] installers = this.adminService.getInstallers();
        if (installers != null && installers.length > 0) {
            for (final Installer installer : installers) {
                final String componentName = installer.getComponentName();
                try {
                    final String[] deployedServiceAssemblies = this.deploymentService
                            .getDeployedServiceAssembliesForComponent(componentName);
                    if (deployedServiceAssemblies != null && deployedServiceAssemblies.length == 0) {
                        this.uninstallComponent(componentName);
                        uninstalledComponents.add(componentName);
                    }
                } catch (final Exception e) {
                    // continue
                }
            }
        }

        return uninstalledComponents.toArray(new String[uninstalledComponents.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * uninstallAllSharedLibraries()
     */
    public String[] uninstallAllSharedLibraries() {
        this.log.call();

        final List<String> removedSharedLibraries = new ArrayList<String>();
        for (final SharedLibrary sharedLibraryStateHolder : this.systemStateService
                .getSharedLibraryStateHolders()) {
            if (this.uninstallSharedLibrary(sharedLibraryStateHolder.getName())) {
                removedSharedLibraries.add(sharedLibraryStateHolder.getName());
            }
        }
        return removedSharedLibraries.toArray(new String[removedSharedLibraries.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public String[] uninstallAllNoMoreUsedSharedLibraries() {
        this.log.call();

        final List<String> removedSharedLibraries = new ArrayList<String>();

        final Map<String, SharedLibraryLifeCycle> sharedLibraries = this.adminService
                .getSharedLibraries();
        if (sharedLibraries != null) {
            for (final Map.Entry<String, SharedLibraryLifeCycle> entry : sharedLibraries.entrySet()) {
                final SharedLibraryLifeCycle sharedLibrary = entry.getValue();
                final List<String> registeredComponents = sharedLibrary.getRegisteredComponents();
                if (registeredComponents != null && registeredComponents.isEmpty()) {
                    final String slName = entry.getKey();
                    if (this.uninstallSharedLibrary(slName)) {
                        removedSharedLibraries.add(slName);
                    }
                }
            }
        }

        return removedSharedLibraries.toArray(new String[removedSharedLibraries.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * uninstallComponent(java.lang.String)
     */
    public boolean uninstallComponent(String componentName) throws JBIException {
        this.log.start("Component Name : " + componentName);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        if (!this.systemStateService.isComponentInstalled(componentName)) {
            throw new JBIException("The component '" + componentName + "' is not installed");
        }

        try {
            this.loadInstaller(componentName);
        } catch (final RuntimeException e) {
            throw new JBIException(e.getMessage());
        }

        final Installer installer = this.adminService.getInstallerByName(componentName);
        try {
            if (installer != null) {
                installer.uninstall();
            } else {
                throw new JBIException("The installer of the component '" + componentName
                        + "' is not loaded");
            }
        } catch (final JBIException e) {
            this.log.error("Unable to uninstall the component '" + componentName + "'", e);
            throw e;
        }

        this.log.info("Component '" + componentName + "' uninstalled");

        this.log.end();
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.InstallationServiceMBean#uninstallSharedLibrary(
     * java.lang.String)
     */
    public boolean uninstallSharedLibrary(String slName) {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(slName, "slName", this.log);

        boolean result = true;

        final List<SharedLibrary> sharedLibraryStateHolders = this.systemStateService
                .getSharedLibraryStateHolders(slName);

        if (sharedLibraryStateHolders.size() == 0) {
            // maybe it is the name-version that is passed as slName
            // this is due to the trick to handle SL version that is not
            // supported by JBI APIs
            for (final SharedLibrary sharedLibraryStateHolder : this.systemStateService
                    .getSharedLibraryStateHolders()) {
                if (slName.startsWith(sharedLibraryStateHolder.getName())
                        && slName.length() > sharedLibraryStateHolder.getName().length() + 1) {
                    final String version = slName.substring(sharedLibraryStateHolder.getName()
                            .length() + 1);
                    if (sharedLibraryStateHolder.getVersion().equals(version)) {
                        sharedLibraryStateHolders.add(sharedLibraryStateHolder);
                        break;
                    }
                }
            }
        }
        if (sharedLibraryStateHolders.size() == 0) {
            throw new RuntimeException("No shared library with identification '" + slName
                    + "' is installed");
        }

        for (final SharedLibrary sharedLibraryStateHolder : sharedLibraryStateHolders) {
            if (this.adminService.getSharedLibraryByName(
                    sharedLibraryStateHolder.getName() + "-"
                            + sharedLibraryStateHolder.getVersion()).getRegisteredComponents()
                    .size() != 0) {
                throw new RuntimeException("At least a component refers the shared library '"
                        + sharedLibraryStateHolder.getName() + "' with version '"
                        + sharedLibraryStateHolder.getVersion() + "'");
            }

            // Create uninstallation execution context
            final Context uninstallationContext = this.createSLUnInstallationContext(
                    sharedLibraryStateHolder.getName(), sharedLibraryStateHolder.getVersion());
            // Create uninstallation processor and fill with tasks
            final TaskProcessor processor = this
                    .createSLUnInstallationProcessor(uninstallationContext);
            // Launch the process

            final Throwable processResult = processor.process();
            if (processResult != null) {
                result = false;
                this.log.warning("The uninstallation of the shared library '"
                        + sharedLibraryStateHolder.getName() + "' failed", processResult);
            } else {
                this.log.info("Shared library '" + slName + "' with version '"
                        + sharedLibraryStateHolder.getVersion() + "' uninstalled");
            }
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * uninstallSharedLibrary(java.lang.String, java.lang.String)
     */
    public boolean uninstallSharedLibrary(String slName, String version) {
        this.log.start();

        ParameterCheckHelper.isNullOrEmptyParameterWithLog(slName, "slName", this.log);
        ParameterCheckHelper.isNullOrEmptyParameterWithLog(slName, "version", this.log);

        boolean result = true;

        final SharedLibrary sharedLibraryStateHolder = this.systemStateService
                .getSharedLibraryStateHolder(slName, version);
        if (sharedLibraryStateHolder == null) {
            throw new RuntimeException("No shared library with identification '" + slName
                    + "' and version " + version + " is installed");
        }

        if (this.adminService.getSharedLibraryByName(slName + "-" + version)
                .getRegisteredComponents().size() != 0) {
            throw new RuntimeException("At least a component refers the shared library '"
                    + sharedLibraryStateHolder.getName() + "'");
        }

        // Create uninstallation execution context
        final Context uninstallationContext = this.createSLUnInstallationContext(
                sharedLibraryStateHolder.getName(), sharedLibraryStateHolder.getVersion());
        // Create uninstallation processor and fill with tasks
        final TaskProcessor processor = this.createSLUnInstallationProcessor(uninstallationContext);
        // Launch the process

        final Throwable processResult = processor.process();
        if (processResult != null) {
            result = false;
            this.log.error("Unable to uninstall the shared library '"
                    + sharedLibraryStateHolder.getName() + "'", processResult);
        } else {
            this.log.info("Shared library '" + slName + "' with version '"
                    + sharedLibraryStateHolder.getVersion() + "' uninstalled");
        }

        this.log.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.installation.InstallationServiceMBean#
     * unloadAllInstallers(boolean)
     */
    public String[] unloadAllInstallers(boolean force) {
        this.log.call();
        final List<String> removedComponents = new ArrayList<String>();
        final List<Component> componentStateHolders = this.systemStateService
                .getComponentStateHolders();
        Collections.reverse(componentStateHolders);
        for (final Component componentStateHolder : componentStateHolders) {
            if (force) {
                if (this.forceUnloadInstaller(componentStateHolder.getName())) {
                    removedComponents.add(componentStateHolder.getName());
                }
            } else {
                if (this.unloadInstaller(componentStateHolder.getName(), true)) {
                    removedComponents.add(componentStateHolder.getName());
                }
            }
        }

        return removedComponents.toArray(new String[removedComponents.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.management.InstallationServiceMBean#unloadInstaller(java.lang
     * .String, boolean)
     */
    public boolean unloadInstaller(String componentName, boolean isToBeDeleted) {
        this.log.start();

        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        boolean result = true;
        try {
            if (this.systemStateService.getComponentStateHolder(componentName) == null) {
                throw new RuntimeException("The installer of the component '" + componentName
                        + "' is not existing");
            }

            // Create uninstallation execution context
            final Context uninstallationContext = this
                    .createComponentUninstallationContext(componentName);

            // Create uninstallation processor and fill it with tasks
            final TaskProcessor processor = this
                    .createComponentUnInstallationProcessor(uninstallationContext);

            // Launch the process
            final Throwable processResult = processor.process();
            if (processResult != null) {
                throw new Exception(processResult);
            }
            this.log
                    .info("Installer of the component '" + componentName + "' succesfully unloaded");
        } catch (final Exception e) {
            this.log.error("Unable to unload the installer of the component '" + componentName
                    + "'", e);
            result = false;
        }

        this.log.end();
        return result;
    }

    /**
     * Initialize the InstallationServiceImpl
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(Logger.getLogger(InstallationService.COMPONENT_LOGGER_NAME));
        this.log.start();

        final ContainerConfiguration containerConfiguration = this.configurationService
                .getContainerConfiguration();

        // Initialize installation task factory
        this.installationTaskFactory = new InstallationTaskFactory(this.systemStateService,
                this.repositoryService, this.log, this.adminService, this.router, this.container,
                containerConfiguration, this.installationExtensionManager.getExtensions());

        // Clean the success file in the work directory
        final File workFile = new File(containerConfiguration.getWorkDirectoryPath());
        final File[] filesOfWorkDir = workFile.listFiles();
        if (filesOfWorkDir != null) {
            for (final File file : filesOfWorkDir) {
                if (!file.isDirectory() && file.getName().endsWith(".success")) {
                    file.delete();
                }
            }
        }

        this.log.end();
    }

    /**
     * Stop the InstallationServiceImpl
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Create the installation processor and add all required task for the
     * complete installation of a component
     * 
     * @param installationContext
     *            the installation context
     * @param archiveURI
     *            the component archive URI
     * @return the filled installation processor
     */
    private TaskProcessor createComponentInstallationProcessor(Context installationContext,
            URL archiveURL) {
        final TaskProcessor processor = new TaskProcessor(installationContext, this.log);
        final String protocol = archiveURL.getProtocol();
        if (protocol.equalsIgnoreCase("http") || protocol.equalsIgnoreCase("ftp")) {
            processor.addTask(this.installationTaskFactory.getDownloadTask());
        } else if (protocol.equalsIgnoreCase("file")) {
            processor.addTask(this.installationTaskFactory.getCopyArchiveToWorkDirTask());
        } else {
            throw new IllegalArgumentException("URL protocol not supported:" + protocol);
        }
        processor.addTask(this.installationTaskFactory.getExtractTask());
        processor.addTask(this.installationTaskFactory.getComponentCheckPackageTask());
        processor.addTask(this.installationTaskFactory.getRegisterComponentInSLTask());
        processor.addTask(this.installationTaskFactory.getCreateComponentFolderTask());
        processor.addTask(this.installationTaskFactory.getCreateComponentStateHolderTask());
        processor.addTask(this.installationTaskFactory.getCreateAndRegisterInstallerMBeanTask());
        processor.addTask(this.installationTaskFactory.getCopyArchiveToInstalledDirTask());
        return processor;
    }

    /**
     * Create the component uninstallation context
     * 
     * @param name
     *            name of the component to uninstall
     * @return the Uninstallation Context HashMap
     */
    private Context createComponentUninstallationContext(String name) {
        final Context uninstallationContext = new Context();
        uninstallationContext.setEntityName(name);
        return uninstallationContext;
    }

    /**
     * Create the uninstallation processor and add all required task for the
     * complete uninstallation of a component
     * 
     * @param uninstallationContext
     *            the uninstallation context
     * @return the filled uninstallation processor
     */
    private TaskProcessor createComponentUnInstallationProcessor(Context uninstallationContext) {
        final TaskProcessor processor = new TaskProcessor(uninstallationContext, this.log);
        processor.addTask(this.installationTaskFactory.getComponentCheckInstalledTask());
        processor.addTask(this.installationTaskFactory.getUnregisterInstallerTask());
        processor.addTask(this.installationTaskFactory.getUnregisterComponentFromSLTask());
        processor.addTask(this.installationTaskFactory.getRemoveComponentStateHolderTask());
        processor.addTask(this.installationTaskFactory.getRemoveComponentFolderTask());
        processor.addTask(this.installationTaskFactory.getRemoveComponentArchiveTask());
        return processor;
    }

    /**
     * Create the installation context
     * 
     * @param zipUrl
     *            String containing the location URL of the Component or Shared
     *            lib ZIP file
     * @return the Installation Context HashMap
     */
    private Context createInstallationContext(URL archiveUrl) {
        final Context installationContext = new Context();
        installationContext.setArchiveUrl(archiveUrl);
        return installationContext;
    }

    /**
     * Create the installation processor and add all required task for the
     * complete installation of a shared lib
     * 
     * @param installationContext
     *            the installation context
     * @param archiveURI
     *            the sl archive URI
     * @return the filled installation processor
     */
    private TaskProcessor createSLInstallationProcessor(Context installationContext, URL archiveURL) {
        final TaskProcessor processor = new TaskProcessor(installationContext, this.log);
        final String scheme = archiveURL.getProtocol();
        if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
            processor.addTask(this.installationTaskFactory.getDownloadTask());
        } else if (scheme.equalsIgnoreCase("file")) {
            processor.addTask(this.installationTaskFactory.getCopyArchiveToWorkDirTask());
        } else {
            throw new IllegalArgumentException("URL protocol not supported:" + scheme);
        }
        processor.addTask(this.installationTaskFactory.getExtractTask());
        processor.addTask(this.installationTaskFactory.getSLCheckPackageTask());
        processor.addTask(this.installationTaskFactory.getCreateSLRepositoryTask());
        processor.addTask(this.installationTaskFactory.getCreateSLStateHolderTask());
        processor.addTask(this.installationTaskFactory.getCreateSLClassloaderTask());
        processor.addTask(this.installationTaskFactory.getCopyArchiveToInstalledDirTask());
        return processor;
    }

    /**
     * Create the shared lib uninstallation context
     * 
     * @param name
     *            name of the shared lib to uninstall
     * @return the Uninstallation Context HashMap
     */
    private Context createSLUnInstallationContext(String name, String version) {
        final Context uninstallationContext = new Context();
        uninstallationContext.setEntityName(name);
        uninstallationContext.setEntityVersion(version);
        return uninstallationContext;
    }

    /**
     * Create the uninstallation processor and add all required task for the
     * complete uninstallation of a shared lib
     * 
     * @param uninstallationContext
     *            the uninstallation context
     * @return the filled uninstallation processor
     */
    private TaskProcessor createSLUnInstallationProcessor(Context uninstallationContext) {
        final TaskProcessor processor = new TaskProcessor(uninstallationContext, this.log);
        processor.addTask(this.installationTaskFactory.getRemoveSLClassloaderTask());
        processor.addTask(this.installationTaskFactory.getRemoveSLStateHolderTask());
        processor.addTask(this.installationTaskFactory.getRemoveSLFolderTask());
        processor.addTask(this.installationTaskFactory.getRemoveSLArchiveTask());
        return processor;
    }

    private void forceShutdownComponent(String componentName) throws JBIException {
        this.log.call();

        ComponentLifeCycleMBean componentLifeCycle = null;
        if (this.adminService.isEngine(componentName)) {
            componentLifeCycle = this.adminService.getEngineComponentLifeCycleByName(componentName);
        } else {
            componentLifeCycle = this.adminService
                    .getBindingComponentLifeCycleByName(componentName);
        }

        if (componentLifeCycle != null) {
            final String compState = componentLifeCycle.getCurrentState();
            if (javax.jbi.management.LifeCycleMBean.STARTED.equals(compState)) {
                this.stopComponent(componentName, true);
                this.shutdownComponent(componentName, true);
            } else if (javax.jbi.management.LifeCycleMBean.STOPPED.equals(compState)) {
                this.shutdownComponent(componentName, true);
            }
        }
    }

    /**
     * Shutdown the component
     * 
     * @param componentName
     * @param force
     * @return
     * @throws JBIException
     */
    private boolean shutdownComponent(final String componentName, boolean force)
            throws JBIException {
        this.log.start("Component Name : " + componentName);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        ComponentLifeCycleMBean componentLifeCycle = null;
        if (this.adminService.isEngine(componentName)) {
            componentLifeCycle = this.adminService.getEngineComponentLifeCycleByName(componentName);
        } else {
            componentLifeCycle = this.adminService
                    .getBindingComponentLifeCycleByName(componentName);
        }

        try {
            if (componentLifeCycle != null) {
                // TODO : dirty, find a better way to do that
                ((ComponentLifeCycleImpl) componentLifeCycle).setForcedState(force);
                try {
                    componentLifeCycle.shutDown();
                } finally {
                    ((ComponentLifeCycleImpl) componentLifeCycle).setForcedState(false);
                }
            } else {
                throw new JBIException("The component '" + componentName + "' is not installed");
            }
        } catch (final JBIException e) {
            this.log.error("Unable to shutdown the component '" + componentName + "'", e);
            throw e;
        }

        this.log.info("Component '" + componentName + "' shut down");

        this.log.end();
        return true;
    }

    private boolean stopComponent(final String componentName, boolean force) throws JBIException {
        this.log.start("Component Name : " + componentName);
        ParameterCheckHelper
                .isNullOrEmptyParameterWithLog(componentName, "componentName", this.log);

        ComponentLifeCycleMBean componentLifeCycle = null;
        if (this.adminService.isEngine(componentName)) {
            componentLifeCycle = this.adminService.getEngineComponentLifeCycleByName(componentName);
        } else {
            componentLifeCycle = this.adminService
                    .getBindingComponentLifeCycleByName(componentName);
        }

        try {
            if (componentLifeCycle != null) {
                // TODO : dirty, find a better way to do that
                ((ComponentLifeCycleImpl) componentLifeCycle).setForcedState(force);
                try {
                    componentLifeCycle.stop();
                } finally {
                    ((ComponentLifeCycleImpl) componentLifeCycle).setForcedState(false);
                }
            } else {
                throw new JBIException("The component '" + componentName + "' is not installed");
            }
        } catch (final JBIException e) {
            this.log.error("Unable to stop the component '" + componentName + "'", e);
            throw e;
        }

        this.log.info("Component '" + componentName + "' stopped");

        this.log.end();
        return true;
    }

}
