/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.recovery;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.JbiArchiveType;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.jbi.management.PackageHelper;
import org.ow2.petals.microkernel.api.jbi.management.TaskProcessor;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.extension.InstallationExtensionsManager;
import org.ow2.petals.microkernel.jbi.management.task.deployment.DeploymentTaskFactory;
import org.ow2.petals.microkernel.jbi.management.task.installation.InstallationTaskFactory;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.systemstate.generated.ServiceAssembly;
import org.ow2.petals.systemstate.generated.SharedLibrary;

/**
 * Allows to restore system state after shutdown or crash.
 * @author Olivier Fabre - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SystemRecoveryService.class))
public class SystemRecoveryServiceImpl implements SystemRecoveryService {

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    private DeploymentTaskFactory deploymentTaskFactory;

    protected InstallationTaskFactory installationTaskFactory;

    /**
     * Configuration Service
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The EndpointRegistry is used to manage all the endpoint interfaces in a
     * distributed context
     */
    @Requires(name = "endpoint")
    protected EndpointRegistry endpointRegistry;

    /**
     * Communication :: JMX Agent Admin.
     */
    @Requires(name = "admin")
    protected AdminService adminService;

    /**
     * JBI messaging : Router Service.
     */
    @Requires(name = "router")
    protected RouterService router;

    /**
     * Platform :: Repository Service.
     */
    @Requires(name = "repository")
    protected RepositoryService repositoryService;

    /**
     * Platform :: SystemState Service.
     */
    @Requires(name = "systemstate")
    protected SystemStateService systemStateService;

    /**
     * Container :: Container Service.
     */
    @Requires(name = "container")
    protected ContainerService container;

    @Requires(name = "installationExtensionManager")
    private InstallationExtensionsManager installationExtensionManager;

    /**
     * Thread pool to recover JBI artifacts.
     */
    private ThreadPoolExecutor threadPoolExecutor;

    /**
     * The excecutor completion service bound to the Thread Pool Executor.
     */
    private CompletionService<RecoveryResult> ecs;

    /**
     * Inner class for {@code Futur} processing result.
     * 
     * @author rnaudin
     * 
     */
    private class RecoveryResult {
        protected final String jbiArtifactName;

        public RecoveryResult(final String jbiArtifactName) {
            this.jbiArtifactName = jbiArtifactName;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.recovery.SystemRecoveryService#
     * recoverAllEntities()
     */
    public void recoverAllEntities() throws ManagementException {
        this.log.start();

        this.recoverAllSharedLibrary();
        this.recoverAllComponent();
        this.recoverAllServiceAssembly();

        this.log.end();
    }

    /**
     * Pre-recover the components.
     * 
     * @return
     * @throws ManagementException
     */
    private void recoverAllComponent() throws ManagementException {
        this.log.start();

        // Retrieve all component directories in the components petals
        // repository. This list will be used to delete components that have
        // been unzipped in the Petals repository but no more persisted in the
        // system recovery file (system-state.xml). These unknown component
        // directories will be put in the Petals "lost+found" directory.
        final List<File> unknownDirs = new ArrayList<File>(Arrays.asList(this.repositoryService
                .getComponentsDirectory().listFiles()));

        // Retrieve all components to recover
        final List<Component> componentStateHolders = this.systemStateService
                .getComponentStateHolders();

        int counter = 0;
        // Recover state for each component
        for (final Component componentStateHolder : componentStateHolders) {

            final Callable<RecoveryResult> recoverComponentTask = new Callable<RecoveryResult>() {
                public RecoveryResult call() throws ManagementException {
                    URL installationUrl = null;
                    Jbi descriptor = null;
                    try {
                        installationUrl = new URL(componentStateHolder.getInstallURL());
                        descriptor = PackageHelper.loadDescriptor(componentStateHolder.getName(),
                                JbiArchiveType.COMPONENT_TYPE,
                                SystemRecoveryServiceImpl.this.repositoryService);
                    } catch (final Exception e) {
                        final StringBuffer msgBuffer = new StringBuffer(
                                "A component consistency is corrupted: ");
                        msgBuffer.append(componentStateHolder.getName());
                        msgBuffer.append("\nCaused by: " + e.getMessage());
                        // try to set the states of the component to UNKNOWN
                        try {
                            SystemRecoveryServiceImpl.this.systemStateService
                                    .updateComponentLifeCycleState(componentStateHolder.getName(),
                                            LifeCycleMBean.UNKNOWN);
                            SystemRecoveryServiceImpl.this.systemStateService
                                    .updateComponentInstallationState(
                                            componentStateHolder.getName(), LifeCycleMBean.UNKNOWN);
                            msgBuffer.append("\nIts state changes to UNKNOWN");
                        } catch (final Exception e1) {
                            // do nothing
                        }
                        msgBuffer.append("\nPlease check the repository consistency.");
                        throw new ManagementException(msgBuffer.toString());
                    }
                    SystemRecoveryServiceImpl.this.recoverComponent(installationUrl, descriptor,
                            componentStateHolder.getInstallState(),
                            componentStateHolder.getLifecycleState(),
                            componentStateHolder.getName());

                    // Delete component from the unknownDirs list because this
                    // component is known now...
                    if (unknownDirs != null) {
                        final File knownComp = new File(installationUrl.getPath()).getParentFile();
                        if (knownComp.exists()) {
                            unknownDirs.remove(knownComp);
                        }
                    }

                    return new RecoveryResult(componentStateHolder.getName());
                };
            };

            this.log.debug("Recover Component '" + componentStateHolder.getName() + "'");
            ecs.submit(recoverComponentTask);
            
                counter++;
        }

        for (int i = 0; i < counter; i++) {
            try {
                RecoveryResult result = ecs.take().get();
                this.log.debug("Component '" + result.jbiArtifactName + "' recovered");
            } catch (InterruptedException e) {
                this.log.error("Component recovering has been interrupted");
            } catch (ExecutionException e) {
                this.log.error(e.getMessage(), e);
            }
        }

        // Move all unknown components from Petals repository to "lost+found"
        // directory
        this.moveUnknownDirsToLostPlusFoundDir(unknownDirs);

        this.log.end();

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.recovery.SystemRecoveryService#
     * recoverAllServiceAssembly()
     */
    public void recoverAllServiceAssembly() throws ManagementException {
        this.log.start();

        // Retrieve all sa directories in the sa petals repository. This list
        // will be used to delete sa that have been unzipped in the Petals
        // repository but no more persisted in the system recovery file
        // (system-state.xml). These unknown sa directories will be put in the
        // Petals "lost+found" directory.

        final List<File> unknownDirs = new ArrayList<File>(Arrays.asList(this.repositoryService
                .getServiceAssembliesDirectory().listFiles()));

        // Retrieve all service assemblies to recover
        final List<ServiceAssembly> saStateHolders = this.systemStateService
                .getServiceAssemblyStateHolders();

        // Recover state for each service assembly
        for (final ServiceAssembly saStateHolder : saStateHolders) {

            final Callable<RecoveryResult> recoverComponentTask = new Callable<RecoveryResult>() {
                public RecoveryResult call() throws ManagementException {
                    URL installUrl = null;
                    Jbi descriptor = null;
                    try {
                        installUrl = new URL(saStateHolder.getInstallURL());
                        descriptor = PackageHelper.loadDescriptor(saStateHolder.getName(),
                                JbiArchiveType.SA_TYPE,
                                SystemRecoveryServiceImpl.this.repositoryService);
                    } catch (final Exception e) {
                        final StringBuffer msgBuffer = new StringBuffer(
                                "A service assembly consistency is corrupted: ");
                        msgBuffer.append(saStateHolder.getName());
                        msgBuffer.append("\nCaused by: " + e.getMessage());
                        // try to set the states of the component to UNKNOWN
                        try {
                            SystemRecoveryServiceImpl.this.systemStateService
                                    .updateServiceAssemblyState(saStateHolder.getName(),
                                            LifeCycleMBean.UNKNOWN);
                            msgBuffer.append("\nIts state changes to UNKNOWN");
                        } catch (final Exception e1) {
                            // do nothing
                        }
                        msgBuffer.append("\nPlease check the repository consistency");
                        throw new ManagementException(msgBuffer.toString());
                    }

                    SystemRecoveryServiceImpl.this.recoverServiceAssembly(installUrl, descriptor,
                            saStateHolder.getLifecycleState());

                    // Delete SA from the unknownDirs list because this
                    // SA is known now...
                    if (unknownDirs != null) {
                        final File knownComp = new File(installUrl.getPath()).getParentFile();
                        if (knownComp.exists()) {
                            unknownDirs.remove(knownComp);
                        }
                    }
                    return new RecoveryResult(saStateHolder.getName());
                };
            };

            this.log.debug("Recover Service Assembly '" + saStateHolder.getName() + "'");
            ecs.submit(recoverComponentTask);
        }

        for (int i = 0; i < saStateHolders.size(); i++) {
            try {
                RecoveryResult result = ecs.take().get();
                this.log.debug("Service Assembly '" + result.jbiArtifactName + "' recovered");
            } catch (InterruptedException e) {
                this.log.error("Service Assembly recovering has been interrupted");
            } catch (ExecutionException e) {
                this.log.error(e.getMessage(), e);
            }
        }

        // Move all unknown SA from Petals repository to
        // "lost+found" directory
        this.moveUnknownDirsToLostPlusFoundDir(unknownDirs);

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.management.recovery.SystemRecoveryService#
     * recoverAllSharedLibrary()
     */
    public void recoverAllSharedLibrary() throws ManagementException {
        this.log.start();

        // Retrieve all sl directories in the sl petals repository. This list
        // will be used to delete sl that have been unzipped in the Petals
        // repository but no more persisted in the system recory file
        // (system-state.xml). These unknown sl directories will be put in the
        // Petals "lost+found" directory.

        final List<File> unknownDirs = new ArrayList<File>(Arrays.asList(this.repositoryService
                .getSharedLibrariesDirectory().listFiles()));

        // Retrieve all shared library to recover
        final List<SharedLibrary> sharedLibraryStateHolders = this.systemStateService
                .getSharedLibraryStateHolders();

        // Recover state for each shared library
        for (final SharedLibrary sharedLibraryState : sharedLibraryStateHolders) {

            final Callable<RecoveryResult> recoverSLTask = new Callable<RecoveryResult>() {
                public RecoveryResult call() throws ManagementException {
                    URL installationUrl = null;
                    try {
                        installationUrl = new URL(sharedLibraryState.getInstallURL());

                        final Jbi descriptor = PackageHelper.loadDescriptor(
                                sharedLibraryState.getName() + "-"
                                        + sharedLibraryState.getVersion(), JbiArchiveType.SL_TYPE,
                                SystemRecoveryServiceImpl.this.repositoryService);

                        SystemRecoveryServiceImpl.this
                                .recoverSharedLib(installationUrl, descriptor);
                    } catch (final Exception e) {
                        final String msg = "A Shared Library can't be recovered : "
                                + sharedLibraryState.getName()
                                + "\nPlease check the repository consistency";
                        SystemRecoveryServiceImpl.this.log.error(msg, e);
                        throw new ManagementException(msg + "Cause: " + e.getCause());
                    }

                    // Delete SL from the unknownDirs list because this
                    // SL is known now...
                    if (unknownDirs != null) {
                        final File knownComp = new File(installationUrl.getPath()).getParentFile();
                        if (knownComp.exists()) {
                            unknownDirs.remove(knownComp);
                        }
                    }

                    return new RecoveryResult(sharedLibraryState.getName());
                }
            };

            this.log.debug("Recover Shared Library '" + sharedLibraryState.getName() + "'");
            ecs.submit(recoverSLTask);
        }

        for (int i = 0; i < sharedLibraryStateHolders.size(); i++) {
            try {
                RecoveryResult result = ecs.take().get();
                this.log.debug("Shared Library '" + result.jbiArtifactName + "' recovered");
            } catch (InterruptedException e) {
                this.log.error("Shared Library recovering has been interrupted");
            } catch (ExecutionException e) {
                this.log.error(e.getMessage(), e);
            }
        }

        // Now, move all unknown SL from Petals repository to
        // "lost+found" directory
        this.moveUnknownDirsToLostPlusFoundDir(unknownDirs);

        this.log.end();
    }

    /**
     * Move all unknown components from Petals repository to "lost+found"
     * directory
     * 
     * @param unknownDirs
     *            the {@link List} of unknown dirs that have to be moved.
     */
    protected void moveUnknownDirsToLostPlusFoundDir(List<File> unknownDirs) {
        if (unknownDirs != null) {
            for (final File unknownDir : unknownDirs) {
                if (!unknownDir.isFile()) {
                    if (!unknownDir.exists()) {
                        this.log.warning("The unknown dir (" + unknownDir.getAbsolutePath()
                                + ") doesn't exist.");
                    }
                    final boolean moveDir = unknownDir.renameTo(new File(this.repositoryService
                            .getLostPlusFoundDirectory(), unknownDir.getName()));
                    if (!moveDir) {
                        this.log.warning("Failed to move unknown directory ("
                                + unknownDir.getAbsolutePath() + ") to lost+found directory");
                    }
                }
            }
        }
    }

    /**
     * Recover the given component. Recovering a component consist in redoing
     * installation process of a component (see
     * <code>InstallationServiceMBean</code>), calling the "install" method on
     * the created Installer and putting the component in its previous state by
     * calling "start" or/and "stop" on the created component lifecycle.
     * 
     * @param installationRoot
     * @param installState
     * @param lifecycleState
     * @throws ManagementException
     */
    protected void recoverComponent(URL installUrl, Jbi descriptor, String installState,
            String lifecycleState, String compName) throws ManagementException {
        this.log.start();

        // Create installation execution context
        final Context installationContext = this.createRecoveryContext(descriptor, installUrl);

        // Recover component installer
        this.recoverComponent(installationContext);

        this.log.info("Installer of the component '"
                + installationContext.getDescriptor().getComponent().getIdentification().getName()
                + "' loaded");

        try {

            if (Installer.INSTALLED.equals(installState)) {
                // Re-process install
                final MBeanServer beanServer = this.adminService.getMBeanServer();
                final Object[] nullObjects = new Object[0];
                final String[] nullStrings = new String[0];
                final ObjectName componentLifeCycleName = (ObjectName) beanServer.invoke(
                        installationContext.getInstallerObjectName(), "install", nullObjects,
                        nullStrings);

                if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
                    // Re-process start followed by stop
                    beanServer.invoke(componentLifeCycleName, "start", nullObjects, nullStrings);
                    beanServer.invoke(componentLifeCycleName, "stop", nullObjects, nullStrings);
                } else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
                    // Re-process start
                    beanServer.invoke(componentLifeCycleName, "start", nullObjects, nullStrings);
                } else if (LifeCycleMBean.UNKNOWN.equals(lifecycleState)) {
                    this.log.warning("The component '" + compName + "' was in state "
                            + LifeCycleMBean.UNKNOWN + ", it is recovered in state "
                            + LifeCycleMBean.SHUTDOWN);
                    try {
                        this.systemStateService.updateComponentLifeCycleState(compName,
                                LifeCycleMBean.SHUTDOWN);
                    } catch (final Exception e) {
                        this.log.error("The component '" + compName
                                + "' failed to be recovered at state " + LifeCycleMBean.SHUTDOWN, e);
                    }
                }
            }
        } catch (final InstanceNotFoundException e) {
            throw new ManagementException("Failed to recover Component '" + compName + "'", e);
        } catch (final MBeanException e) {
            throw new ManagementException("Failed to recover Component '" + compName + "'", e);
        } catch (final ReflectionException e) {
            throw new ManagementException("Failed to recover Component '" + compName + "'", e);
        }

        this.log.end();
    }

    /**
     * Create recovery context for component and shared libraries
     * 
     * @param descriptor
     * @param installRoot
     * @return
     */
    protected Context createRecoveryContext(Jbi descriptor, URL installUrl) {
        final Context installationContext = new Context();
        installationContext.setDescriptor(descriptor);
        installationContext.setInstallUrl(installUrl);
        installationContext.setUnzipRoot(new File(installUrl.getPath()));
        return installationContext;
    }

    /**
     * Recover the given Service Assembly. Recovering a Service Assembly consist
     * in redoing deployment process of a Service Assembly (see
     * <code>DeploymentServiceMBean</code>) and putting the Service Assembly in
     * its previous state by calling "start" or/and "stop" on the created
     * component lifecycle.
     * 
     * @param installationRoot
     * @param lifecycleState
     * @throws ManagementException
     */
    protected void recoverServiceAssembly(URL installRoot, Jbi descriptor, String lifecycleState)
            throws ManagementException {
        this.log.start();

        final String saName = descriptor.getServiceAssembly().getIdentification().getName();
        try {

            // Create deployment execution context
            final Context deploymentContext = this.createDeploymentContext(installRoot, descriptor);

            // Recover the given service assembly
            this.recoverSA(deploymentContext);

            if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
                // Retrieve the STOPPED state
                final ServiceAssemblyLifeCycle salc = this.adminService
                        .getServiceAssemblyByName(saName);
                salc.start();
                salc.stop();
            } else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
                // Retrieve the STARTED state
                final ServiceAssemblyLifeCycle salc = this.adminService
                        .getServiceAssemblyByName(saName);
                salc.start();
            } else if (LifeCycleMBean.UNKNOWN.equals(lifecycleState)) {
                this.log.warning("The Service Assembly '" + saName + "' was in state "
                        + LifeCycleMBean.UNKNOWN + ", it is recovered in state "
                        + LifeCycleMBean.SHUTDOWN);
                try {
                    this.systemStateService.updateServiceAssemblyState(saName,
                            LifeCycleMBean.SHUTDOWN);
                } catch (final Exception e) {
                    this.log.error("The Service Assembly '" + saName
                            + "' failed to be recovered at state " + LifeCycleMBean.SHUTDOWN, e);
                }
            }
        } catch (final JBIException e) {
            throw new ManagementException("Failed to recover Service Assembly '" + saName + "'", e);
        }

        this.log.end();
    }

    /**
     * Create the deployment context
     * 
     * @param descriptor
     *            the sa jbi descriptor
     * @return the Deployment Context HashMap
     */
    protected Context createDeploymentContext(URL installUrl, Jbi descriptor) {
        final Context deploymentContext = new Context();
        deploymentContext.setUnzipRoot(new File(installUrl.getPath()));
        deploymentContext.setDescriptor(descriptor);
        return deploymentContext;
    }

    /**
     * Recover the given shared library. Recovering a shared library consist in
     * redoing installation process of a shared library (see
     * <code>InstallationServiceMBean</code>).
     * 
     * @param installationRoot
     * @throws ManagementException
     */
    protected void recoverSharedLib(URL installUrl, Jbi descriptor) throws ManagementException {
        this.log.start();

        // Create installation execution context
        final Context installationContext = this.createRecoveryContext(descriptor, installUrl);

        // Re-process installation of the shared libraries
        this.recoverSharedLib(installationContext);

        this.log.end();
    }

    /**
     * Start FRACTAL component.
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(SystemRecoveryService.COMPONENT_LOGGER_NAME));
        this.log.start();

        // Initialize task factory
        ContainerConfiguration containerConfiguration = this.configurationService
                .getContainerConfiguration();
        this.installationTaskFactory = new InstallationTaskFactory(this.systemStateService,
                this.repositoryService, this.log, this.adminService, this.router, this.container,
                containerConfiguration, this.installationExtensionManager.getExtensions());
        this.deploymentTaskFactory = new DeploymentTaskFactory(this.endpointRegistry, null,
                this.systemStateService, this.repositoryService, this.log, this.adminService,
                this.container, containerConfiguration,
                this.installationExtensionManager.getExtensions());

        /*
         * Initialize the recovering workers. As we use an unbouded queue, the
         * size of the tread pool is driven by the core pool size. The max pool
         * size has no effect.
         */
        this.threadPoolExecutor = new ThreadPoolExecutor(
                containerConfiguration.getRecoveryCorePoolSize(),
                containerConfiguration.getRecoveryMaxPoolSize(),
                containerConfiguration.getRecoveryKeepAliveTime(), TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>());

        this.ecs = new ExecutorCompletionService<RecoveryResult>(threadPoolExecutor);

        this.log.end();
    }

    /**
     * Stop the FRACTAL component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();

        this.threadPoolExecutor.shutdownNow();
    }

    /**
     * Process the deployment of a given Service assembly
     * 
     * @param saDeploymentContext
     *            the deployment context of the given SA
     * @throws Exception
     *             if an error occurred during deployment process
     */
    private void recoverSA(Context deploymentContext) throws ManagementException {
        final Jbi descriptor = deploymentContext.getDescriptor();

        // Create deployment processor and fill it with tasks
        final TaskProcessor processor = this.createSARecoveryProcessor(deploymentContext);

        // Launch the recover processor
        final Throwable processResult = processor.process();
        if (processResult != null) {
            throw new ManagementException("Failed to recover Service Assembly '"
                    + descriptor.getServiceAssembly().getIdentification().getName() + "'",
                    processResult);
        }
    }

    /**
     * Create the deployment processor and add all required task for the
     * complete recovery of a service assembly
     * 
     * @param deploymentContext
     *            the deployment context
     * @return the filled deployment processor
     */
    private TaskProcessor createSARecoveryProcessor(Context deploymentContext) {
        final TaskProcessor processor = new TaskProcessor(deploymentContext, this.log);
        processor.addTask(this.deploymentTaskFactory.getSaCheckPackageTask());
        processor.addTask(this.deploymentTaskFactory.getCreateXMLDeploymentResultTask());
        processor.addTask(this.deploymentTaskFactory.getRetrieveSAAndSUInstallRootsTask());
        processor.addTask(this.deploymentTaskFactory.getRegisterSALifeCycleTask());
        processor.addTask(this.deploymentTaskFactory.getDeployAllSUTask());
        processor.addTask(this.deploymentTaskFactory.getRegisterAllConnectionTask());
        return processor;
    }

    /**
     * Load the installer for a previously successfully installed component.
     * Used only by the Recovery Service.
     * 
     * @param installationContext
     *            the installation context for the recovery process : handles
     *            descriptor, archive URI and installRoot information
     * @throws ManagementException
     *             if the component installer can't be recovered
     */
    private void recoverComponent(Context installationContext) throws ManagementException {
        final Jbi descriptor = installationContext.getDescriptor();

        // Create installation processor and fill with tasks
        final TaskProcessor processor = this.createComponentRecoveryProcessor(installationContext);

        // Launch the recover processor
        final Throwable processResult = processor.process();
        if (processResult != null) {
            throw new ManagementException("Failed to recover Component '"
                    + descriptor.getComponent().getIdentification().getName() + "'", processResult);
        }
    }

    /**
     * Create the installation processor and add all required task for the
     * complete recovery of a component
     * 
     * @param installationContext
     *            the installation context
     * @return the filled installation processor
     */
    private TaskProcessor createComponentRecoveryProcessor(Context installationContext) {
        final TaskProcessor processor = new TaskProcessor(installationContext, this.log);
        processor.addTask(this.installationTaskFactory.getRegisterComponentInSLTask());
        processor.addTask(this.installationTaskFactory.getCreateAndRegisterInstallerMBeanTask());
        return processor;
    }

    /**
     * Reload the shared lib classloader for the given shared lib
     * 
     * @param installationContext
     *            the installation context for the recory process : handles
     *            descriptor, archive uri and installRoot information
     * @throws ManagementException
     *             if the shared library classloader can't be recovered
     */
    private void recoverSharedLib(Context installationContext) throws ManagementException {
        final Jbi descriptor = installationContext.getDescriptor();

        // Create installation processor and fill it with tasks
        final TaskProcessor processor = this.createSLRecoveryProcessor(installationContext);

        // Launch the recover processor
        final Throwable processResult = processor.process();
        if (processResult != null) {
            throw new ManagementException("Failed to recover Shared Library '"
                    + descriptor.getSharedLibrary().getIdentification().getName() + "'",
                    processResult);
        }

    }

    /**
     * Create the installation processor and add all required task for the
     * complete recovery of a shared lib
     * 
     * @param installationContext
     *            the installation context
     * @return the filled installation processor
     */
    private TaskProcessor createSLRecoveryProcessor(Context installationContext) {
        final TaskProcessor processor = new TaskProcessor(installationContext, this.log);
        processor.addTask(this.installationTaskFactory.getSLCheckPackageTask());
        processor.addTask(this.installationTaskFactory.getCreateSLClassloaderTask());
        return processor;
    }

}
