/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.api.util.ZipUtil;

/**
 * This task extracts entity (component, shared lib, service assembly) archive into the work directory.
 * <p>
 * Inputs : archive {@link URI} <br/>
 * Outputs : {@link File} pointing on unzipped archive <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ExtractTask extends AbstractFileManipulationTask {

    /**
     * The temporary unique id suffix
     */
    private static AtomicLong uniqueIdSuffix = new AtomicLong();

    public ExtractTask(final LoggingUtil log, final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
    }

    public void execute(Context context) throws Exception {
        URL archiveUrl = context.getArchiveUrl();
        File unzipRoot = this.explodeEntityPackage(archiveUrl);
        context.setUnzipRoot(unzipRoot);
    }

    private File explodeEntityPackage(URL archiveUrl) throws PetalsException {
        File zipFile = new File(URLDecoder.decode(archiveUrl.getPath()));
        File destFile = new File(this.getWorkDirectory(), zipFile.getName() + "-"
                + uniqueIdSuffix.incrementAndGet());
        try {
            destFile.mkdir();
            ZipUtil.unzipArchive(zipFile, destFile);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(destFile);
            } catch (IOException e1) {
                // Nothing to do, just log
                this.log.warning("Failed to delete the temporary directory '"
                        + destFile.getAbsolutePath() + "'", e);
            }
            throw new PetalsException(e);
        }
        return destFile;
    }

    public void undo(Context context) throws Exception {
        File tempDir = context.getUnzipRoot();
        if (tempDir != null) {
            try {
                if (tempDir.exists()) {
                    FileUtils.forceDelete(tempDir);
                }
            } catch (IOException e) {
                this.log.error("Failed to delete the temporary directory '"
                        + tempDir.getAbsolutePath() + "'", e);
            }
        }
    }

}
