/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;

import javax.jbi.JBIException;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.jbi.management.Task;

/**
 * This task creates a service unit lifecycle and registers this lifecycle into
 * the sa lifecycle.
 * 
 * <p>
 * Inputs : {@link Jbi} of the SU, {@link ServiceAssemblyLifeCycle}, SU install
 * root {@link URI} <br/>
 * The task has a robust approach: it stops at the first subtask failed and has
 * undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CreateAndRegisterSULifeCycleTask implements Task {
    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    public CreateAndRegisterSULifeCycleTask(AdminService adminService) {
        this.adminService = adminService;
    }

    @SuppressWarnings("deprecation")
    public void execute(Context context) throws Exception {
        URL installUrl = context.getSuInstallUrl();
        ServiceUnit serviceUnitInSa = context.getSuDescriptorInSa();
        final ServiceAssemblyLifeCycle saLifeCycle = context.getSaLifeCycle();
        // Create SU lifecycle
        ServiceUnitLifeCycle suLifeCycle = new ServiceUnitLifeCycle(serviceUnitInSa, URLDecoder
                .decode(installUrl.getPath()), this.adminService);
        // Register this SU lifecycle into the SA lifecycle
        this.registerSULifeCycle(suLifeCycle, saLifeCycle);
    }

    public void undo(Context context) throws Exception {
        if ((context.getSuDescriptorInSa() != null) && (context.getSaLifeCycle() != null)) {
            this.unregisterSULifeCycle(context.getSaLifeCycle(), context.getSuDescriptorInSa());
        }
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param suLifeCycle
     *            the {@link ServiceUnitLifeCycle} to register
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    private void registerSULifeCycle(ServiceUnitLifeCycle suLifeCycle,
            final ServiceAssemblyLifeCycle saLifeCycle) throws ManagementException {
        try {
            saLifeCycle.registerSU(suLifeCycle);
        } catch (JBIException e) {
            throw new ManagementException("Error while registering the service unit '"
                    + suLifeCycle.getSuName()
                    + "' life cycle into the service assembly life cycle '"
                    + saLifeCycle.getServiceAssembly().getIdentification().getName() + "'", e);
        }
    }

    /**
     * Unregister the {@link ServiceUnitLifeCycle} from the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be unregistered
     * @param serviceUnit
     *            the service unit production element of the su to unregister
     */
    private void unregisterSULifeCycle(ServiceAssemblyLifeCycle saLifeCycle, ServiceUnit serviceUnit) {
        for (ServiceUnitLifeCycle suLifeCycle : saLifeCycle.getServiceUnitLifeCycles()) {
            if (suLifeCycle.getSuName().equals(serviceUnit.getIdentification().getName())) {
                saLifeCycle.unregisterSU(suLifeCycle);
                break;
            }
        }
    }
}
