/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.deploy;

import java.net.URL;
import java.net.URLDecoder;

import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task deploys service unit to the targeted component.
 * <p>
 * This task MUST be executed with a {@link Context} containing:
 * <ul>
 * <li>the service unit part of the service-assembly JBI descriptor (
 * {@link ServiceUnit}),</li>
 * <li>the root directory of the service-unit installation as {@link URL},</li>
 * </ul>
 * and returns following data in the {@link Context}:
 * <ul>
 * <li>the XML result for of the deployment as {@link String}.</li>
 * </ul>
 * </p>
 * <p>
 * The task has got a robust approach : it stops at the first subtask failed
 * and has undo capability.
 * </p>
 * 
 * @author Frederic Gardes - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public class DeploySUToComponentTask extends AbstractLoggableTask {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    public DeploySUToComponentTask(final LoggingUtil log, final AdminService adminService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.adminService = adminService;
    }

    public void execute(Context context) throws Exception {
        final URL installUrl = context.getSuInstallUrl();
        final ServiceUnit serviceUnit = context.getSuDescriptorInSa();
        String xmlResult = null;
        xmlResult = this.deploySUtoComponent(installUrl, serviceUnit);
        context.setComponentResult(xmlResult);
        if (xmlResult.contains("<task-result>FAILED</task-result>")) {
            // throw an exception to trigger the unprocessing of the
            // processor
            throw new Exception();
        }
    }

    /**
     * Deploy the given service unit to the targeted component. The component
     * must be in the STARTED state to allow deployment of service units.
     * 
     * @param installationRoot
     *            the SU installation {@link URL}
     * @param serviceUnit
     *            the service unit production element
     * @return a {@link String} xml result of the service unit deployment on the
     *         targeted component (see section 6.9 of the JSR 208)
     * @throws ManagementException
     *             if component and related object are missing, if the call of
     *             the service unit manager method "deploy" fails or if the
     *             component isn't in the STARTED state
     */
    @SuppressWarnings("deprecation")
    protected String deploySUtoComponent(URL installUrl, ServiceUnit serviceUnit) throws Exception {
        String xmlReturn = null;
        // Retrieve the target component life cycle
        String targetComponent = serviceUnit.getTarget().getComponentName();
        Installer installer = this.adminService.getInstallerByName(targetComponent);
        if (installer == null) {
            throw new ManagementException(
                    "The Service Unit can not be deployed, as the component '" + targetComponent
                            + "' is not installed");
        }

        // Retrieve the service unit manager of the component that allow to
        // process deployment
        ServiceUnitManager sum = installer.getComponent().getServiceUnitManager();
        if (sum == null) {
            throw new ManagementException("Component '" + targetComponent
                    + "' doesn't support deployment : service unit manager is null");
        }
        // Set the context class loader to the component's one just for the
        // method invocation
        final String serviceUnitName = serviceUnit.getIdentification().getName();
        Thread.currentThread().setContextClassLoader(sum.getClass().getClassLoader());
        try {
            xmlReturn = sum.deploy(serviceUnitName, URLDecoder.decode(installUrl.getPath()));
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }

        return xmlReturn;
    }

    public void undo(Context context) throws Exception {
        this.log.debug("Revert DeploySUToComponent");
        if ((context.getSuInstallUrl() != null) && (context.getSuDescriptorInSa() != null)) {
            try {
                this.undeploySUFromComponent(context.getSuInstallUrl(), context
                        .getSuDescriptorInSa());
            } catch (ManagementException e) {
                this.log.error("Failed to revert a DeploySUToComponentask", e);
            }
        }
    }

    /**
     * Undeploy the given service unit from the targeted component.
     * 
     * @param installUrl
     *            the su installation {@link URL}
     * @param serviceUnit
     *            the service unit production element
     * @throws ManagementException
     *             if component and related object are missing, if the call of
     *             the service unit manager method "undeploy" fails
     */
    @SuppressWarnings("deprecation")
    private void undeploySUFromComponent(URL installUrl, ServiceUnit serviceUnit)
            throws ManagementException {
        // Retrieve the target component life cycle
        String targetComponent = serviceUnit.getTarget().getComponentName();
        if (targetComponent != null) {
            Installer installer = this.adminService.getInstallerByName(targetComponent);
            if (installer == null) {
                throw new ManagementException(
                        "The component lifecycle element for service unit deployment must be non null.");
            }
            // Retrieve the service unit manager of the component that allow to
            // process deployment
            ServiceUnitManager sum = installer.getComponent().getServiceUnitManager();
            if (sum == null) {
                throw new ManagementException(
                        "The service unit manager is null for the component '" + targetComponent
                                + "'. The service units can not be deployed.");
            }
            String serviceUnitName = serviceUnit.getIdentification().getName();
            try {
                Thread.currentThread().setContextClassLoader(sum.getClass().getClassLoader());
                sum.undeploy(serviceUnitName, URLDecoder.decode(installUrl.getPath()));
            } catch (DeploymentException e) {
                throw new ManagementException("Undeployement of the service unit '"
                        + serviceUnitName + "' has failed (caused by a target component '"
                        + serviceUnitName + "' SU manager exception)", e);
            } finally {
                Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
            }
        }
    }
}
