/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.deploy;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task extracts service unit archives into the repository.
 * <p>
 * Inputs : {@link Jbi}, parent service assembly install root {@link URI} <br/>
 * Outputs : SU install roots {@link Map} <br/>
 * The task has a non-robust approach: it stops at the first subtask failed
 * and has no undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ExtractSUsTask extends AbstractLoggableTask {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositoryService;

    public ExtractSUsTask(final LoggingUtil log, final RepositoryService repositoryService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.repositoryService = repositoryService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management
     * .Context)
     */
    public void execute(Context context) throws Exception {

        Map<String, URL> suInstallUrls = this.expandSUs(context.getInstallUrl(), context
                .getDescriptor());
        context.setSuInstallUrls(suInstallUrls);
    }

    /**
     * Explode service unit packages into the repository
     * 
     * @param saInstallRoot
     *            the root {@link URI} of the service assembly
     * @param descriptor
     *            the service assembly descriptor
     * @return the map containing all su install roots
     * @throws ManagementException
     *             if the repository service doesn't manage to expend SU archive
     * @throws PetalsException
     * @throws MalformedURLException
     */
    @SuppressWarnings("deprecation")
    protected Map<String, URL> expandSUs(URL saInstallUrl, Jbi descriptor)
            throws ManagementException, PetalsException, MalformedURLException {
        Map<String, URL> suInstallUrls = new HashMap<String, URL>();
        List<ServiceUnit> sus = descriptor.getServiceAssembly().getServiceUnit();
        for (ServiceUnit unit : sus) {
            String suZipName = unit.getTarget().getArtifactsZip().trim();
            URL suArchiveUrl = new File(URLDecoder.decode(saInstallUrl.getPath()), suZipName).toURI().toURL();
            URL suInstallUrl = this.expandSUIntoSA(unit.getIdentification().getName(),
                    suArchiveUrl, descriptor.getServiceAssembly().getIdentification().getName());
            suInstallUrls.put(unit.getIdentification().getName(), suInstallUrl);
        }
        return suInstallUrls;
    }

    /**
     * Expand a service unit package into its service assembly installRoot.
     * 
     * @param suId
     *            the unique identifier of the su
     * @param suZipLocation
     *            URI representing the su zip location
     * @param saId
     *            the unique identifier of the sa
     * @return SU installationRoot, null if it fails
     * @throws ManagementException
     *             if it fails to expend the su package to the repository
     */
    protected URL expandSUIntoSA(String suId, URL suZipLocation, String saId)
            throws ManagementException {
        URL result = null;
        try {
            final File resultFile = this.repositoryService.explodeSUIntoSAInstallDirectory(suId,
                    suZipLocation, saId);
            result = resultFile.toURI().toURL();
        } catch (IOException e) {
            throw new ManagementException("Unexpected error expending service JBI installation "
                    + "package into repository", e);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.management.Task#undo(org.ow2.petals.jbi.management
     * .Context)
     */
    public void undo(Context context) throws Exception {
        // The folder will be removed by the undo of the CreateSARepository task
    }
}
