/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.deploy;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Connection;
import org.ow2.petals.jbi.descriptor.original.generated.Connections;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task registers connections if they exist.
 * <p>
 * Inputs : {@link Jbi}<br/>
 * The task has a robust approach: it stops at the first subtask failed
 * and has undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RegisterAllConnectionTask extends AbstractLoggableTask {

    protected EndpointRegistry endpointRegistry;

    public RegisterAllConnectionTask(final LoggingUtil log,
            final EndpointRegistry endpointRegistry,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.endpointRegistry = endpointRegistry;
    }

    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        Connections connections = descriptor.getServiceAssembly().getConnections();
        List<Connection> successfulyRegisteredConnections = new ArrayList<Connection>();
        if (connections != null) {
            for (Connection connection : connections.getConnection()) {
                try {
                    this.registerConnection(connection, descriptor);
                } catch (ManagementException e) {
                    // Unregister previously registered connection
                    for (Connection successConnection : successfulyRegisteredConnections) {
                        try {
                            this.removeConnection(successConnection);
                        } catch (Exception e1) {
                            this.log.error("Can't delete an already registered connection");
                        }
                    }
                    throw e;
                }
                successfulyRegisteredConnections.add(connection);
            }
        }
    }

    /**
     * Register a connection into the endpoint registry of the JBI container
     * 
     * @param connection
     *            The {@link Connection} to register
     * @param descriptor
     *            the service assembly descriptor
     * @throws ManagementException
     *             when container fails to register a connection into the
     *             endpoint registry
     */
    protected void registerConnection(Connection connection, Jbi descriptor)
            throws ManagementException {
        String saName = descriptor.getServiceAssembly().getIdentification().getName();
        QName consInterface = connection.getConsumer().getInterfaceName();
        QName consService = connection.getConsumer().getServiceName();
        String consEndpoint = connection.getConsumer().getEndpointName();
        QName provService = connection.getProvider().getServiceName();
        String provEndpoint = connection.getProvider().getEndpointName();
        try {
            if (consInterface != null) {
                this.endpointRegistry.registerConnection(consInterface, provService, provEndpoint);
            } else {
                this.endpointRegistry.registerConnection(consService, consEndpoint, provService,
                        provEndpoint);
            }
        } catch (JBIException e) {
            throw new ManagementException(
                    "Error while registering a service unit connection for the service assembly '"
                            + saName + "'", e);
        }

    }

    public void undo(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        if (descriptor != null) {
            Connections connections = descriptor.getServiceAssembly().getConnections();
            if (connections != null) {
                for (Connection connection : connections.getConnection()) {
                    try {
                        this.removeConnection(connection);
                    } catch (Exception e) {
                        this.log.error("Failed to revert a RegisterAllConnectionTask", e);
                    }
                }
            }
        }
    }

    /**
     * Remove a connection registered during the SA deployment process
     * 
     * @param connection
     *            a Connection to remove
     * @throws Exception
     *             if problem occurs during the deletion of a specific
     *             connection
     */
    protected void removeConnection(Connection connection) throws Exception {
        if ((connection != null) && (connection.getConsumer() != null)) {
            QName consInterface = connection.getConsumer().getInterfaceName();
            QName consService = connection.getConsumer().getServiceName();
            String consEndpoint = connection.getConsumer().getEndpointName();
            QName provService = connection.getProvider().getServiceName();
            String provEndpoint = connection.getProvider().getEndpointName();
            if ((provEndpoint != null) && (provService != null)) {
                if (consInterface != null) {
                    this.endpointRegistry.deregisterConnection(consInterface, provService,
                            provEndpoint);
                } else if ((consService != null) && (consEndpoint != null)) {
                    this.endpointRegistry.deregisterConnection(consService, consEndpoint,
                            provService, provEndpoint);
                }
            }
        }
    }

}
