/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.deploy;

import java.net.URI;

import javax.jbi.JBIException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task registers ServiceAssemblyLifeCycle into manager service map.
 * <p>
 * Inputs : {@link Jbi}, {@link ServiceAssemblyLifeCycle} <br/>
 * The task has a robust approach: it stops at the first subtask failed and has
 * undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RegisterSALifeCycleTask extends AbstractLoggableTask {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    protected ContainerService containerService;

    public RegisterSALifeCycleTask(final LoggingUtil log, final AdminService adminService,
            final ContainerService containerService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.adminService = adminService;
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        ServiceAssemblyLifeCycle saLifeCycle = this.createSALifeCycle(descriptor);
        // set the descriptor before registering to the admin service
        saLifeCycle.init(descriptor.getServiceAssembly());
        this.adminService.registerServiceAssembly(saLifeCycle, descriptor.getServiceAssembly()
                .getIdentification().getName());
        context.setSaLifeCycle(saLifeCycle);
    }

    /**
     * Creates a new {@link ServiceAssemblyLifeCycle}
     * 
     * @param installationRoot
     *            the root {@link URI} of the service assembly into the
     *            repository
     * @param descriptor
     *            the service assembly jbi descriptor
     * @return a new {@link ServiceAssemblyLifeCycle}
     * @throws PetalsException
     * @throws ADLException
     * @throws IllegalLifeCycleException
     * @throws NoSuchInterfaceException
     * @throws JBIException
     */
    protected ServiceAssemblyLifeCycle createSALifeCycle(Jbi descriptor) throws PetalsException,
            NoSuchInterfaceException, IllegalLifeCycleException, ADLException, JBIException {
        Component serviceAssemblyLifeCycleComponent = this.containerService
                .createServiceAssemblyLifeCycle(descriptor.getServiceAssembly());
        FractalHelper.startComponent(serviceAssemblyLifeCycleComponent);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = (ServiceAssemblyLifeCycle) serviceAssemblyLifeCycleComponent
                .getFcInterface("service");
        return serviceAssemblyLifeCycle;

    }

    public void undo(Context context) throws Exception {
        if (context.getDescriptor() != null) {
            Jbi descriptor = context.getDescriptor();
            final String saName = descriptor.getServiceAssembly().getIdentification().getName();
            try {
                this.adminService.unregisterServiceAssembly(saName);
            } catch (ManagementException e) {
                this.log.error("Failed to revert a RegisterSALifeCycleTask: " + e.getMessage());
            }
            try {
                this.containerService.removeServiceAssemblyLifeCycle(saName);
            } catch (PetalsException e) {
                this.log.error("Failed to revert a RegisterSALifeCycleTask: " + e.getMessage());
            }
        }
        context.setSaLifeCycle(null);
    }

}
