/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import java.io.File;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Component;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractCheckPackageTask;
import org.ow2.petals.microkernel.jbi.management.task.installation.InstallationUtils;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * This task checks the validity of the component package.
 * @author Olivier Fabre - EBM WebSourcing
 */
public class ComponentCheckPackageTask extends AbstractCheckPackageTask {

    protected AdminService adminService;

    public ComponentCheckPackageTask(final LoggingUtil log, final AdminService adminService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.adminService = adminService;
    }

    /**
     * Check the validity of the component archive
     * 
     * @param descriptor
     *            the component <code>Jbi</code>
     * @param filePackage
     *            the service assembly package root
     * @return true if checking is ok and false otherwise
     * @throws PetalsException
     *             if component element or name is null or empty, if component
     *             is already installed or if some classpath elements of the
     *             component are missing
     */
    @Override
    protected void checkPackage(Jbi descriptor, File filePackage) throws PetalsException {
        /*
         * check if JBI descriptor has a component element
         */
        Component comp = descriptor.getComponent();
        if (comp == null) {
            throw new PetalsException("The component production element was not found in "
                    + "JBI Descriptor within installation package: "
                    + filePackage.getAbsolutePath());
        } else {
            /*
             * check if it is already loaded by package uuid
             */
            String compName = comp.getIdentification().getName();
            if (StringHelper.isNullOrEmpty(compName)) {
                throw new PetalsException(
                        "The component name must be non null and non empty for the new component :"
                                + filePackage.getAbsolutePath());
            }
            if (adminService.getInstallerByName(compName) != null) {
                throw new PetalsException("Other installation package already exists loaded in "
                        + "the component framework: " + compName);
            } else {
                /*
                 * Check the classpath elements of the component
                 */
                InstallationUtils.checkClassPathElements(filePackage, comp.getBootstrapClassPath()
                        .getPathElement());
                InstallationUtils.checkClassPathElements(filePackage, comp.getComponentClassPath()
                        .getPathElement());
            }
        }
    }

    @Override
    protected String extractEntityName(Jbi descriptor) {
        return descriptor.getComponent().getIdentification().getName();
    }

}
