/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import java.io.IOException;

import javax.jbi.JBIException;
import javax.management.ObjectName;
import javax.naming.NamingException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task creates the InstallerMBean for a component and register it in the
 * JMX server.
 * <p>
 * The task has got a non-robust approach : it stops at the first subtask failed
 * <br />
 * and have the undo capability.<br />
 * Inputs : {@link Jbi}, component install root URI, ComponentContext<br/>
 * Outputs : component installer {@link ObjectName}
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CreateAndRegisterInstallerMBeanTask extends AbstractLoggableTask {
    /**
     * JBI Container :: JMX Agent Admin
     */
    private final AdminService adminService;

    protected ContainerService containerService;

    /**
     * Creates a new instance of {@link CreateAndRegisterInstallerMBeanTask}
     * 
     * @param log
     * @param adminService
     * @param logger
     * @param router
     * @param containerService
     */
    public CreateAndRegisterInstallerMBeanTask(final LoggingUtil log,
            final AdminService adminService, final ContainerService container,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.adminService = adminService;
        this.containerService = container;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.Map)
     */
    public void execute(Context context) throws Exception {
        final ObjectName result = this.createAndRegisterInstallerMBean(context);
        context.setInstallerObjectName(result);
    }

    /**
     * Create and register a new InstallerMBean for the loaded component
     * 
     * @param componentContext
     *            the new Component execution context
     * @param descriptor
     *            the component Jbi
     * @param installationRoot
     *            the URI representing the component installation folder
     * @return the new InstallerMBean name
     * @throws PetalsException
     */
    private ObjectName createAndRegisterInstallerMBean(Context ctxt)
            throws PetalsException {
        ObjectName result = null;
        try {
            final Jbi descriptor = ctxt.getDescriptor();
            final org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription = descriptor
                    .getComponent();
            // Create the Fractal installer
            final Component installer = this.containerService.createInstaller(componentDescription);
            // Start the Fractal installer
            FractalHelper.startComponent(installer);
            // Initialize the Fractal installer
            ((Installer) installer.getFcInterface("service")).init(ctxt);
            // Register the InstallerMBean
            result = this.adminService
                    .registerInstaller((Installer) installer.getFcInterface("/content"),
                            componentDescription.getIdentification().getName());
        } catch (final ManagementException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        } catch (final JBIException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        } catch (IOException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        } catch (NamingException e) {
            throw new PetalsException("Error creating or registring installer MBean", e);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.Map)
     */
    public void undo(Context context) throws Exception {
        final Jbi descriptor = context.getDescriptor();
        final String componentName = descriptor.getComponent().getIdentification().getName();

        try {
            // Unregister the InstallerMBean
            this.adminService.unregisterInstaller(componentName);
        } catch (final ManagementException e) {
            this.log.error("Component installer can't be unregistered or doesn't exist", e);
        }

        try {
            this.containerService.removeInstaller(componentName);
        } catch (PetalsException e) {
            this.log.error("Fractal installer can't be removed or doesn't exist", e);
        }
    }
}
