/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.uninstall.component;

import javax.jbi.JBIException;
import javax.management.ObjectName;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This task unregister component installer from JMX server.
 * <p>
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * Inputs : component name <br/>
 * Outputs : component installer {@link ObjectName}
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing 
 * @author Frederic Gardes - EBM WebSourcing
 */
public class UnregisterInstallerTask extends AbstractLoggableTask {

    private final AdminService adminService;

    private final ContainerService containerService;

    public UnregisterInstallerTask(final LoggingUtil log, final AdminService adminService,
            final ContainerService containerService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.adminService = adminService;
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        if (context.getEntityName() != null) {
            String componentName = context.getEntityName();
            if (componentName != null) {
                // Clean up the Installer bootstrap
                this.cleanUpInstaller(componentName);
                // Unregister the Installer MBean
                try {
                    this.adminService.unregisterInstaller(componentName);
                } catch (ManagementException e) {
                    this.log.error("Installer MBean can not be unregistered", e);
                }
                // Remove the Fractal installer
                try {
                    this.containerService.removeInstaller(componentName);
                } catch (PetalsException e) {
                    this.log.error("Fractal installer can not be removed", e);
                }
            }

        }
    }

    /**
     * Clean up the resources used by the boostrap installer of the component,
     * for example, the MBean used to configure component properties.
     * 
     * @param componentName
     *            The JBI identifier of the component for which the installer
     *            must be cleaned up.
     * @throws JBIException
     */
    private final void cleanUpInstaller(final String componentName) throws JBIException {
        final Installer installer = this.adminService.getInstallerByName(componentName);
        if (installer != null) {
            installer.shutdown();
        }

    }

    public void undo(Context context) throws Exception {
        // TODO : recreate and register the installer
    }

}
