/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.registry;

import static org.ow2.petals.microkernel.api.configuration.ConfigurationService.MASTER;
import static org.ow2.petals.microkernel.api.configuration.ConfigurationService.PEER;
import static org.ow2.petals.microkernel.api.configuration.ConfigurationService.PROPERTY_REGISTRY_MODE;
import static org.ow2.petals.microkernel.api.configuration.ConfigurationService.SLAVE;
import static org.ow2.petals.microkernel.api.configuration.ConfigurationService.STANDALONE;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.ScheduledExecutorService;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Cardinality;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Contingency;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration.RegistryMode;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryListener;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.registry.api.LocalRegistry;
import org.ow2.petals.registry.api.config.Configuration;
import org.ow2.petals.registry.api.config.ConfigurationLoader;
import org.ow2.petals.registry.api.config.ConfigurationLoaderFactory;
import org.ow2.petals.registry.api.config.RemoteConfiguration;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.client.RegistryClientFactory;
import org.ow2.petals.registry.core.factory.RegistryFactory;

/**
 * The registry facade implementation based on the petals-registry-* project.
 * @author Christophe Hamerling - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = EndpointRegistry.class))
public class DistributedEndpointRegistryImpl extends BaseEndpointRegistry {

    /**
     * Configuration service fractal component
     */
    @Requires(name = "configuration")
    protected ConfigurationService configurationService;

    @Requires(name = "topology")
    protected TopologyService localTopologyService;

    @Requires(name = LISTENER_FRACTAL_PREFIX, cardinality = Cardinality.COLLECTION, contingency = Contingency.OPTIONAL, signature = RegistryListener.class)
    protected Hashtable<String, Object> listeners = new Hashtable<String, Object>();

    protected ContainerConfiguration localContainerConfiguration;

    protected LocalRegistry registry;
    
    private ScheduledExecutorService registryTopologyUpdater;

    private Configuration localConfig;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(EndpointRegistry.COMPONENT_LOGGER_NAME));
        this.log.start();
        this.localContainerConfiguration = this.configurationService.getContainerConfiguration();

        try {
            // FIXME : To be moved in the registry configuration part
            // load the registry configuration file
            this.createRegistryConfig();
            this.loadRegistry();
            this.createRegistryTopology();

            this.registry.init();
            this.registry.start();

            // clean local data
            // FIXME : clean at init not after start !
            this.cleanData();

        } catch (RegistryException e) {
            this.log.error(e.getMessage(), e);
            if ((this.registry != null)
                    && (this.registry.isInitialized() || this.registry.isStarted())) {
                try {
                    this.registry.stop();
                } catch (LifeCycleException e1) {
                    throw new org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryException(
                            e1);
                }
            }
            throw new org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryException(e);
        } catch (Exception e) {
            this.log.error(e.getMessage(), e);
            throw e;
        }

        // start the update thread when we are sure that the registry is really
        // started
        this.createTopologyUpdater();

        // recovering data from network
        try {
            if (this.localContainerConfiguration.isPeer() || this.localContainerConfiguration.isSlave()) {
                this.registry.synchronizeData();
            }
        } catch (RegistryException e) {
            final String message = "Can not synchronize data from network, mode will be downgraded";
            if (this.log.isDebugEnabled()) {
                this.log.warning(message, e);
            } else {
                this.log.warning(message);
            }
        }

        this.client = RegistryClientFactory.getNewClient(this.registry);

        this.log.info("The registry is ready process requests on "
                + this.registry.getContext().getConfiguration().getMessageReceiverURL());
        this.log.end();
    }

    /**
     * @throws RegistryException
     * @throws PetalsException
     */
    protected void loadRegistry() throws RegistryException, PetalsException {
        this.registry = RegistryFactory.getInstance().loadLocal(this.localConfig);
        if (this.registry == null) {
            throw new PetalsException("Registry can not be loaded!");
        }
    }

    /**
     * @param updatePeriod
     */
    protected void createTopologyUpdater() {
        if (this.localTopologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {

            try {
                this.localTopologyService.registerLocalContainerOnMaster();
            } catch (Throwable ex) {
                this.log
                        .warning("Can't register local node onto the master node (nor updating the topology)");
            }
        }
    }

    /**
     * @throws PetalsException
     */
    protected void createRegistryTopology() throws PetalsException {
        // add the remote registries, all the registries are embedded in
        // each petals runtime...
        Set<ContainerConfiguration> containers = this.localTopologyService.getContainersConfiguration(null);
        String currentContainerSubdomainName = null;

        //Retrieve the subdomain of the current container
        for (ContainerConfiguration containerConfiguration : containers) {
            if (containerConfiguration.getName().equals(this.localContainerConfiguration.getName())) {
                currentContainerSubdomainName = containerConfiguration.getSubdomainName();
                break;
            }
        }

        List<ContainerConfiguration> remoteContainerConfiguration = new ArrayList<ContainerConfiguration>();

        //Searching the master node
        for (ContainerConfiguration containerConfiguration : containers) {
            if (! containerConfiguration.getSubdomainName().equals(currentContainerSubdomainName)) {
                continue;
            }

            //Skipping local container
            if (containerConfiguration.getSubdomainName().equals(this.localContainerConfiguration.getName())) {
                continue;
            }

            //Now handling local subdomain container configurations
            if (containerConfiguration.getRegistryMode().equals(RegistryMode.MASTER) ||
                    containerConfiguration.getRegistryMode().equals(RegistryMode.PEER)) {
                remoteContainerConfiguration.add(containerConfiguration);
            }
        }

        if (this.localContainerConfiguration.isSlave() || this.localContainerConfiguration.isPeer()) {
            for (ContainerConfiguration containerConfiguration : remoteContainerConfiguration) {
                RemoteConfiguration configuration = new RemoteConfiguration(
                        "Registry@PETALSESB-" + containerConfiguration.getName());
                configuration.setMessageSenderClassName(this.localConfig.getMessageSenderClassName());

                // TO be updated when new transport will be available
                String url = "http://" + containerConfiguration.getHost() + ":" + containerConfiguration.getRegistryPort();
                configuration.setUri(url);
                this.registry.getContext().getTopology().add(configuration);
            }
        }

        if (this.localContainerConfiguration.isPeer()) {
            this.registry.getContext().getProperties().put(PROPERTY_REGISTRY_MODE, PEER);
        }

        if (this.localContainerConfiguration.isMaster()) {
            this.registry.getContext().getProperties().put(PROPERTY_REGISTRY_MODE, MASTER);
        }

        if (this.localContainerConfiguration.isSlave()) {
            this.registry.getContext().getProperties().put(PROPERTY_REGISTRY_MODE, SLAVE);
        }

        if (this.localContainerConfiguration.isStandalone()) {
            this.registry.getContext().getProperties().put(PROPERTY_REGISTRY_MODE, STANDALONE);
        }
    }
    
    /**
     * @throws RegistryException
     */
    protected void createRegistryConfig() throws RegistryException {

        final Map<String, String> serverPropertiesMap = this.configurationService
                .getAllServerProperties();
        final Properties serverProperties = new Properties();
        for (final Map.Entry<String, String> property : serverPropertiesMap.entrySet()) {
            final String value = property.getValue();
            if (value != null) {
                serverProperties.setProperty(property.getKey(), value);
            }
        }
        final ConfigurationLoader loader = ConfigurationLoaderFactory.getPropertiesLoader();
        this.localConfig = loader.loadLocal(serverProperties);

        // update things...
        this.localConfig
                .setName("Registry@PEtALSESB-" + this.localContainerConfiguration.getName());
        this.localConfig.setRootPath(this.localContainerConfiguration.getWorkDirectoryPath());

        String receiverURL = "http://" + this.localContainerConfiguration.getHost() + ":"
                + this.localContainerConfiguration.getRegistryPort();
        this.localConfig.setMessageReceiverURL(receiverURL);
    }

    /**
     * 
     */
    private void cleanData() {
        // stop receiving things...
        this.registry.pauseReceive();

        // delete all the local entries
        try {
            this.registry.delete(this.getRootPath(), true);
        } catch (RegistryException e) {
            if (this.log.isErrorEnabled()) {
                this.log.error(e.getMessage());
            }
        }

        try {
            this.registry.clean();
        } catch (RegistryException e) {
            if (this.log.isErrorEnabled()) {
                this.log.error(e.getMessage());
            }
        }
        this.registry.resumeReceive();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {

        this.log.start();

        // stop receiving things...
        this.registry.pauseReceive();

        // and clean the remote data from the registry. This is because for now
        // there is no other solution to stay in sync with other registries when
        // this petals node come back alive... Remote data will be retrieved at
        // startup and runtime if needed next time this node will be up and
        // running...
        try {
            this.registry.cleanRemoteData();
        } catch (RegistryException e) {
            if (this.log.isErrorEnabled()) {
                this.log.error(e.getMessage());
            }
        }

        // delete all the local entries
        try {
            this.registry.delete(this.getRootPath(), true);
        } catch (RegistryException e) {
            if (this.log.isErrorEnabled()) {
                this.log.error(e.getMessage());
            }
        }

        // Note : Coming next, leave registry group and more...

        if (this.registryTopologyUpdater != null) {
            // this.configurationService.removeContainerConfiguration(localContainerConfiguration);
            this.registryTopologyUpdater.shutdownNow();
        }

        try {
            this.registry.stop();
        } catch (LifeCycleException e) {
            throw new org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryException(e);
        }

        this.log.end();
    }

    @Override
    protected final String getRootPath() {
        return "/endpoints/" + this.configurationService.getContainerConfiguration().getName()
                + "/";
    }

    /**
     * {@inheritDoc}
     */
    public List<RegistryListener> getListeners() {
        List<RegistryListener> result = null;
        if (this.listeners != null) {
            result = new ArrayList<RegistryListener>();
            for (Object o : this.listeners.values()) {
                if (o instanceof RegistryListener) {
                    result.add((RegistryListener) o);
                }
            }
        }
        return result;
    }

    class TopologyExceptionHandler implements Thread.UncaughtExceptionHandler {

        public void uncaughtException(Thread t, Throwable e) {
            DistributedEndpointRegistryImpl.this.log.error(e.getMessage()
                    + " : " + e.getCause());
        }

    }

    public void synchronizeRegistry() throws Exception {
        if (this.registry == null) {
            throw new Exception(
                    "Registry is null and can not be managed!");
        }

        if (this.localContainerConfiguration.isInMasterSlaveMode()
                && this.localContainerConfiguration.isMaster()) {
            throw new Exception(
                    "Data synchronization is not allowed on the master node");
        }

        try {
            this.registry.synchronizeData();
        } catch(RegistryException re) {
            throw new Exception(re);
        }
    }
}
