/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.jbi.component.Component;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.namespace.QName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Contingency;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.launcher.api.service.ServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.control.ExchangeCheckerClient;
import org.ow2.petals.microkernel.jbi.messaging.routing.module.endpoint.EndpointOrderer;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

/**
 * Router module to resolve address.
 * @author Christophe HAMERLING - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class EndpointResolverModule implements SenderModule {

    /**
     * the different kind of research of a endpoint: by service or by interface.
     */
    private static enum kindSearch {
        /**
         * the research by interface.
         */
        INTERFACE_SEARCH,

        /**
         * the research by service.
         */
        SERVICE_SEARCH
    }

    /**
     * the highest strategy.
     */
    public static final String HIGHEST = "highest";

    /**
     * Number of parameters defined in strategy.
     */
    public static final int NUMBER_STRATEGY_PARAMETERS = 4;

    /**
     * Constant used to specify the strategy to use.
     */
    public static final String PROPERTY_STRATEGY_PROTOCOLS = "org.ow2.petals.routing.strategy";

    /**
     * the load balancing (random strategy).
     */
    public static final String RANDOM = "random";

    /**
     * the strategy separator.
     */
    public static final String STRATEGY_SEPARATOR = ",";

    /**
     * the default configured strategy parameters.
     */
    private static List<Object> defaultStrategyParameters;

    /**
     * The Topology service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * the endpoint service.
     */
    @Requires(name = "endpoint")
    private EndpointRegistry endpointRegistry;

    /**
     * FIXME : Use it instaed of the registry!
     */
    @Requires(name = "checker")
    private ExchangeCheckerClient exchangeCheckerService;

    /**
     * Logger wrapper.
     */
    private LoggingUtil log;

    /**
     * The Topology service fractal component
     */
    @Requires(name = "topology", contingency = Contingency.OPTIONAL)
    private TopologyService topologyService;

    /**
     * The elected endpoint orderer
     */
    private EndpointOrderer endpointOrderer;

    /**
     * Analyze a strategy.
     * 
     * @param strategy
     * @return
     * @throws RoutingException
     */
    private final static List<Object> analyseStrategy(final String strategy)
            throws RoutingException {
        final List<Object> result = new ArrayList<Object>();
        final List<String> parameters = new ArrayList<String>();

        final StringTokenizer st = new StringTokenizer(strategy, STRATEGY_SEPARATOR);
        while (st.hasMoreTokens()) {
            parameters.add(st.nextToken().trim());
        }

        if (parameters.size() != NUMBER_STRATEGY_PARAMETERS) {
            throw new RoutingException("4 parameters are required for a routing strategy '"
                    + strategy + "'");
        }

        final String type = parameters.get(0).toLowerCase();
        final Float localWeighting;
        final Float remoteActiveWeighting;
        final Float remoteInactiveWeighting;
        try {
            localWeighting = Float.valueOf(parameters.get(1));
            remoteActiveWeighting = Float.valueOf(parameters.get(2));
            remoteInactiveWeighting = Float.valueOf(parameters.get(3));
        } catch (final NumberFormatException e) {
            throw new RoutingException("Impossible to convert a "
                    + "parameter to a float for the routing strategy '" + strategy + "'");
        }

        if ((!type.equals(HIGHEST)) && (!type.equals(RANDOM))) {
            throw new RoutingException("the " + type + " routing strategy is unknown");
        }
        if ((localWeighting < 0) || (remoteActiveWeighting < 0) || (remoteInactiveWeighting < 0)) {
            throw new RoutingException("Negative parameter is forbidden in routing strategy");
        }
        if (localWeighting == 0 && remoteActiveWeighting == 0 && remoteInactiveWeighting == 0) {
            throw new RoutingException("Weightings '0,0,0' is forbidden");
        }

        result.add(type);
        result.add(localWeighting);
        result.add(remoteActiveWeighting);
        result.add(remoteInactiveWeighting);

        return result;
    }

    /**
     * Get the strategy and analyze it. If the strategy is defined as message
     * property, it is used. Otherwise the default configured strategy is used.
     * 
     * @param exchange
     *            the message exchange in which the strategy to use can be
     *            provided. Must be non-null
     * @throws RoutingException
     *             impossible to create the endpoint chooser strategy
     * @return the list of parameters for the strategy
     */
    private final static List<Object> getStrategy(final MessageExchange exchange)
            throws RoutingException {

        final String strategy = (String) exchange.getProperty(PROPERTY_STRATEGY_PROTOCOLS);
        final List<Object> result;
        if (strategy == null) {
            result = defaultStrategyParameters;
        } else {
            result = analyseStrategy(strategy);

        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map,
     * org.ow2.petals.jbi.component.context.ComponentContextImpl,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl)
     */
    public final void electEndpoints(
            final Map<ServiceEndpoint, TransportSendContext> electedEndpoints,
            final ComponentContext sourceComponentContext,
            final MessageExchangeWrapper exchangeDecorator) throws RoutingException {
        this.log.call();

        if (Role.CONSUMER.equals(exchangeDecorator.getRole())) {
            final List<ServiceEndpoint> endpoints = this.resolveEndpoints(sourceComponentContext
                    .getComponent(), exchangeDecorator);
            for (final ServiceEndpoint endpoint : endpoints) {
                electedEndpoints.put(endpoint, new TransportSendContext(endpoint.getLocation()));
            }
        } else {
            ServiceEndpoint serviceEndpoint = (ServiceEndpoint) exchangeDecorator.getConsumerEndpoint();
            electedEndpoints.put(serviceEndpoint, new TransportSendContext(
                    serviceEndpoint.getLocation()));
        }
    }

    /**
     * Start the Fractal component
     * 
     * @throws RoutingException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws RoutingException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_ENDPOINT_RESOLVER));
        this.log.start();

        final ContainerConfiguration containerConfiguration = this.configurationService
                .getContainerConfiguration();

        // TODO not very clean to initialize the static variable like that
        try {
            defaultStrategyParameters = analyseStrategy(containerConfiguration.getRouterStrategy());
        } catch (final RoutingException e) {
            this.log.error("Failed to start Endpoint Resolver module", e);
            throw e;
        }

        this.endpointOrderer = new EndpointOrderer(this.log, containerConfiguration,
                this.topologyService);

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws RoutingException {
        this.log.call();
    }

    /**
     * Return only the enabled endPoints among the potentials endpoints.
     * 
     * @param type
     *            the type of research (by service or interface)
     * @param consumer
     *            the consumer
     * @param givenName
     *            the given service or interface name
     * @param exchange
     *            the exchange
     * @return the list of valide endpoints
     * @throws RegistryException
     */
    private final List<ServiceEndpoint> getEnabledEndpoints(final kindSearch type,
            final Component consumer, final QName givenName, final MessageExchange exchange,
            final LinkType linkType) throws RegistryException {
        this.log.call();

        final ServiceEndpoint[] potentialEndpoints;
        if (type == kindSearch.SERVICE_SEARCH) {
            potentialEndpoints = this.endpointRegistry.getInternalEndpointsForService(givenName,
                    linkType);
        } else {
            potentialEndpoints = this.endpointRegistry.getInternalEndpointsForInterface(givenName,
                    linkType);
        }

        final List<ServiceEndpoint> enabledEndpoints = new ArrayList<ServiceEndpoint>();
        for (final org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint providerEP : potentialEndpoints) {
            if (this.isExchangeOK(consumer, providerEP, exchange)) {
                enabledEndpoints.add(providerEP);
            }
        }

        return enabledEndpoints;
    }

    /**
     * Get the real endpoint matching the given endpoint.
     * 
     * @param endpoint
     *            the endpoint in the message exchange. May be null
     * @return the targeted endpoint. May be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private final ServiceEndpoint getEndpointForEndpoint(final ServiceEndpoint endpoint,
            final LinkType linktype) throws RoutingException {
        this.log.call();

        ServiceEndpoint targetEndpoint = endpoint;

        if (endpoint.getType() == EndpointType.INTERNAL && linktype == LinkType.SOFT) {
            throw new RoutingException("The target endpoint '" + endpoint.getEndpointName()
                    + "' is not a SOFT link");
        } else if (endpoint.getType() == EndpointType.LINKED && linktype == LinkType.HARD) {
            throw new RoutingException("The target endpoint '" + endpoint.getEndpointName()
                    + "' is not an HARD link");
        } else if (endpoint.getType() != EndpointType.INTERNAL) {
            try {
                targetEndpoint = this.endpointRegistry.getEndpoint(endpoint.getServiceName(),
                        endpoint.getEndpointName());
            } catch (final RegistryException e) {
                throw new RoutingException(e);
            }
        }

        return targetEndpoint;
    }

    /**
     * Get the endpoints matching the given interface.
     * 
     * @param interfaceName
     *            the given interface Name in the message exchange. May be null
     * @param exchange
     *            the exchange.
     * @param component
     *            the component.
     * @return the targeted endpoint following the specific strategy used. May
     *         be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private final List<ServiceEndpoint> getEndpointsForInterface(final QName interfaceName,
            final MessageExchange exchange, final Component component, LinkType linkType)
            throws RoutingException {
        this.log.call();

        final List<ServiceEndpoint> retrievedEndpoints;
        try {
            retrievedEndpoints = this.getEnabledEndpoints(kindSearch.INTERFACE_SEARCH, component,
                    interfaceName, exchange, linkType);
        } catch (final RegistryException e) {
            throw new RoutingException(e);
        }

        final List<Object> strategyParameters = getStrategy(exchange);

        List<ServiceEndpoint> orderedEndpoints = null;
        if (retrievedEndpoints.size() == 0) {
            throw new RoutingException("No endpoint(s) matching the target interface '"
                    + interfaceName + "' for Message Exchange with id '" + exchange.getExchangeId()
                    + "'");
        } else if (retrievedEndpoints.size() == 1) {
            orderedEndpoints = retrievedEndpoints;
        } else {
            orderedEndpoints = this.endpointOrderer.orderEndpoints(retrievedEndpoints,
                    strategyParameters);
        }

        return orderedEndpoints;
    }

    /**
     * Get the endpoints matching the given service.
     * 
     * @param serviceName
     *            the given service Name in the message exchange.
     * @param exchange
     *            the exchange.
     * @param component
     *            the component.
     * @return the targeted endpoint following the specific strategy used. May
     *         be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private final List<ServiceEndpoint> getEndpointsForService(final QName serviceName,
            final MessageExchange exchange, final Component component, LinkType linkType)
            throws RoutingException {
        this.log.call();

        final List<ServiceEndpoint> retrievedEndpoints;
        try {
            retrievedEndpoints = this.getEnabledEndpoints(kindSearch.SERVICE_SEARCH, component,
                    serviceName, exchange, linkType);
        } catch (final RegistryException e) {
            throw new RoutingException(e);
        }

        List<ServiceEndpoint> orderedEndpoints = null;
        if (retrievedEndpoints.size() == 0) {
            throw new RoutingException("No endpoint(s) matching the target service '" + serviceName
                    + "' for Message Exchange with id '" + exchange.getExchangeId() + "'");
        } else if (retrievedEndpoints.size() == 1) {
            orderedEndpoints = retrievedEndpoints;
        } else {
            final List<Object> strategyParameters = getStrategy(exchange);
            orderedEndpoints = this.endpointOrderer.orderEndpoints(retrievedEndpoints,
                    strategyParameters);
        }

        return orderedEndpoints;
    }

    /**
     * Control that the exchange is possible. Verify if the consumer and
     * provider are agree to realize the exchange.
     * 
     * @param consumer
     *            the consumer
     * @param providerEP
     *            the provider
     * @param exchange
     *            the exchange
     * @return true if ok
     */
    private final boolean isExchangeOK(final Component consumer,
            final org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint providerEP,
            final MessageExchange exchange) {
        this.log.call();

        boolean result = true;
        if (this.configurationService.getContainerConfiguration().isExchangeValidation()) {
            if (!consumer.isExchangeWithProviderOkay(providerEP, exchange)
                    || !this.exchangeCheckerService.isExchangeWithConsumerOkayForComponent(
                            providerEP, exchange)) {
                result = false;
            }
        }

        return result;
    }

    /**
     * Resolve the address if necessary. Resolution consist in finding an
     * endpoint satisfying <code>interface</code> or <code>service</code>. the
     * source of the exchange and the exchange help the
     * <code>AddressResolver</code> to find the endpoint. The
     * <code>containerName</code> and <code>ComponentName</code> properties are
     * set. If <code>containerName</code> and <code>ComponentName</code> already
     * exist, do nothing. The check of acceptance from the two part is done here
     * (the Component method isExchangeWithXXXOkay()).
     * 
     * @param source
     *            the component context
     * @param exchange
     *            the exchange
     * @throws RoutingException
     *             no destination can be found, or problem accessing the
     *             registry
     */
    private final List<ServiceEndpoint> resolveEndpoints(final Component sourceComponent,
            final MessageExchange exchange) throws RoutingException {
        this.log.start();

        // get message exchange values
        final ServiceEndpoint givenEndpoint = (ServiceEndpoint) exchange.getEndpoint();
        final QName serviceName = exchange.getService();
        final QName interfaceName = exchange.getInterfaceName();

        // resolve the link type
        LinkType linkType = LinkType.STANDARD;
        if (exchange.getProperty(RouterService.PROPERTY_ROUTER_PROVIDER_LINKTYPE) != null) {
            linkType = LinkType.fromValue((String) exchange
                    .getProperty(RouterService.PROPERTY_ROUTER_PROVIDER_LINKTYPE));
        }

        // Construct an array of potential matching endpoints.
        // If the endpoint has been set in the messageExchange,
        // this array contains only one element, which is this endpoint.
        List<ServiceEndpoint> electedEndpoints = new ArrayList<ServiceEndpoint>();

        if (givenEndpoint != null) {
            // Case 1 : The endpoint is explicit
            final ServiceEndpoint targetEndpoint = this.getEndpointForEndpoint(givenEndpoint,
                    linkType);
            if (targetEndpoint != null) {
                electedEndpoints.add(targetEndpoint);
            }

        } else if (serviceName != null) {
            // Case 2 : The endpoint is implicit, the service name is set.
            electedEndpoints = this.getEndpointsForService(serviceName, exchange, sourceComponent,
                    linkType);
            if (interfaceName != null) {
                // control that the interface name matches
                Iterator<ServiceEndpoint> endpointsIterator = electedEndpoints.iterator();
                while (endpointsIterator.hasNext()) {
                    final ServiceEndpoint endpoint = endpointsIterator.next();
                    if (!endpoint.getInterfacesName().get(0).equals(interfaceName)) {
                        endpointsIterator.remove();
                    }
                }
            }
        } else if (interfaceName != null) {
            // Case 3 : The endpoint is implicit, the interface name is set.
            electedEndpoints = this.getEndpointsForInterface(interfaceName, exchange,
                    sourceComponent, linkType);
        }

        if (electedEndpoints.size() == 0) {
            throw new RoutingException(
                    "Failed to find an endpoint for the Message Exchange with id '"
                            + exchange.getExchangeId() + "'");
        }

        if (this.log.isDebugEnabled()) {
            this.log.debug("Elected endpoints:");
            for (final ServiceEndpoint electedEndpoint : electedEndpoints) {
                this.log.debug("\t - " + electedEndpoint.toString());
            }
        }

        this.log.end();
        return electedEndpoints;
    }

}
