/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.routing.module.flow.PriorityModifier;
import org.ow2.petals.microkernel.jbi.messaging.routing.monitoring.RouterMonitorFilter;
import org.ow2.petals.microkernel.jbi.messaging.routing.monitoring.RouterMonitorService;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;
import org.ow2.petals.regulation.RegulationBuilder;
import org.ow2.petals.regulation.RegulationException;
import org.ow2.petals.regulation.generated.Flow;
import org.ow2.petals.regulation.generated.Regulation;

/**
 * The Flow Controller Module.
 * <p>
 * The configured flows to control are activating in the Router Monitor. Then
 * the module calculates at each {@code RATE_CALCULATE_PERIOD} the effective
 * rate on the controlled flow.
 * </p>
 * <p>
 * It tries to regulates the flow according the target rate, by slowing down the
 * flow if necessary, using the priority Priority Queues.
 * </p>
 * 
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class FlowControllerModule implements SenderModule {

    public static final String REGULATION_CONFIGURATION_FILE = "regulation.xml";

    /**
     * The period to calculate the flow rates.
     */
    private static final long RATE_CALCULATE_PERIOD = 10000;

    /**
     * The period to update the exchange priority.
     */
    private static final long PRIORITY_UPDATE_PERIOD = 500;

    /**
     * The base delay to slow down the flow
     */
    private static final long EXCHANGE_DELAY = 300;

    /**
     * Logger wrapper.
     */
    private LoggingUtil log;

    /**
     * The Priority Modifier fractal component
     */
    @Requires(name = "prioritymodifier")
    private PriorityModifier priorityModifier;

    /**
     * The Router Monitor fractal component
     */
    @Requires(name = "routermonitor")
    private RouterMonitorService routerMonitorService;

    private Map<RouterMonitorFilter, _Rate> rates;

    private Map<Long, MessageExchangeWrapper> pendingExchanges;

    private Timer rateControlTimer;

    /**
     * Inner class to calculate the current flow rates.
     * 
     * @author rnaudin
     * 
     */
    private class RateCaculator extends TimerTask {

        @Override
        public void run() {

            final long currentTime = System.currentTimeMillis();

            for (RouterMonitorFilter routerMonitorFilter : FlowControllerModule.this.rates.keySet()) {
                try {
                    int exchangeFlows = FlowControllerModule.this.routerMonitorService
                            .getExchanges(routerMonitorFilter.getInterfaceName(),
                                    routerMonitorFilter.getServiceName(), routerMonitorFilter
                                            .getEndpointName(), routerMonitorFilter
                                            .getOperationName(), currentTime
                                            - RATE_CALCULATE_PERIOD, currentTime);
                    float currentRate = 0;
                    if (exchangeFlows != 0) {
                        currentRate = ((float) exchangeFlows * 1000f)
                                / (float) RATE_CALCULATE_PERIOD;
                    }
                    _Rate rate = FlowControllerModule.this.rates.get(routerMonitorFilter);

                    if (currentRate > rate.expectedRate) {
                        rate.diffRate += new Double(Math.exp(currentRate - rate.expectedRate))
                                .intValue();
                    } else if (currentRate < rate.expectedRate && rate.diffRate > 0) {
                        rate.diffRate -= new Double(Math.exp((rate.expectedRate - currentRate)))
                                .intValue();
                        if (rate.diffRate < 0) {
                            rate.diffRate = 0;
                        }
                    }

                    FlowControllerModule.this.log.debug("Flow rate for " + routerMonitorFilter
                            + " : " + currentRate);
                } catch (MessagingException e) {
                    FlowControllerModule.this.log.warning("Failed to calculate a flow rate for "
                            + routerMonitorFilter, e);
                }
            }
        }
    }

    /**
     * Inner class to update the priority of pending exchanges.
     * 
     * @author rnaudin
     * 
     */
    private class PriorityUpdater extends TimerTask {

        @Override
        public void run() {

            final long currentTime = System.currentTimeMillis();

            for (Long targetTime : FlowControllerModule.this.pendingExchanges.keySet()) {
                if (currentTime > targetTime) {
                    final MessageExchangeWrapper exchangeDecorator = FlowControllerModule.this.pendingExchanges
                            .remove(targetTime);
                    try {
                        FlowControllerModule.this.priorityModifier.modifyPriority(
                                exchangeDecorator, (short) 3);
                        FlowControllerModule.this.log
                                .debug("Reassign priority for exchange with Id '"
                                        + exchangeDecorator.getExchangeId() + "'");
                    } catch (RoutingException e) {
                        FlowControllerModule.this.log.warning(
                                "Failed to reassign a priority to the exchange with Id '"
                                        + exchangeDecorator.getExchangeId()
                                        + "', maybe a timeout has occurred", e);
                    }
                }
            }
        }
    }

    /**
     * Inner structure.
     * 
     * @author rnaudin
     * 
     */
    private class _Rate {

        public float expectedRate;

        public int diffRate;

        public _Rate(float expectedRate) {
            this.expectedRate = expectedRate;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map, org.ow2.petals.jbi.component.context.ComponentContext,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void electEndpoints(Map<ServiceEndpoint, TransportSendContext> electedEndpoints,
            ComponentContext sourceComponentContext, MessageExchangeWrapper exchangeDecorator)
            throws RoutingException {

        if (Role.CONSUMER.equals(exchangeDecorator.getRole())
                && exchangeDecorator.getMessage(MessageExchangeImpl.IN_MSG) != null) {
            for (RouterMonitorFilter routerMonitorFilter : FlowControllerModule.this.rates.keySet()) {
                String endpointName = null;
                QName interfaceName = exchangeDecorator.getInterfaceName();
                QName serviceName = exchangeDecorator.getService();
                QName operationName = exchangeDecorator.getOperation();
                if (exchangeDecorator.getEndpoint() != null) {
                    endpointName = exchangeDecorator.getEndpoint().getEndpointName();
                    interfaceName = exchangeDecorator.getEndpoint().getInterfaces()[0];
                    serviceName = exchangeDecorator.getEndpoint().getServiceName();
                }

                if ((routerMonitorFilter.getInterfaceName() == null || (interfaceName != null && interfaceName
                        .equals(routerMonitorFilter.getInterfaceName())))
                        && (routerMonitorFilter.getServiceName() == null || (serviceName != null && serviceName
                                .equals(routerMonitorFilter.getServiceName())))
                        && (routerMonitorFilter.getEndpointName() == null || (endpointName != null && endpointName
                                .equals(routerMonitorFilter.getEndpointName())))
                        && (routerMonitorFilter.getOperationName() == null || (operationName != null && operationName
                                .equals(routerMonitorFilter.getOperationName())))) {
                    _Rate rate = FlowControllerModule.this.rates.get(routerMonitorFilter);
                    if (rate.diffRate > 0) {
                        exchangeDecorator.setProperty(
                                PriorityOrdererModule.PROPERTY_ROUTER_PRIORITY, "0");
                        long delay = rate.diffRate * EXCHANGE_DELAY;
                        this.log.debug("Slow down the exchange with Id '"
                                + exchangeDecorator.getExchangeId() + "' of " + delay + "ms");
                        this.pendingExchanges.put(System.currentTimeMillis() + delay,
                                exchangeDecorator);
                    }
                }

            }
        }

    }

    /**
     * Start the Fractal component
     * 
     * @throws RegulationException
     * 
     * @throws RoutingException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws RegulationException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_FLOW_CONTROLLER));
        this.log.start();

        this.rates = new HashMap<RouterMonitorFilter, _Rate>();
        this.pendingExchanges = new ConcurrentHashMap<Long, MessageExchangeWrapper>();
        this.rateControlTimer = new Timer("Router - Flow controller module - Rate controller");

        try {
            if (this.loadRegulationConfiguration()) {
                for (RouterMonitorFilter routerMonitorFilter : this.rates.keySet()) {
                    try {
                        this.routerMonitorService.monitorExchanges(routerMonitorFilter
                                .getInterfaceName(), routerMonitorFilter.getServiceName(),
                                routerMonitorFilter.getEndpointName(), routerMonitorFilter
                                        .getOperationName(), false);
                    } catch (MessagingException e) {
                        this.log.warning(e.getMessage());
                    }
                }

                this.rateControlTimer.schedule(new RateCaculator(), RATE_CALCULATE_PERIOD,
                        RATE_CALCULATE_PERIOD);
                this.rateControlTimer.schedule(new PriorityUpdater(), RATE_CALCULATE_PERIOD,
                        PRIORITY_UPDATE_PERIOD);
            }
        } catch (RegulationException e) {
            this.log.error("Failed to start Priority Regulator service", e);
            throw e;
        }

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();

        this.rateControlTimer.cancel();
    }

    /**
     * load the Regulation configuration.
     * 
     * @throws RegulationException
     */
    private boolean loadRegulationConfiguration() throws RegulationException {
        boolean result = false;
        final URL regulationUrl = this.getClass().getResource("/" + REGULATION_CONFIGURATION_FILE);

        if (regulationUrl != null) {
            final Regulation regulation = RegulationBuilder.createRegulation(regulationUrl
                    .getFile());
            for (Flow flow : regulation.getFlow()) {
                RouterMonitorFilter routerMonitorFilter = new RouterMonitorFilter(flow
                        .getInterface(), flow.getService(), flow.getEndpoint(),
                        flow.getOperation(), false);
                if (flow.getRate() != null) {
                    this.rates.put(routerMonitorFilter, new _Rate(flow.getRate()));
                    this.log.info("Add a flow control on " + routerMonitorFilter + " with rate "
                            + flow.getRate());
                }
            }
            result = true;
        }

        return result;
    }
}
