/**
 * Copyright (c) 1999 Bull S.A., 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader.locator;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * A <code>DirLocator</code> is used to look up for a file inside a directory.
 * 
 * @author Guillaume Sauthier
 */
public class DirLocator extends LocatorAbstract {

    /**
     * Wrapped File where search will be performed.
     */
    private File file;

    /**
     * Construct a new DirLocator from an URL pointing to a directory.
     * 
     * @param jar
     *            URL pointing to a directory.
     * 
     * @throws IOException
     *             When
     */
    public DirLocator(URL jar) throws IOException {
        String fileName = jar.getFile();
        fileName = fileName.replace("%20", " ");
        file = new File(fileName);
        if (!file.exists()) {
            throw new IOException("File " + file + " does not exists.");
        }
        if (!file.isDirectory()) {
            throw new IOException("File " + file + " is not a directory.");
        }
    }

    /**
     * Returns true when directory was found.
     * 
     * @param path
     *            the path to the directory to look up
     * 
     * @return true when directory was found, otherwise false.
     */
    @Override
    public boolean hasDirectory(String path) {

        File child = new File(file, path);
        return child.exists() && child.isDirectory();
    }

    /**
     * Returns true when file was found.
     * 
     * @param path
     *            the path to the file to look up
     * 
     * @return true when file was found, otherwise false.
     */
    @Override
    public boolean hasFile(String path) {

        File child = new File(file, path);
        return child.exists() && child.isFile();
    }

    /**
     * Returns a list of filename stored in path.
     * 
     * @param path
     *            the path to the directory where looking for files
     * 
     * @return a list of filename stored in path.
     */
    @Override
    public List<String> listContent(String path) {

        File child = new File(file, path);
        List<String> libs = new ArrayList<String>();
        // List directory content
        if (child.isDirectory()) {
            addContent(child, libs);
        }

        return libs;
    }

    /**
     * Add only files in the given List. Recursive!
     * 
     * @param f
     *            base directory to explore
     * @param l
     *            file list to be filled
     */
    private void addContent(File f, List<String> l) {
        File[] childs = f.listFiles();
        if (childs != null) {
            for (int i = 0; i < childs.length; i++) {
                if (childs[i].isDirectory()) {
                    addContent(childs[i], l);
                } else if (childs[i].isFile()) {
                    l.add(childs[i].getPath().substring(file.getPath().length()));
                }
            }
        }
    }

}
