/**
 * Copyright (c) 1999 Bull S.A., 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader.locator;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

/**
 * A <code>JarfileLocator</code> is used to look up for a file inside a Jar
 * archive. Works only for local jars.
 * 
 * @author Guillaume Sauthier
 */
public class JarFileLocator extends LocatorAbstract {

    /**
     * Wrapped JarFile where searchs will be performed
     */
    private JarFile file;

    /**
     * Construct a new JarFileLocator from an URL representing a Jar.
     * 
     * @param jarUrl
     *            URL pointing to a Jar file
     * 
     * @throws IOException
     *             When URL is not a JarFile.
     * @throws URISyntaxException
     *             If the URL is invalid
     */
    public JarFileLocator(URL jarUrl) throws IOException, URISyntaxException {
        final File jarFile = new File(jarUrl.toURI());
        String filename = jarFile.getAbsolutePath();
        file = new JarFile(filename);
    }

    /**
     * Returns true when directory was found.
     * 
     * @param path
     *            the path to the directory to look up
     * 
     * @return true when directory was found, otherwise false.
     */
    @Override
    public boolean hasDirectory(String path) {
        boolean found = false;
        for (Enumeration<JarEntry> e = file.entries(); e.hasMoreElements() && !found;) {
            ZipEntry entry = (ZipEntry) e.nextElement();
            if (entry.getName().startsWith(path)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true when file was found.
     * 
     * @param path
     *            the path to the file to look up
     * 
     * @return true when file was found, otherwise false.
     */
    @Override
    public boolean hasFile(String path) {
        ZipEntry entry = file.getEntry(path);
        return entry != null;
    }

    /**
     * Returns a list of filename stored in path.
     * 
     * @param path
     *            the path to the directory where looking for files
     * 
     * @return a list of filename stored in path.
     */
    @Override
    public List<String> listContent(String path) {
        List<String> libs = new ArrayList<String>();
        for (Enumeration<JarEntry> e = file.entries(); e.hasMoreElements();) {
            ZipEntry entry = (ZipEntry) e.nextElement();
            if (entry.getName().startsWith(path)) {
                libs.add(entry.getName());
            }
        }
        return libs;
    }

}
