/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.persistence;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.dbcp.BasicDataSource;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.commons.Constants;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.system.persistence.PersistenceService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * The Persistence Service.
 * <p>
 * It is using a JDBC pool of connection to persist Petals data.
 * </p>
 * 
 * @author Roland Naudin - EBM WebSourcing
 * @since 3.1
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = PersistenceService.class))
public class PersistenceServiceImpl implements PersistenceService {

    private static final int UUID_LENGTH = new QualifiedUUIDGenerator(Constants.UUID_DOMAIN)
            .getUuidMaxLength();

    private class PersistenceCleaner extends TimerTask {

        @Override
        public void run() {
            PersistenceServiceImpl.this.log.debug("Start Persistence clean up");
            final long endTime = System.currentTimeMillis()
                    - PersistenceServiceImpl.this.storageDuration;
            try {
                final Connection connection = PersistenceServiceImpl.this.jdbcDataSource
                        .getConnection();
                try {
                    final PreparedStatement getStatement = connection
                            .prepareStatement("SELECT EXCHANGE_ID FROM MESSAGE_EXCHANGE WHERE CREATION_TIMESTAMP < ?");
                    getStatement.setFetchSize(PersistenceServiceImpl.this.fetchSize);
                    try {
                        getStatement.setLong(1, endTime);
                        final ResultSet resultSet = getStatement.executeQuery();
                        try {
                            while (resultSet.next()) {
                                final String exchangeId = resultSet.getString(1);

                                log.debug("Delete Message Exchange with Id '" + exchangeId
                                        + "' related data");
                                {
                                    final PreparedStatement deleteStatement = connection
                                            .prepareStatement("DELETE FROM MESSAGE_EXCHANGE_PROPERTY WHERE EXCHANGE_ID=?");
                                    try {
                                        deleteStatement.setString(1, exchangeId);
                                        deleteStatement.execute();
                                    } finally {
                                        deleteStatement.close();
                                    }
                                }
                                {
                                    final PreparedStatement deleteStatement = connection
                                            .prepareStatement("DELETE FROM NORMALIZED_MESSAGE WHERE EXCHANGE_ID=?");
                                    try {
                                        deleteStatement.setString(1, exchangeId);
                                        deleteStatement.execute();
                                    } finally {
                                        deleteStatement.close();
                                    }
                                }
                                {
                                    final PreparedStatement deleteStatement = connection
                                            .prepareStatement("DELETE FROM EXCHANGE_TIMESTAMP WHERE EXCHANGE_ID=?");
                                    try {
                                        deleteStatement.setString(1, exchangeId);
                                        deleteStatement.execute();
                                    } finally {
                                        deleteStatement.close();
                                    }
                                }
                                {
                                    final PreparedStatement deleteStatement = connection
                                            .prepareStatement("DELETE FROM MESSAGE_EXCHANGE WHERE EXCHANGE_ID=?");
                                    try {
                                        deleteStatement.setString(1, exchangeId);
                                        deleteStatement.execute();
                                    } finally {
                                        deleteStatement.close();
                                    }
                                }
                            }
                        } finally {
                            resultSet.close();
                        }
                    } finally {
                        getStatement.close();
                    }
                } finally {
                    connection.close();
                }
            } catch (final SQLException e) {
                PersistenceServiceImpl.this.log.warning("Failed to delete old exchange data", e);
            }
        }
    }

    private int fetchSize;

    private Timer cleanerTimer;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    private String databaseDriver;

    /**
     * The data source to handle JDBC connections pooling
     */
    private BasicDataSource jdbcDataSource;

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    private long storageDuration;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#addMessageExchange
     * (org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, boolean,
     * java.lang.Boolean)
     */
    public void addMessageExchange(final MessageExchangeImpl exchange, final boolean storeExchange,
            final Boolean isSent) throws SQLException, IOException {
        this.log.call("ExchangeId : " + exchange.getExchangeId());
        final long currentTime = System.currentTimeMillis();
        final Connection connection = jdbcDataSource.getConnection();
        try {
            final PreparedStatement storeStatement;
            if (storeExchange) {
                storeStatement = connection
                        .prepareStatement("INSERT INTO MESSAGE_EXCHANGE VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
            } else {
                storeStatement = connection
                        .prepareStatement("INSERT INTO MESSAGE_EXCHANGE (EXCHANGE_ID, CONSUMER_ENDPOINT, ENDPOINT, INTERFACE_NAME, SERVICE_NAME, ENDPOINT_NAME, OPERATION_NAME, PATTERN, CREATION_TIMESTAMP) VALUES(?,?,?,?,?,?,?,?,?)");
            }
            storeStatement.setString(1, exchange.getExchangeId());
            storeStatement.setObject(2, exchange.getConsumerEndpoint());
            storeStatement.setObject(3, exchange.getServiceEndpoint());
            if (exchange.getInterfaceName() == null) {
                storeStatement.setString(4, null);
            } else {
                storeStatement.setString(4, exchange.getInterfaceName().toString());
            }
            if (exchange.getService() == null) {
                storeStatement.setString(5, null);
            } else {
                storeStatement.setString(5, exchange.getService().toString());
            }
            if (exchange.getServiceEndpoint() == null) {
                storeStatement.setString(6, null);
            } else {
                storeStatement.setString(6, exchange.getServiceEndpoint().getEndpointName());
            }
            if (exchange.getOperation() == null) {
                storeStatement.setString(7, null);
            } else {
                storeStatement.setString(7, exchange.getOperation().toString());
            }
            String mepString = null;
            // TODO remove that part when the deprecated MEP constant
            // would be removed from code
            if (MessageExchangeImpl.IN_ONLY_PATTERN.equals(exchange.getPattern())) {
                mepString = AbsItfOperation.MEPPatternConstants.IN_ONLY.toString();
            } else if (MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN.equals(exchange.getPattern())) {
                mepString = AbsItfOperation.MEPPatternConstants.ROBUST_IN_ONLY.toString();
            } else if (MessageExchangeImpl.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                mepString = AbsItfOperation.MEPPatternConstants.IN_OUT.toString();
            } else if (MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN.equals(exchange.getPattern())) {
                mepString = AbsItfOperation.MEPPatternConstants.IN_OPTIONAL_OUT.toString();
            }
            storeStatement.setString(8, mepString);
            if (storeExchange) {
                storeStatement.setObject(9, exchange.getError());
                if (Role.CONSUMER.equals(exchange.getRole())) {
                    storeStatement.setShort(10, MessageExchangeImpl.SERIALIZE_ROLE_CONSUMER);
                } else {
                    storeStatement.setShort(10, MessageExchangeImpl.SERIALIZE_ROLE_PROVIDER);
                }
                if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                    storeStatement.setShort(11, MessageExchangeImpl.SERIALIZE_STATUS_ACTIVE);
                } else if (ExchangeStatus.DONE.equals(exchange.getStatus())) {
                    storeStatement.setShort(11, MessageExchangeImpl.SERIALIZE_STATUS_DONE);
                } else {
                    storeStatement.setShort(11, MessageExchangeImpl.SERIALIZE_STATUS_ERROR);
                }
                storeStatement.setBoolean(12, exchange.isTerminated());
                storeStatement.setBoolean(13, exchange.isTransacted());
                storeStatement.setBoolean(14, exchange.isMonitored());
                storeStatement.setBoolean(15, exchange.isPersisted());
                storeStatement.setLong(16, currentTime);
            } else {
                storeStatement.setLong(9, currentTime);
            }

            try {
                storeStatement.addBatch();
                storeStatement.executeBatch();
            } finally {
                storeStatement.close();
            }

            if (storeExchange) {
                this.insertMessageExchangeProperties(connection, exchange);
                this.insertNormalizedMessages(connection, exchange);
            }

            if (isSent != null) {
                final String type;
                if (exchange.getMessage(MessageExchangeImpl.IN_MSG) != null) {
                    type = MessageExchangeImpl.IN_MSG;
                } else if (exchange.getMessage(MessageExchangeImpl.OUT_MSG) != null) {
                    type = MessageExchangeImpl.OUT_MSG;
                } else if (exchange.getMessage(MessageExchangeImpl.FAULT_MSG) != null) {
                    type = MessageExchangeImpl.FAULT_MSG;
                } else if (exchange.getError() != null) {
                    type = "error-ack";
                } else {
                    type = "done-ack";
                }

                final PreparedStatement addStatement = connection
                        .prepareStatement("INSERT INTO EXCHANGE_TIMESTAMP (EVENT_TIMESTAMP, EXCHANGE_ID, IS_SENT, TYPE) VALUES(?, ?, ?, ?)");
                try {
                    addStatement.setLong(1, currentTime);
                    addStatement.setString(2, exchange.getExchangeId());
                    if (ORACLE_DRIVER.equals(databaseDriver)) {
                        addStatement.setShort(3, isSent ? (short) 1 : (short) 0);
                    } else {
                        addStatement.setBoolean(3, isSent);
                    }
                    addStatement.setString(4, type);

                    addStatement.execute();
                } finally {
                    addStatement.close();
                }
            }
        } finally {
            connection.close();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#clearExchangeStorage
     * ()
     */
    public void clearExchangeStorage() {
        long _storageDuration = this.storageDuration;
        this.storageDuration = 0;
        new PersistenceCleaner().run();
        if (this.storageDuration == 0) {
            // no concurrent modification, reset the old value
            this.storageDuration = _storageDuration;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#deleteMessageExchange
     * (java.lang.String)
     */
    public void deleteMessageExchange(final String exchangeId) throws SQLException {
        this.log.call("ExchangeId : " + exchangeId);
        this.deleteMessageExchangeFromTable("MESSAGE_EXCHANGE_PROPERTY", exchangeId);
        this.deleteMessageExchangeFromTable("NORMALIZED_MESSAGE", exchangeId);
        this.deleteMessageExchangeFromTable("EXCHANGE_TIMESTAMP", exchangeId);
        // respect the constraints :do not delete the exchange before its
        // messages, properties and timestamp
        this.deleteMessageExchangeFromTable("MESSAGE_EXCHANGE", exchangeId);
    }

    private void deleteMessageExchangeFromTable(final String table, final String exchangeId)
            throws SQLException {
        final Connection connection = jdbcDataSource.getConnection();
        try {
            final PreparedStatement deleteStatement = connection.prepareStatement("DELETE FROM "
                    + table + " WHERE EXCHANGE_ID=?");
            deleteStatement.setString(1, exchangeId);
            try {
                deleteStatement.execute();
            } finally {
                deleteStatement.close();
            }
        } finally {
            connection.close();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.persistence.PersistenceService#
     * getExchangeStorageDuration()
     */
    public long getExchangeStorageDuration() {
        return this.storageDuration;
    }

    private final MessageExchangeImpl createMessageExchange(String exchangeId,
            org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint consumerEndpoint,
            org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint endpoint,
            QName interfaceName,
            QName service, QName operation, String pattern, Map<String, Object> properties,
            Map<String, NormalizedMessage> messages, Exception error, short role, short status,
            boolean terminated, boolean transacted, boolean monitored, boolean persisted) {
        MessageExchangeImpl me = new MessageExchangeImpl();
        me.setExchangeId(exchangeId);
        me.setConsumerEndpoint(consumerEndpoint);
        me.setEndpoint(endpoint);
        me.setInterfaceName(interfaceName);
        me.setService(service);
        me.setOperation(operation);
        try {
            me.setPattern(new URI(pattern));
        } catch (URISyntaxException e) {
            throw new UncheckedException(e);
        }
        for (Map.Entry<String, Object> entry : properties.entrySet()) {
            me.setProperty(entry.getKey(), entry.getValue());
        }
        for (Map.Entry<String, NormalizedMessage> entry : messages.entrySet()) {
            try {
                me.setMessage(entry.getValue(), entry.getKey());
            } catch (MessagingException e) {;
                throw new UncheckedException(e);
            }
        }
        me.setError(error);
        if (role == MessageExchangeImpl.SERIALIZE_ROLE_CONSUMER) {
            me.setRole(Role.CONSUMER);
        } else {
            me.setRole(Role.PROVIDER);
        }
        try {
            if (status == MessageExchangeImpl.SERIALIZE_STATUS_ACTIVE) {
                me.setStatus(ExchangeStatus.ACTIVE);
            } else if (status == MessageExchangeImpl.SERIALIZE_STATUS_DONE) {
                me.setStatus(ExchangeStatus.DONE);
            } else {
                me.setStatus(ExchangeStatus.ERROR);
            }
        } catch (MessagingException e) {
            throw new UncheckedException(e);
        }

        me.setTerminated(terminated);
        me.setTransacted(transacted);
        me.setMonitored(monitored);
        me.setPersisted(persisted);
        return me;
    }

    public MessageExchangeImpl getMessageExchange(final String exchangeId) throws SQLException,
            IOException, ClassNotFoundException {
        this.log.call("ExchangeId : " + exchangeId);
        MessageExchangeImpl result;
        final Connection connection = jdbcDataSource.getConnection();

        try {

            final PreparedStatement getStatement = connection
                    .prepareStatement("SELECT CONSUMER_ENDPOINT, ENDPOINT, INTERFACE_NAME, SERVICE_NAME, OPERATION_NAME, PATTERN, ERROR, ROLE, STATUS, TTERMINATED, TRANSACTED, MONITORED, PERSISTED FROM MESSAGE_EXCHANGE WHERE EXCHANGE_ID=?");

            try {
                getStatement.setFetchSize(this.fetchSize);
                getStatement.setString(1, exchangeId);

                ResultSet resultSet = getStatement.executeQuery();
                if (resultSet.next()) {
                    final ServiceEndpoint consumerEndpoint;
                    final ServiceEndpoint endpoint;
                    final QName interfaceName;
                    final QName service;
                    final QName operation;
                    final String pattern;
                    final Exception error;
                    final short role;
                    final short status;
                    final boolean terminated;
                    final boolean transacted;
                    final boolean monitored;
                    final boolean persisted;
                    try {
                        if (HSQLDB_DRIVER.equals(databaseDriver)) {
                            consumerEndpoint = (ServiceEndpoint) resultSet.getObject(1);
                        } else {
                            consumerEndpoint = (ServiceEndpoint) PersistenceServiceImpl
                                    .convertInpustStreamToObject(resultSet.getBinaryStream(1));
                        }

                        if (HSQLDB_DRIVER.equals(databaseDriver)) {
                            endpoint = (ServiceEndpoint) resultSet.getObject(2);
                        } else {
                            endpoint = (ServiceEndpoint) PersistenceServiceImpl
                                    .convertInpustStreamToObject(resultSet.getBinaryStream(2));
                        }

                        final String interfaceNameAsString = resultSet.getString(3);
                        if (interfaceNameAsString != null) {
                            interfaceName = QName.valueOf(interfaceNameAsString);
                        } else {
                            interfaceName = null;
                        }
                        final String serviceAsString = resultSet.getString(4);
                        if (serviceAsString != null) {
                            service = QName.valueOf(serviceAsString);
                        } else {
                            service = null;
                        }
                        final String operationAsString = resultSet.getString(5);
                        if (operationAsString != null) {
                            operation = QName.valueOf(operationAsString);
                        } else {
                            operation = null;
                        }
                        pattern = resultSet.getString(6);
                        if (HSQLDB_DRIVER.equals(databaseDriver)) {
                            error = (Exception) resultSet.getObject(7);
                        } else {
                            error = (Exception) PersistenceServiceImpl
                                    .convertInpustStreamToObject(resultSet.getBinaryStream(7));
                        }
                        role = resultSet.getShort(8);
                        status = resultSet.getShort(9);
                        terminated = resultSet.getBoolean(10);
                        transacted = resultSet.getBoolean(11);
                        monitored = resultSet.getBoolean(12);
                        persisted = resultSet.getBoolean(13);
                    } finally {
                        resultSet.close();
                    }
                    
                    final Map<String, Object> properties = this.getMessageExchangeProperties(
                            connection, exchangeId);
                    final Map<String, NormalizedMessage> messages = this.getNormalizedMessages(
                            connection, exchangeId);

                    result = createMessageExchange(exchangeId, consumerEndpoint, endpoint,
                            interfaceName, service, operation, pattern, properties, messages,
                            error, role, status, terminated, transacted, monitored, persisted);

                } else {
                    result = null;
                }
                return result;
            } finally {
                getStatement.close();
            }
        } finally {
            connection.close();
        }
    }

    private static Object convertInpustStreamToObject(final InputStream sqlInputStream)
            throws IOException, ClassNotFoundException {
        final Object result;
        if (sqlInputStream != null) {
            final ObjectInputStream objectInputStream = new ObjectInputStream(sqlInputStream);
            result = objectInputStream.readObject();
            objectInputStream.close();
        } else {
            result = null;
        }

        return result;
    }

    private Map<String, Object> getMessageExchangeProperties(final Connection connection,
            final String exchangeId) throws SQLException, IOException, ClassNotFoundException {
        final PreparedStatement getStatementFromMexP = connection
                .prepareStatement("SELECT KKEY, VALUE FROM MESSAGE_EXCHANGE_PROPERTY WHERE EXCHANGE_ID=?");
        final Map<String, Object> properties = new HashMap<String, Object>();
        try {
            getStatementFromMexP.setString(1, exchangeId);
            final ResultSet resultSet = getStatementFromMexP.executeQuery();
            try {
                while (resultSet.next()) {
                    if (HSQLDB_DRIVER.equals(databaseDriver)) {
                        properties.put(resultSet.getString(1), resultSet.getObject(2));
                    } else {
                        properties.put(resultSet.getString(1), PersistenceServiceImpl
                                .convertInpustStreamToObject(resultSet.getBinaryStream(2)));
                    }
                }
            } finally {
                resultSet.close();
            }
        } finally {
            getStatementFromMexP.close();
        }

        return properties;
    }

    private Map<String, NormalizedMessage> getNormalizedMessages(final Connection connection,
            final String exchangeId) throws SQLException, IOException, ClassNotFoundException {
        final PreparedStatement getStatementFromNMsg = connection
                .prepareStatement("SELECT KKEY, VALUE FROM NORMALIZED_MESSAGE WHERE EXCHANGE_ID=?");
        final Map<String, NormalizedMessage> messages = new HashMap<String, NormalizedMessage>();
        try {
            getStatementFromNMsg.setString(1, exchangeId);
            final ResultSet resultSet = getStatementFromNMsg.executeQuery();
            try {
                while (resultSet.next()) {
                    if (HSQLDB_DRIVER.equals(databaseDriver)) {
                        messages.put(resultSet.getString(1),
                                (NormalizedMessage) resultSet.getObject(2));
                    } else {
                        messages.put(resultSet.getString(1),
                                (NormalizedMessage) PersistenceServiceImpl
                                        .convertInpustStreamToObject(resultSet.getBinaryStream(2)));
                    }
                }
            } finally {
                resultSet.close();
            }
        } finally {
            getStatementFromNMsg.close();
        }

        return messages;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.system.persistence.PersistenceService#
     * getMessageExchangeHistory(java.lang.String)
     */
    public Map<String, Long> getMessageExchangeHistory(final String exchangeId) throws SQLException {
        log.call("ExchangeId : " + exchangeId);
        Map<String, Long> result = null;
        final Connection connection = jdbcDataSource.getConnection();
        try {
            final PreparedStatement getStatement = connection
                    .prepareStatement("SELECT EVENT_TIMESTAMP,IS_SENT,TYPE FROM EXCHANGE_TIMESTAMP WHERE EXCHANGE_ID=?");
            try {
                getStatement.setFetchSize(this.fetchSize);
                getStatement.setString(1, exchangeId);
                final ResultSet resultSet = getStatement.executeQuery();
                try {
                    String suffix;
                    while (resultSet.next()) {
                        if (result == null) {
                            result = new HashMap<String, Long>();
                        }
                        if (resultSet.getBoolean(2)) {
                            suffix = "sent";
                        } else {
                            suffix = "received";
                        }
                        result.put(resultSet.getString(3) + "-" + suffix, resultSet.getLong(1));
                    }
                } finally {
                    resultSet.close();
                }
            } finally {
                getStatement.close();
            }
        } finally {
            connection.close();
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#getMessageExchangeIds
     * (javax.xml.namespace.QName, javax.xml.namespace.QName, java.lang.String,
     * javax.xml.namespace.QName, long, long)
     */
    public List<String> getMessageExchangeIds(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws SQLException {
        this.log.call();
        final Connection connection = jdbcDataSource.getConnection();
        try {
            final StringBuilder conditionStatementBuilder = new StringBuilder();
            short colIdx = 1;
            short itfColIdx = 0;
            if (interfaceName != null) {
                conditionStatementBuilder.append("INTERFACE_NAME=? AND ");
                itfColIdx = colIdx++;
            }
            short svcColIdx = 0;
            if (serviceName != null) {
                conditionStatementBuilder.append("SERVICE_NAME=? AND ");
                svcColIdx = colIdx++;
            }
            short epColIdx = 0;
            if (endpointName != null) {
                conditionStatementBuilder.append("ENDPOINT_NAME=? AND ");
                epColIdx = colIdx++;
            }
            short opColIdx = 0;
            if (operationName != null) {
                conditionStatementBuilder.append("OPERATION_NAME=? AND ");
                opColIdx = colIdx++;
            }

            final PreparedStatement getStatement = connection
                    .prepareStatement("SELECT DISTINCT MESSAGE_EXCHANGE.EXCHANGE_ID FROM MESSAGE_EXCHANGE,EXCHANGE_TIMESTAMP WHERE "
                            + conditionStatementBuilder.toString()
                            + "EXCHANGE_TIMESTAMP.EXCHANGE_ID=MESSAGE_EXCHANGE.EXCHANGE_ID AND EXCHANGE_TIMESTAMP.EVENT_TIMESTAMP BETWEEN ? AND ?");

            getStatement.setFetchSize(this.fetchSize);

            if (interfaceName != null) {
                getStatement.setString(itfColIdx, interfaceName.toString());
            }
            if (serviceName != null) {
                getStatement.setString(svcColIdx, serviceName.toString());
            }
            if (endpointName != null) {
                getStatement.setString(epColIdx, endpointName);
            }
            if (operationName != null) {
                getStatement.setString(opColIdx, operationName.toString());
            }
            getStatement.setLong(colIdx++, startDate);
            getStatement.setLong(colIdx, endDate);

            try {
                final ResultSet resultSet = getStatement.executeQuery();
                final List<String> result = new ArrayList<String>();
                try {
                    while (resultSet.next()) {
                        result.add(resultSet.getString(1));
                    }
                    return result;
                } finally {
                    resultSet.close();
                }
            } finally {
                getStatement.close();
            }
        } finally {
            connection.close();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#getMessageExchangeIds
     * (java.lang.String, java.lang.String)
     */
    public List<String> getMessageExchangeIds(String propertyKey, String propertyValue)
            throws SQLException, IOException {
        this.log.call();
        ResultSet resultSet = null;
        Connection connection = null;
        PreparedStatement getStatement = null;
        final List<String> result = new ArrayList<String>();
        try {
            connection = this.jdbcDataSource.getConnection();
            getStatement = connection
                    .prepareStatement("SELECT EXCHANGE_ID FROM MESSAGE_EXCHANGE_PROPERTY WHERE KKEY='"
                            + propertyKey + "' AND VALUE LIKE ?");
            
            if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                getStatement.setObject(1, propertyValue);
            } else {
                // With MYSQL, the setObject() method does serialize not
                // properly primitive alter-ego Objects, use Streams
                // instead
                final ByteArrayOutputStream bytesOutputStream = new ByteArrayOutputStream();
                new ObjectOutputStream(bytesOutputStream).writeUnshared(propertyValue);
                final ByteArrayInputStream bytesInputStream = new ByteArrayInputStream(
                        bytesOutputStream.toByteArray());
                getStatement.setBinaryStream(1, bytesInputStream, bytesInputStream.available());
            }
            getStatement.setFetchSize(this.fetchSize);
            resultSet = getStatement.executeQuery();
            while (resultSet.next()) {
                result.add(resultSet.getString(1));
            }
        } finally {
            if (resultSet != null) {
                resultSet.close();
            }
            if (getStatement != null) {
                getStatement.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#getMessageExchanges
     * (javax.xml.namespace.QName, javax.xml.namespace.QName, java.lang.String,
     * javax.xml.namespace.QName, long, long)
     */
    public int getMessageExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws SQLException {
        this.log.call();
        final Connection connection = jdbcDataSource.getConnection();
        try {
            final StringBuilder conditionStatementBuilder = new StringBuilder();

            short colIdx = 1;
            short itfColIdx = 0;
            if (interfaceName != null) {
                conditionStatementBuilder.append("MESSAGE_EXCHANGE.INTERFACE_NAME=? AND ");
                itfColIdx = colIdx++;
            }
            short svcColIdx = 0;
            if (serviceName != null) {
                conditionStatementBuilder.append("MESSAGE_EXCHANGE.SERVICE_NAME=? AND ");
                svcColIdx = colIdx++;
            }
            short epColIdx = 0;
            if (endpointName != null) {
                conditionStatementBuilder.append("MESSAGE_EXCHANGE.ENDPOINT_NAME=? AND ");
                epColIdx = colIdx++;
            }
            short opColIdx = 0;
            if (operationName != null) {
                conditionStatementBuilder.append("MESSAGE_EXCHANGE.OPERATION_NAME=? AND ");
                opColIdx = colIdx++;
            }

            final PreparedStatement getStatement = connection
                    .prepareStatement("SELECT COUNT(DISTINCT MESSAGE_EXCHANGE.EXCHANGE_ID) FROM MESSAGE_EXCHANGE,EXCHANGE_TIMESTAMP WHERE "
                            + conditionStatementBuilder.toString()
                            + " MESSAGE_EXCHANGE.EXCHANGE_ID=EXCHANGE_TIMESTAMP.EXCHANGE_ID AND EXCHANGE_TIMESTAMP.EVENT_TIMESTAMP BETWEEN ? AND ?");

            getStatement.setFetchSize(this.fetchSize);
            if (interfaceName != null) {
                getStatement.setString(itfColIdx, interfaceName.toString());
            }
            if (serviceName != null) {
                getStatement.setString(svcColIdx, serviceName.toString());
            }
            if (endpointName != null) {
                getStatement.setString(epColIdx, endpointName);
            }
            if (operationName != null) {
                getStatement.setString(opColIdx, operationName.toString());
            }
            getStatement.setLong(colIdx++, startDate);
            getStatement.setLong(colIdx, endDate);

            try {
                final ResultSet resultSet = getStatement.executeQuery();
                try {
                    final int result;
                    if (resultSet.next()) {
                        result = resultSet.getInt(1);
                    } else {
                        result = 0;
                    }
                    return result;
                } finally {
                    resultSet.close();
                }
            } finally {
                getStatement.close();
            }
        } finally {
            connection.close();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.persistence.PersistenceService#
     * setExchangeStorageDuration(long)
     */
    public void setExchangeStorageDuration(final long duration) {
        this.storageDuration = duration;
        this.cleanerTimer.cancel();
        this.allocateCleanerTimer();
        this.cleanerTimer.schedule(new PersistenceCleaner(), 0, this.storageDuration);
        this.log.info("Persistence storage duration reset to " + this.storageDuration
                + " milliseconds");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.persistence.PersistenceService#updateMessageExchange
     * (org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, boolean,
     * java.lang.Boolean)
     */
    public void updateMessageExchange(final MessageExchangeImpl exchange,
            final boolean storeExchange, final Boolean isSent) throws SQLException, IOException {
        this.log.call("ExchangeId : " + exchange.getExchangeId());
        final Connection connection = jdbcDataSource.getConnection();
        try {
            if (!storeExchange) {
                if (!ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                    this.updateMessageExchangeStatus(connection, exchange);
                }
            } else {
                this.updateMessageExchange(connection, exchange);
                this.insertMessageExchangeProperties(connection, exchange);
                this.insertNormalizedMessages(connection, exchange);
            }
            if (isSent != null) {
                short isSentbinary = 0;
                if (isSent) {
                    isSentbinary = 1;
                }
                final String type;
                if (exchange.getMessage(MessageExchangeImpl.IN_MSG) != null) {
                    type = MessageExchangeImpl.IN_MSG;
                } else if (exchange.getMessage(MessageExchangeImpl.OUT_MSG) != null) {
                    type = MessageExchangeImpl.OUT_MSG;
                } else if (exchange.getMessage(MessageExchangeImpl.FAULT_MSG) != null) {
                    type = MessageExchangeImpl.FAULT_MSG;
                } else if (exchange.getError() != null) {
                    type = "error-ack";
                } else {
                    type = "done-ack";
                }
                String isSentValue = null;
                /*
                 * If Hsqldb use 'true'/'false' values for is_sent column, else
                 * use '1'/'0' values
                 */
                if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                    if (isSentbinary == 0) {
                        isSentValue = "false";
                    } else {
                        isSentValue = "true";
                    }
                } else {
                    isSentValue = String.valueOf(isSentbinary);
                }
                final PreparedStatement addStatement = connection
                        .prepareStatement("INSERT INTO EXCHANGE_TIMESTAMP (EVENT_TIMESTAMP, EXCHANGE_ID, IS_SENT, TYPE) VALUES(?,?,?,?)");
                try {
                    addStatement.setLong(1, System.currentTimeMillis());
                    addStatement.setString(2, exchange.getExchangeId());
                    addStatement.setString(3, isSentValue);
                    addStatement.setString(4, type);

                    // TODO: Manage the returned value
                    addStatement.execute();
                } finally {
                    addStatement.close();
                }
            }
        } finally {
            connection.close();
        }
    }

    private void updateMessageExchangeStatus(final Connection connection,
            final MessageExchangeImpl exchange) throws SQLException {
        final short status;
        if (ExchangeStatus.DONE.equals(exchange.getStatus())) {
            status = MessageExchangeImpl.SERIALIZE_STATUS_DONE;
        } else if (ExchangeStatus.ERROR.equals(exchange.getStatus())) {
            status = MessageExchangeImpl.SERIALIZE_STATUS_ERROR;
        } else {
            status = MessageExchangeImpl.SERIALIZE_STATUS_ACTIVE;
        }
        final PreparedStatement updateStatement = connection
                .prepareStatement("UPDATE MESSAGE_EXCHANGE SET STATUS=? WHERE EXCHANGE_ID=?");
        updateStatement.setShort(1, status);
        updateStatement.setString(2, exchange.getExchangeId());
        try {
            // TODO: manage the returned value
            updateStatement.execute();
        } finally {
            updateStatement.close();
        }
    }

    private void updateMessageExchange(final Connection connection,
            final MessageExchangeImpl exchange) throws SQLException {
        final PreparedStatement storeStatement = connection
                .prepareStatement("UPDATE MESSAGE_EXCHANGE SET CONSUMER_ENDPOINT=?, ENDPOINT=?, INTERFACE_NAME=?, SERVICE_NAME=?, ENDPOINT_NAME=?, ERROR=?, ROLE=?, STATUS=?, TTERMINATED=?, TRANSACTED=?, MONITORED=?, PERSISTED=? WHERE EXCHANGE_ID=?");
        try {
            storeStatement.setObject(1, exchange.getConsumerEndpoint());
            storeStatement.setObject(2, exchange.getServiceEndpoint());
            storeStatement.setString(3, exchange.getInterfaceName().toString());
            storeStatement.setString(4, exchange.getService().toString());
            storeStatement.setString(5, exchange.getServiceEndpoint().getEndpointName());
            storeStatement.setObject(6, exchange.getError());
            if (Role.CONSUMER.equals(exchange.getRole())) {
                storeStatement.setShort(7, MessageExchangeImpl.SERIALIZE_ROLE_CONSUMER);
            } else {
                storeStatement.setShort(7, MessageExchangeImpl.SERIALIZE_ROLE_PROVIDER);
            }
            if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                storeStatement.setShort(8, MessageExchangeImpl.SERIALIZE_STATUS_ACTIVE);
            } else if (ExchangeStatus.DONE.equals(exchange.getStatus())) {
                storeStatement.setShort(8, MessageExchangeImpl.SERIALIZE_STATUS_DONE);
            } else {
                storeStatement.setShort(8, MessageExchangeImpl.SERIALIZE_STATUS_ERROR);
            }
            storeStatement.setBoolean(9, exchange.isTerminated());
            storeStatement.setBoolean(10, exchange.isTransacted());
            storeStatement.setBoolean(11, exchange.isMonitored());
            storeStatement.setBoolean(12, exchange.isPersisted());
            storeStatement.setString(13, exchange.getExchangeId());

            // TODO: Manage the returned value
            storeStatement.execute();
        } finally {
            storeStatement.close();
        }
    }

    private void insertMessageExchangeProperties(final Connection connection,
            final MessageExchangeImpl exchange) throws SQLException, IOException {
        // TODO: To replace by a batch of SQL commands
        final PreparedStatement storeStatement = connection
                .prepareStatement("INSERT INTO MESSAGE_EXCHANGE_PROPERTY VALUES(?,?,?)");
        try {
            for (final Object key : exchange.getPropertyNames()) {
                storeStatement.setString(1, exchange.getExchangeId());
                storeStatement.setString(2, (String) key);
                if (HSQLDB_DRIVER.equals(databaseDriver)) {
                    storeStatement.setObject(3, exchange.getProperty((String) key));
                } else {
                    // With MYSQL, the setObject() method serializes not
                    // properly primitive alter-ego Objects, use Streams
                    // instead.
                    final ByteArrayOutputStream bytesOutputStream = new ByteArrayOutputStream();
                    new ObjectOutputStream(bytesOutputStream).writeUnshared(exchange
                            .getProperty((String) key));
                    final ByteArrayInputStream bytesInputStream = new ByteArrayInputStream(
                            bytesOutputStream.toByteArray());
                    storeStatement.setBinaryStream(3, bytesInputStream,
                            bytesInputStream.available());
                }
                storeStatement.addBatch();
                try {
                    // TODO: Manage the returned value
                    storeStatement.executeBatch();
                } catch (final SQLException e) {
                    // TODO: Improve error management to accept only the error
                    // associated to the duplicated entry.

                    // the entry may already exist, do nothing
                }
            }
        } finally {
            storeStatement.close();
        }
    }

    private void insertNormalizedMessages(final Connection connection,
            final MessageExchangeImpl exchange) throws SQLException {
        // TODO: To replace by a batch of SQL commands
        final PreparedStatement storeStatement = connection
                .prepareStatement("INSERT INTO NORMALIZED_MESSAGE VALUES(?,?,?)");
        try {
            for (final String key : exchange.getMessages().keySet()) {
                storeStatement.setString(1, exchange.getExchangeId());
                storeStatement.setString(2, key);
                storeStatement.setObject(3, exchange.getMessage(key));
                storeStatement.addBatch();
                try {
                    // TODO: Manage the returned value
                    storeStatement.executeBatch();
                } catch (final SQLException e) {
                    // TODO: Improve error management to accept only the error
                    // associated to the duplicated entry.

                    // the entry may already exist, do nothing
                }
            }
        } finally {
            storeStatement.close();
        }
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws SQLException {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(PersistenceService.COMPONENT_LOGGER_NAME));
        this.log.start();

        final ContainerConfiguration containerConfiguration = this.configurationService
                .getContainerConfiguration();
        this.storageDuration = containerConfiguration.getPersistenceDuration();
        fetchSize = containerConfiguration.getPersistenceFetchSize();
        this.databaseDriver = containerConfiguration.getRegistryDatabaseDriver();

        this.jdbcDataSource = new BasicDataSource();
        this.jdbcDataSource.setDriverClassName(containerConfiguration.getRegistryDatabaseDriver());
        this.jdbcDataSource.setUrl(containerConfiguration.getRegistryDatabaseUrl());
        this.jdbcDataSource.setUsername(containerConfiguration.getRegistryDatabaseUser());
        this.jdbcDataSource.setPassword(containerConfiguration.getRegistryDatabasePassword());

        // Check that the database supports batch update
        final Connection connection = this.jdbcDataSource.getConnection();
        try {
            if (!jdbcDataSource.getConnection().getMetaData().supportsBatchUpdates()) {
                log.error("The configured database does not support batch update that is required.");
                throw new SQLException(
                        "The configured database does not support batch update that is required.");
            }
        } finally {
            connection.close();
        }

        try {
            checkPersistenceTables(this.jdbcDataSource);
        } catch (final SQLException e) {
            this.log.error("Failed to start persistence service", e);
            throw e;
        }

        this.allocateCleanerTimer();
        this.cleanerTimer.schedule(new PersistenceCleaner(), this.storageDuration,
                this.storageDuration);

        this.log.end();
    }

    private final void allocateCleanerTimer() {
        this.cleanerTimer = new Timer("Persistence Cleaner");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.start();
        this.cleanerTimer.cancel();
        try {
            this.jdbcDataSource.close();
        } catch (final SQLException e) {
            this.log.debug(e);
        }
        this.log.end();
    }

    /**
     * Check id the tables exists. <br/>
     * If not the method create the persistence tables.
     * 
     * @param jdbcDataSource
     * @throws SQLException
     */
    private final void checkPersistenceTables(final BasicDataSource jdbcDataSource)
            throws SQLException {
        Statement statementCreate = null;
        String stringStatement = null;
        Statement statementCheck = null;
        boolean isTablesCreated;
        Connection connection = null;
        try {
            connection = jdbcDataSource.getConnection();
            statementCheck = connection.createStatement();
            try {
                statementCheck.setFetchSize(this.fetchSize);
                statementCheck.executeQuery("SELECT 1 FROM MESSAGE_EXCHANGE");
                isTablesCreated = true;
            } catch (final SQLException e) {
                isTablesCreated = false;
            }
            if (!isTablesCreated) {
                this.log.info("Creating database tables");
                statementCreate = connection.createStatement();
                // create the Message Exchange table
                if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                    statementCreate.executeUpdate("SET WRITE_DELAY FALSE");
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") PRIMARY KEY, CONSUMER_ENDPOINT OBJECT, ENDPOINT OBJECT, INTERFACE_NAME VARCHAR(255), SERVICE_NAME VARCHAR(255), ENDPOINT_NAME VARCHAR(255), OPERATION_NAME VARCHAR(255), PATTERN VARCHAR(63), ERROR OBJECT, ROLE TINYINT, STATUS TINYINT, TTERMINATED BOOLEAN, TRANSACTED BOOLEAN, MONITORED BOOLEAN, PERSISTED BOOLEAN, CREATION_TIMESTAMP BIGINT)";
                } else if (ORACLE_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") PRIMARY KEY, CONSUMER_ENDPOINT BLOB, ENDPOINT BLOB, INTERFACE_NAME VARCHAR(255), SERVICE_NAME VARCHAR(255), ENDPOINT_NAME VARCHAR(255), OPERATION_NAME VARCHAR(255), PATTERN VARCHAR(63), ERROR BLOB, ROLE NUMBER(3), STATUS NUMBER(3), TTERMINATED NUMBER(1), TRANSACTED NUMBER(1), MONITORED NUMBER(1), PERSISTED NUMBER(1), CREATION_TIMESTAMP NUMBER(38))";
                } else {
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") PRIMARY KEY, CONSUMER_ENDPOINT LONGBLOB, ENDPOINT LONGBLOB, INTERFACE_NAME VARCHAR(255), SERVICE_NAME VARCHAR(255), ENDPOINT_NAME VARCHAR(255), OPERATION_NAME VARCHAR(255), PATTERN VARCHAR(63), ERROR LONGBLOB, ROLE TINYINT, STATUS TINYINT, TTERMINATED BOOLEAN, TRANSACTED BOOLEAN, MONITORED BOOLEAN, PERSISTED BOOLEAN, CREATION_TIMESTAMP BIGINT)";
                }
                statementCreate.executeUpdate(stringStatement);
                stringStatement = "CREATE INDEX INTERFACE_INDEX ON MESSAGE_EXCHANGE (INTERFACE_NAME)";
                statementCreate.executeUpdate(stringStatement);
                stringStatement = "CREATE INDEX SERVICE_INDEX ON MESSAGE_EXCHANGE (SERVICE_NAME)";
                statementCreate.executeUpdate(stringStatement);
                stringStatement = "CREATE INDEX ENDPOINT_INDEX ON MESSAGE_EXCHANGE (ENDPOINT_NAME)";
                statementCreate.executeUpdate(stringStatement);
                stringStatement = "CREATE INDEX OPERATION_INDEX ON MESSAGE_EXCHANGE (OPERATION_NAME)";

                // create the Message Exchange Property table
                if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE_PROPERTY (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(255) NOT NULL, VALUE OBJECT, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                } else if (ORACLE_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE_PROPERTY (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(255) NOT NULL, VALUE BLOB, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                } else {
                    stringStatement = "CREATE TABLE MESSAGE_EXCHANGE_PROPERTY (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(255) NOT NULL, VALUE LONGBLOB, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                }
                statementCreate.executeUpdate(stringStatement);
                statementCreate
                        .executeUpdate("ALTER TABLE MESSAGE_EXCHANGE_PROPERTY ADD CONSTRAINT M_E_PROPERTY_FOREIGN_KEY FOREIGN KEY (EXCHANGE_ID) REFERENCES MESSAGE_EXCHANGE(EXCHANGE_ID)");

                // create the Normalized Message table
                if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE NORMALIZED_MESSAGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(5) NOT NULL, VALUE OBJECT, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                } else if (ORACLE_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE NORMALIZED_MESSAGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(5) NOT NULL, VALUE BLOB, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                } else {
                    stringStatement = "CREATE TABLE NORMALIZED_MESSAGE (EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH
                            + ") NOT NULL, KKEY VARCHAR(5) NOT NULL, VALUE LONGBLOB, PRIMARY KEY (EXCHANGE_ID, KKEY))";
                }
                statementCreate.executeUpdate(stringStatement);
                statementCreate
                        .executeUpdate("ALTER TABLE NORMALIZED_MESSAGE ADD CONSTRAINT N_M_FOREIGN_KEY FOREIGN KEY (EXCHANGE_ID) REFERENCES MESSAGE_EXCHANGE(EXCHANGE_ID)");

                // create exchange send/receive date table
                if (HSQLDB_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE EXCHANGE_TIMESTAMP (ID IDENTITY, EVENT_TIMESTAMP BIGINT, EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH + ") NOT NULL, IS_SENT BOOLEAN, TYPE VARCHAR(9))";
                } else if (ORACLE_DRIVER.equals(this.databaseDriver)) {
                    stringStatement = "CREATE TABLE EXCHANGE_TIMESTAMP (ID NUMBER(38), EVENT_TIMESTAMP NUMBER(38), EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH + ") NOT NULL, IS_SENT NUMBER(1), TYPE VARCHAR(9))";
                } else {
                    stringStatement = "CREATE TABLE EXCHANGE_TIMESTAMP (ID MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY, EVENT_TIMESTAMP BIGINT, EXCHANGE_ID VARCHAR("
                            + UUID_LENGTH + ") NOT NULL, IS_SENT BOOLEAN, TYPE VARCHAR(9))";
                }
                statementCreate.executeUpdate(stringStatement);
                statementCreate
                        .executeUpdate("ALTER TABLE EXCHANGE_TIMESTAMP ADD CONSTRAINT E_T_FOREIGN_KEY FOREIGN KEY (EXCHANGE_ID) REFERENCES MESSAGE_EXCHANGE(EXCHANGE_ID)");
            }
        } catch (final SQLException e) {
            isTablesCreated = false;
        } finally {
            if (statementCreate != null) {
                statementCreate.close();
            }
            if (statementCheck != null) {
                statementCheck.close();
            }
            if (connection != null) {
                connection.close();
            }
        }

    }
}
