/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.repository;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.management.JbiArchiveType;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.system.repository.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.EntityAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.ServiceAssemblyAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.SharedLibraryAlreadyExistsException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.api.util.ZipUtil;

/**
 * Platform Component Repository service implementation.
 * 
 * @since Petals 1.0
 * @author Rafael Marins - Fossil E-Commerce
 * @author Adrien Louis - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 * @author ddesjardins - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = RepositoryService.class))
public class RepositoryServiceImpl implements RepositoryService {

    /**
     * Path to components directory into petals repository
     */
    private static final String COMPONENTS_PATH = "components";

    /**
     * The date pattern
     */
    private final static String FOLDER_DATE_PATTERN = "MM-dd-yyyy-hhmmssSSS";

    /**
     * DateFormat used for repository folders
     */
    private final static SimpleDateFormat FOLDER_DATE_FORMAT = new SimpleDateFormat(
            RepositoryServiceImpl.FOLDER_DATE_PATTERN);

    /**
     * Lock to use with the DateFormat used for repository folders
     */
    private final static Object folderDatePatternLock = new Object();

    /**
     * Path to component install directory
     */
    private static final String INSTALL_PATH = "install";

    /**
     * Path to lost+found directory into petals repository
     */
    private static final String LOSTPLUSFOUND_PATH = "lost+found";

    /**
     * Path to components directory into petals repository
     */
    private static final String SERVICEASSEMBLIES_PATH = "service-assemblies";

    /**
     * Path to components directory into petals repository
     */
    private static final String SHAREDLIBRARIES_PATH = "shared-libraries";

    /**
     * Path to component working directory
     */
    private static final String WORK_PATH = "work";

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * The components directory path.
     */
    private File componentsDirectory;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The lost+found directory path.
     */
    private File lostFoundDirectory;

    /**
     * The complete repository directory path.
     */
    private File repositoryDirectory;

    /**
     * The service assemblies directory path.
     */
    private File serviceAssembliesDirectory;

    /**
     * The shared libraries directory path.
     */
    private File sharedLibrariesDirectory;
    
	// Because of PETALSESBCONT-17, it is needed to keep removed directory. They
	// are not completly removed because of component classloader not freed.
	// When adding a artifact to the repository, a check has been added to
	// detect a artifact already registered, and if the artifact directory is
	// not completly deleted, the check fails even if the artifact is no more
	// registered.
	// FIXME: Remove this attribute whene PETALSESBCONT-17 will be fixed or the
	// repository will integrate an index to check if an artifact is already
	// registered.
    private Set<String> removedArtifacts = Collections.synchronizedSet(new TreeSet<String>());

    /*
     * SimpleDateFormat methods must be synchronized.
     */
    private static String getCurrentDateAsString() {
        synchronized (RepositoryServiceImpl.folderDatePatternLock) {
            return RepositoryServiceImpl.FOLDER_DATE_FORMAT.format(new Date(System
                    .currentTimeMillis()));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addComponent(java.
     * lang.String, java.io.File)
     */
    public File addComponent(String componentId, File tempFile) throws IOException,
            ComponentAlreadyExistsException {
        try {
            return this.addPackage(componentId, tempFile, JbiArchiveType.COMPONENT_TYPE);
        } catch (final EntityAlreadyExistsException e) {
            throw new ComponentAlreadyExistsException(componentId, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addServiceAssembly
     * (java.lang.String, java.io.File)
     */
    public File addServiceAssembly(String serviceAssemblyId, File tempFile) throws IOException,
            ServiceAssemblyAlreadyExistsException {
        try {
            return this.addPackage(serviceAssemblyId, tempFile, JbiArchiveType.SA_TYPE);
        } catch (final EntityAlreadyExistsException e) {
            throw new ServiceAssemblyAlreadyExistsException(serviceAssemblyId, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addSharedLibrary(java
     * .lang.String, java.io.File)
     */
    public File addSharedLibrary(final String sharedLibraryId, final String version,
            final File tempFile) throws IOException, SharedLibraryAlreadyExistsException {
        try {
            return this.addPackage(sharedLibraryId + "-" + version, tempFile, JbiArchiveType.SL_TYPE);
        } catch (final EntityAlreadyExistsException e) {
            throw new SharedLibraryAlreadyExistsException(sharedLibraryId, version, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * explodeSUIntoSAInstallDirectory(java.lang.String, java.net.URL,
     * java.lang.String)
     */
    @SuppressWarnings("deprecation")
    public File explodeSUIntoSAInstallDirectory(String serviceUnitId, URL suZipLocation,
            String serviceAssemblyId) throws IOException {
        File installationRoot = this.createSUInstallDirectory(serviceUnitId, serviceAssemblyId);
        try {
            File zipArchive = new File(URLDecoder.decode(suZipLocation.getPath()));
            ZipUtil.unzipArchive(zipArchive, installationRoot);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(installationRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                this.log.warning("Failed to clean a failed installation", e);
            }
            throw e;
        }
        return installationRoot;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentDirectory
     * (java.lang.String)
     */
    public File getComponentDirectory(String componentId) throws IOException {
        return this.getEntityRoot(componentId, JbiArchiveType.COMPONENT_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getComponentInstallDirectory(java.lang.String)
     */
    public File getComponentInstallDirectory(String componentId) throws IOException {
        return new File(this.getComponentDirectory(componentId), INSTALL_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentsDirectory
     * ()
     */
    public File getComponentsDirectory() {
        return this.componentsDirectory;

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentWorkDirectory
     * (java.lang.String)
     */
    public File getComponentWorkDirectory(String componentId) throws IOException {
        return new File(this.getComponentDirectory(componentId), WORK_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getLostPlusFoundDirectory
     * ()
     */
    public File getLostPlusFoundDirectory() {
        return this.lostFoundDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getRepositoryDirectory
     * ()
     */
    public File getRepositoryDirectory() {
        return this.repositoryDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssembliesDirectory()
     */
    public File getServiceAssembliesDirectory() {
        return this.serviceAssembliesDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyDirectory(java.lang.String)
     */
    public File getServiceAssemblyDirectory(String serviceAssemblyId) throws IOException {
        return this.getEntityRoot(serviceAssemblyId, JbiArchiveType.SA_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyInstallDirectory(java.lang.String)
     */
    public File getServiceAssemblyInstallDirectory(String serviceAssemblyId) throws IOException {
        return new File(this.getServiceAssemblyDirectory(serviceAssemblyId), INSTALL_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyWorkDirectory(java.lang.String)
     */
    public File getServiceAssemblyWorkDirectory(String serviceAssemblyId) throws IOException {
        return new File(this.getServiceAssemblyDirectory(serviceAssemblyId), WORK_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibrariesDirectory()
     */
    public File getSharedLibrariesDirectory() {
        return this.sharedLibrariesDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getSharedLibraryDirectory
     * (java.lang.String)
     */
    public File getSharedLibraryDirectory(String sharedLibraryId) throws IOException {
        return this.getEntityRoot(sharedLibraryId, JbiArchiveType.SL_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibraryInstallDirectory(java.lang.String)
     */
    public File getSharedLibraryInstallDirectory(String sharedLibraryId) throws IOException {
        String name = INSTALL_PATH;
        return new File(this.getSharedLibraryDirectory(sharedLibraryId), name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibraryWorkDirectory(java.lang.String)
     */
    public File getSharedLibraryWorkDirectory(String sharedLibraryId) throws IOException {
        return new File(this.getSharedLibraryDirectory(sharedLibraryId), WORK_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeComponent(java
     * .lang.String)
     */
    public void removeComponent(String componentId) throws FileNotFoundException, IOException {
        this.removePackage(componentId, JbiArchiveType.COMPONENT_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeServiceAssembly
     * (java.lang.String)
     */
    public void removeServiceAssembly(String serviceAssemblyId) throws FileNotFoundException, IOException {
        this.removePackage(serviceAssemblyId, JbiArchiveType.SA_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeSharedLibrary
     * (java.lang.String)
     */
    public void removeSharedLibrary(final String sharedLibraryId, final String version) throws FileNotFoundException, IOException {
        this.removePackage(sharedLibraryId + "-" + version, JbiArchiveType.SL_TYPE);
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(RepositoryService.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.containerConfiguration = this.configurationService.getContainerConfiguration();

        this.repositoryDirectory = new File(
                this.containerConfiguration.getRepositoryDirectoryPath());

        try {
            if (!this.repositoryDirectory.exists() && !this.repositoryDirectory.mkdir()) {
                throw new IOException("Directory '" + this.repositoryDirectory.getAbsolutePath()
                        + "' can not be create");
            }

            this.lostFoundDirectory = new File(this.repositoryDirectory, LOSTPLUSFOUND_PATH);
            if (!this.lostFoundDirectory.exists()) {
                this.lostFoundDirectory.mkdir();
            }

            this.componentsDirectory = new File(this.repositoryDirectory, COMPONENTS_PATH);
            if (!this.componentsDirectory.exists()) {
                this.componentsDirectory.mkdir();
            }

            this.serviceAssembliesDirectory = new File(this.repositoryDirectory,
                    SERVICEASSEMBLIES_PATH);
            if (!this.serviceAssembliesDirectory.exists()) {
                this.serviceAssembliesDirectory.mkdir();
            }

            this.sharedLibrariesDirectory = new File(this.repositoryDirectory, SHAREDLIBRARIES_PATH);
            if (!this.sharedLibrariesDirectory.exists()) {
                this.sharedLibrariesDirectory.mkdir();
            }
        } catch (SecurityException e) {
            this.log.error(e);
            throw e;
        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Copy an entity package into a PEtALS repository
     * 
     * @param entityId
     *            {@link String} unique entity id that identifies an unique
     *            entity directory into the petals repository.
     * @param tempFile
     *            {@link File} the entity package file to add into the
     *            repository.
     * @param entityType
     *            the entity type: component, shared lib or service assembly
     * @return installation root {@link File}
     * @throws IOException
     *             if entity installation or work root can't be created or if
     *             entity files can't be copied in the installation root
     * @throws EntityAlreadyExistsException
     *             The entity to add already exists in the repository
     */
    private File addPackage(String entityId, File tempFile, JbiArchiveType entityType)
            throws IOException, EntityAlreadyExistsException {

        File rootDirectory = this.createEntityDirectory(entityId, entityType);
        File installationRoot = this.createEntityInstallDirectory(rootDirectory);
        File workRoot = this.createEntityWorkDirectory(rootDirectory);

        try {
            FileUtils.copyDirectory(tempFile, installationRoot);
            FileUtils.forceDelete(tempFile);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(rootDirectory);
                FileUtils.forceDelete(workRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                this.log.warning("Residual incomplete component directory");
            }
            throw e;
        }

        return installationRoot;
    }

    private File createComponentDirectory(String componentId) throws IOException,
            EntityAlreadyExistsException {

        if (this.getComponentDirectory(componentId) != null) {
            throw new EntityAlreadyExistsException(componentId, JbiArchiveType.COMPONENT_TYPE);
        }
        String name = componentId + "-" + RepositoryServiceImpl.getCurrentDateAsString();
        File componentDirectory = new File(this.componentsDirectory, name);
        componentDirectory.mkdir();

        return componentDirectory;
    }

    private File createEntityDirectory(String entityId, JbiArchiveType entityType) throws IOException,
            EntityAlreadyExistsException {
        File rootFile = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                rootFile = this.createComponentDirectory(entityId);
                break;
            case SA_TYPE:
                rootFile = this.createServiceAssemblyDirectory(entityId);
                break;
            case SL_TYPE:
                rootFile = this.createSharedLibraryDirectory(entityId);
                break;
        }
        return rootFile;
    }

    private File createEntityInstallDirectory(File rootDirectory) {
        File installDirectory = new File(rootDirectory, INSTALL_PATH);
        installDirectory.mkdir();

        return installDirectory;
    }

    private File createEntityWorkDirectory(File rootDirectory) {
        File workDirectory = new File(rootDirectory, WORK_PATH);
        workDirectory.mkdir();

        return workDirectory;
    }

    /**
     * Create the shared library root directory.
     * 
     * @param sharedLibraryId
     * @return
     */
    private File createServiceAssemblyDirectory(String serviceAssemblyId) throws IOException,
            EntityAlreadyExistsException {

        if (this.getServiceAssemblyDirectory(serviceAssemblyId) != null) {
            throw new EntityAlreadyExistsException(serviceAssemblyId, JbiArchiveType.SA_TYPE);
        }
        String name = serviceAssemblyId + "-" + RepositoryServiceImpl.getCurrentDateAsString();
        File serviceAssemblyDirectory = new File(this.serviceAssembliesDirectory, name);
        serviceAssemblyDirectory.mkdir();

        return serviceAssemblyDirectory;
    }

    /**
     * Create the shared library root directory.
     * 
     * @param sharedLibraryId
     * @return
     */
    private File createSharedLibraryDirectory(String sharedLibraryId) throws IOException,
            EntityAlreadyExistsException {

        if (this.getSharedLibraryDirectory(sharedLibraryId) != null) {
            throw new EntityAlreadyExistsException(sharedLibraryId, JbiArchiveType.SL_TYPE);
        }
        String name = sharedLibraryId + "-" + RepositoryServiceImpl.getCurrentDateAsString();
        File sharedLibraryDirectory = new File(this.sharedLibrariesDirectory, name);
        sharedLibraryDirectory.mkdir();

        return sharedLibraryDirectory;
    }

    /**
     * Create the SU install root into the SA install root
     * 
     * @param suId
     *            the service unit unique identifier
     * @param saId
     *            the service assembly unique identifier
     * @return the SU install root File
     * @throws PetalsException
     *             if SU install root can't be created
     */
    private File createSUInstallDirectory(String serviceUnitId, String serviceAssemblyId)
            throws IOException {
        File installationRoot = new File(
                this.getServiceAssemblyInstallDirectory(serviceAssemblyId), serviceUnitId);

        if (!installationRoot.exists()) {
            installationRoot.mkdirs();
        }

        return installationRoot;
    }

    private File getEntityRoot(String entityId, JbiArchiveType entityType) throws IOException {
        File entitiesDirectory = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                entitiesDirectory = this.getComponentsDirectory();
                break;
            case SA_TYPE:
                entitiesDirectory = this.getServiceAssembliesDirectory();
                break;
            case SL_TYPE:
                entitiesDirectory = this.getSharedLibrariesDirectory();
                break;
        }

        String[] entityFilePaths = entitiesDirectory.list(new RepositoryFileFilter(entityId));
        if (entityFilePaths != null) {
            String entityFile = "";
            for (String _entityFile : entityFilePaths) {
            	if (!this.removedArtifacts.contains(_entityFile)) {
                    entityFile = _entityFile;
                }
            }

            if (entityFile.isEmpty()) {
                return null;
            } else {
                return new File(entitiesDirectory, entityFile);
            }
        } else {
            // An error occurs during listing the content of the entity
            // directory (does not exist or IOException)
            throw new IOException("The PEtALS repository ("
                    + this.repositoryDirectory.getAbsolutePath()
                    + ") is corrupted. Please check it!");
        }
    }

    private void removePackage(String entityId, JbiArchiveType entityType) throws FileNotFoundException, IOException {
        File entityRootFile = null;
        switch (entityType) {
            case COMPONENT_TYPE:
                entityRootFile = this.getComponentDirectory(entityId);
                break;
            case SA_TYPE:
                entityRootFile = this.getServiceAssemblyDirectory(entityId);
                break;
            case SL_TYPE:
                entityRootFile = this.getSharedLibraryDirectory(entityId);
                break;
        }

        if (entityRootFile != null) {
            this.removedArtifacts.add(entityRootFile.getName());
            FileUtils.deleteDirectory(entityRootFile);
        } else {
            throw new FileNotFoundException("The entity named '" + entityId + "' of type '"
                    + entityType + "' do not exist in the repository");
        }
    }

}
