/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

/**
 * The <code>Transporter</code> is used to send a <code>MessageExchange</code>
 * from a Petals container to another one. <br>
 * The local <code>Router</code> calls
 * <code>send(exchange,distantContainer)</code> on this <code>Transporter</code>
 * to send an exchange. <br>
 * This <code>Transporter</code> sends the <code>MessageExchange</code> to the
 * distant <code>Transporter</code>. <br>
 * When the distant <code>Transporter</code> receives the
 * <code>MessageExchange</code>, it has to call <code>receive(exchange)</code>
 * on its local <code>Router</code>. <br>
 * The localization of the distant container is in charge of the
 * <code>Transporter</code>. <br>
 * <br>
 * <em>Note that the destination container can be the same than this one.</em>

 * @author Adrien Louis - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public interface Transporter {

    /**
     * The list of provided transporter. Note that in standalone distribution,
     * only the <code>LOCAL_FRACTAL_TRANSPORTER</code> is supported
     */
    static final String LOCAL_FRACTAL_TRANSPORTER = "local";

    static final String TCP_FRACTAL_TRANSPORTER = "tcp";

    /* TODO Not yet implemented! */
    static final String SSL_FRACTAL_TRANSPORTER = "ssl";

    /* TODO Not yet implemented! */
    static final String HTTP_FRACTAL_TRANSPORTER = "http";

    /**
     * Send the specified exchange to the specified container. According to the
     * transport policy, the message is may be not persisted by the Transport
     * layer, so it can be lost in case of problem.
     * 
     * @param exchangeDecorator
     *            the exchange decorator to send.
     * @param componentName
     *            the name of the distant container.
     * @param containerName
     *            the name of the distant container.
     * @param subDomainName
     *            the name of the sub-domain of the remote container.
     * @param timeToLive
     *            duration of the message life (in ms). 0 for no duration
     *            
     * @throws TransportException
     *             If the exchange is invalid, or can not be serialized. If the
     *             distant container can not be found.
     */
    void send(final MessageExchangeWrapper exchangeDecorator, final TransportSendContext transportContext)
            throws TransportException;

    /**
     * Send the specified exchange to the specified container and wait until the
     * response is received or the timeout is reached. According to the
     * transport policy, the message is may be not persisted by the Transport
     * layer, so it can be lost in case of problem.
     * 
     * @param exchangeDecorator
     *            the exchange decorator to send.
     * @param componentName
     *            the name of the distant container.
     * @param containerName
     *            the name of the distant container.
     *            non empty
     * @param subDomainName
     *            the name of the sub-domain of the remote container.
     * @param timeToLive
     *            duration of the message life (in ms). 0 for no duration
     *            
     * @throws TransportException
     *             If the exchange is invalid, or can not be serialized. If the
     *             distant container can not be found.
     */
    void sendSync(final MessageExchangeWrapper exchangeDecorator,
            final TransportSendContext transportContext) throws TransportException;

    /**
     * Prepare the stopping of the component. It means that all ongoing message
     * traffic must terminate during the method invocation and all new message
     * traffic is rejected.
     */
    void stopTraffic();
}
