/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.local.monitoring;

import java.util.Collections;
import java.util.Map;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.local.Constants;
import org.ow2.petals.probes.api.KeyedProbesFactory;
import org.ow2.petals.probes.api.KeyedProbesFactoryBuilder;
import org.ow2.petals.probes.api.enums.ExecutionStatus;
import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;
import org.ow2.petals.probes.api.exceptions.ProbeKeyMissingException;
import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;
import org.ow2.petals.probes.api.key.ProbeKey;
import org.ow2.petals.probes.api.probes.KeyedCounterProbeWithExecutionStatus;

/**
 * Local transporter monitoring service implementation.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = LocalTransporterMonitoringItf.class) })
public class LocalTransporterMonitoring implements LocalTransporterMonitoringItf,
        LocalTransporterMonitoringMBean {

    /**
     * The logger.
     */
    private LoggingUtil log;

    /**
     * The probe counting messages sent by this transporter.
     */
    private final KeyedCounterProbeWithExecutionStatus<ProbeKey> deliveredMessagesCount;

    public LocalTransporterMonitoring() throws MultipleProbesFactoriesFoundException,
            NoProbesFactoryFoundException {
        final KeyedProbesFactoryBuilder<LocalTransporterProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<LocalTransporterProbeKey, Long, Long>();
        final KeyedProbesFactory<LocalTransporterProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                .getKeyedProbesFactory();
        this.deliveredMessagesCount = probesFactory.createKeyedCounterProbeWithExecutionStatus();
    }

    public void incMessageSentProbe(final MessageExchangeWrapper exchangeDecorator) {

        final ProbeKey messageSentProbeKey = new LocalTransporterProbeKey(exchangeDecorator);
        try {
            this.deliveredMessagesCount.incPending(messageSentProbeKey);
            this.deliveredMessagesCount.move(messageSentProbeKey, ExecutionStatus.SUCCEEDED);
        } catch (ProbeNotStartedException e) {
            this.log.warning("Probe counting asynchronous message exchanges sent is not started. Values of probes could be incorrect.");
        } catch (final ProbeKeyMissingException e) {
            this.log.warning("Probe key missing. Values of probe counting message exchanges could be incorrect.");
        }
    }

    public Map<String[], Long> getDeliveredMessagesCount() {
        try {
            return this.deliveredMessagesCount.getConvertedValues();
        } catch (final ProbeNotInitializedException e) {
            this.log.warning("Probe counting message exchanges sent is not initialized. No value available.");
            return Collections.EMPTY_MAP;
        }
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        this.deliveredMessagesCount.init();
        this.deliveredMessagesCount.start();

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        this.log.start();

        this.deliveredMessagesCount.stop();
        this.deliveredMessagesCount.shutdown();

        this.log.end();
    }

}
