/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio;

import java.io.IOException;
import java.util.ConcurrentModificationException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.exchange.PersistedMessageExchangeWrapper;
import org.ow2.petals.microkernel.transport.TransportException;
import org.ow2.petals.microkernel.transport.TransportListener;
import org.ow2.petals.microkernel.transport.Transporter;
import org.ow2.petals.microkernel.transport.platform.nio.client.NioClientObjectPool;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioClient;
import org.ow2.petals.microkernel.transport.platform.nio.server.NioReceiver;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;
import org.ow2.petals.microkernel.transport.util.TransporterUtil;

/**
 * NIO Implementation of the Transporter.
 * @author Christophe DENEUX (Capgemini Sud)
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = Transporter.class),
        @Interface(name = NioReceiver.NIO_MSG_RECEIVER_ITF, signature = NioReceiver.class) })
public class NioTransporterImpl implements Transporter, NioReceiver {

    /**
     * the Transporter listener (router) Fractal component
     */
    @Requires(name = "transportlistener")
    private TransportListener transportListener;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * All connections to remote containers are pooled into a keyed-pool. The
     * key is the name of the remote container.
     */
    @Requires(name = NioClientObjectPool.NIO_CLIENT_OBJECT_ITF)
    private NioClientObjectPool clientConnections;

    /**
     * Traffic flag to handle stop of traffic
     */
    private boolean stopTraffic;

    /**
     * Map of exchanges that are blocked during a synchronous send
     */
    private Map<String, MessageExchangeWrapper> pendingSyncExchanges;

    /**
     * Logger Wrapper
     */
    private LoggingUtil log;

    /**
     * Timeout to send a message exchange.
     */
    private long sendTimeout;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.transport.platform.nio.server.NioReceiver#onMessage(java
     * .lang.Object)
     */
    public void onMessage(final Object object) {
        this.log.start();

        if (object instanceof MessageExchangeImpl) {
            final MessageExchangeImpl messageExchange = (MessageExchangeImpl) object;
            final PersistedMessageExchangeWrapper exchangeDecorator = new PersistedMessageExchangeWrapper(
                    messageExchange, null);

            if (TransporterUtil.getSyncMode(exchangeDecorator, true)) {
                this.log.debug("The message " + messageExchange.getExchangeId()
                        + " is a synchronized response");
                TransporterUtil.updateSyncProperties(exchangeDecorator);
                MessageExchangeWrapper oldExchangeDecorator = this.pendingSyncExchanges.put(
                        messageExchange.getExchangeId(), exchangeDecorator);
                synchronized (oldExchangeDecorator) {
                    oldExchangeDecorator.notify();
                }
            } else {
                this.log.debug("The message " + exchangeDecorator.getExchangeId()
                        + " is not a synchronized response");
                this.transportListener.onExchange(exchangeDecorator);
            }
        } else {
            // This case doesn't have to exist.
            this.log.warning("Unexpected received data. Data are discarded");
        }

        this.log.end();
    }

    /**
     * This method does following action :
     * <ol>
     * <li>Serialize the message exchange.</li>
     * <li>Get the destination of the message.</li>
     * <li>Add it to the message.</li>
     * <li>Send it to the transport agent.</li>
     * </ol>
     * 
     * @see Transporter#send(MessageExchangeImpl, java.lang.String,
     *      java.lang.String, long)
     */
    public void send(MessageExchangeWrapper exchangeDecorator,
            TransportSendContext transportContext) throws TransportException {
        this.log.start("Send exchange to destination '" + transportContext.destination + "'");

        this.checkTransporterState();

        try {
            final NioClient clientConnection = this.clientConnections
                    .borrowObject(transportContext.destination.getContainerName());

            try {
                clientConnection.send(exchangeDecorator.getMessageExchange(), this.sendTimeout);

                // notify that the exchange is effectively sent
                this.transportListener.exchangeSent(exchangeDecorator);

                // In case of error during send, the clientConnection is
                // automatically invalidate.
            } finally {
                this.clientConnections.returnObject(
                        transportContext.destination.getContainerName(), clientConnection);
            }

        } catch (final IOException e) {
            throw new TransportException(e);
        } catch (final InterruptedException e) {
            throw new TransportException(e);
        } catch (final Exception e) {
            throw new TransportException(e);
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.transport.Transporter#sendSync(org.ow2.petals.jbi.messaging
     * .exchange.MessageExchangeDecorator,
     * org.ow2.petals.transport.util.TransportSendContext)
     */
    public void sendSync(final MessageExchangeWrapper exchangeDecorator,
            final TransportSendContext transportContext) throws TransportException {
        this.log
                .start("Send synchronous exchange to destination : " + transportContext.destination);
        MessageExchangeWrapper responseExchange;

        this.checkTransporterState();

        // set the relevant properties to mark the message as a
        // synchronous send
        try {
            TransporterUtil.setSendSyncProperties(exchangeDecorator);
        } catch (final MessagingException e) {
            throw new TransportException(e);
        }

        this.pendingSyncExchanges.put(exchangeDecorator.getExchangeId(), exchangeDecorator);

        try {
            synchronized (exchangeDecorator) {
                this.send(exchangeDecorator, transportContext);
                // wait for a response or a timeout reached
                exchangeDecorator.wait(transportContext.timeout);
            }
            responseExchange = this.pendingSyncExchanges.remove(exchangeDecorator.getExchangeId());
        } catch (final InterruptedException e) {
            TransporterUtil.updateSyncProperties(exchangeDecorator);
            this.pendingSyncExchanges.remove(exchangeDecorator.getExchangeId());
            throw new TransportException(
                    "Failed to receive the response message of a synchronous send", e);
        }

        // If responseExchange is the exchange,
        // it means that the timeout has been reached or the transporter is
        // stopping
        if (responseExchange == exchangeDecorator) {
            this.checkTransporterState();
            exchangeDecorator.setTimeout(true);

            this.log.warning("Failed to send synchronously the exchange: "
                    + exchangeDecorator.getExchangeId() + ". Timeout occured");

            this.log.end();
        } else {
            try {
                exchangeDecorator.setMessageExchange(responseExchange.getMessageExchange());
            } catch (MessagingException e) {
                throw new TransportException(e);
            }
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.transport.Transporter#stopTraffic()
     */
    public void stopTraffic() {
        this.log.start();
        this.stopTraffic = true;

        // Awake all the pending 'SYNC_SEND' issuer threads to abort the
        // invocation
        boolean redo = true;
        while (redo) {
            try {
                for (final MessageExchange messageExchangeImpl : this.pendingSyncExchanges.values()) {
                    synchronized (messageExchangeImpl) {
                        messageExchangeImpl.notify();
                    }
                }
                redo = false;
            } catch (final ConcurrentModificationException e) {
                // Try again
            }
        }

        this.log.end();
    }

    /**
     * Init variables with Fractal attributes value and create/bind the topic
     * for this container.
     * 
     * @throws NoSuchInterfaceException
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException, NoSuchInterfaceException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        this.sendTimeout = this.configurationService.getContainerConfiguration()
                .getTCPSendTimeout();
        this.pendingSyncExchanges = new ConcurrentHashMap<String, MessageExchangeWrapper>(100);

        this.log.end();
    }

    /**
     * Method called when the fractal component is stopped
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        this.log.call();
    }

    /**
     * Check if the transporter is running, if not, raise an exception.
     * 
     * @throws TransportException
     */
    private void checkTransporterState() throws TransportException {
        if (this.stopTraffic) {
            Thread.currentThread().interrupt();
            throw new TransportException("The Transporter traffic is stopped",
                    new InterruptedException());
        }
    }

}
