/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.client;

import org.apache.commons.pool.impl.GenericKeyedObjectPool;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioClient;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioSelector;

/**
 * This class is only a Fractal Wrapper of the Apache generic keyed object pool.
 * @author Christophe DENEUX - Capgemini Sud
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = NioClientObjectPool.class))
public class NioClientObjectPoolImpl implements NioClientObjectPool {

    /**
     * Interface used to access configuration
     */
    @Requires(name = "configuration", signature = ConfigurationService.class)
    protected ConfigurationService configurationService;

    /**
     * Interface used to get host and port of message recipient
     */
    @Requires(name = "topology")
    protected TopologyService topologyService;

    /**
     * <p>
     * The socket selector Fractal component to use to process network IO.
     * </p>
     */
    @Requires(name = NioSelector.NIO_SELECTOR_ITF, signature = NioSelector.class)
    protected NioSelector selectorAgent;

    /**
     * All connections to remote containers are pooled into a keyed-pool. The
     * key is the name of the remote container.
     */
    private GenericKeyedObjectPool clientConnections;

    /**
     * Logger Wrapper
     */
    protected LoggingUtil log;

    public NioClient borrowObject(final String targetContainerName) throws Exception {
        return (NioClient) this.clientConnections.borrowObject(targetContainerName);
    }

    public void returnObject(final String targetContainerName, final NioClient clientConnection)
            throws Exception {
        this.clientConnections.returnObject(targetContainerName, clientConnection);

    }

    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException, NoSuchInterfaceException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        // Connection pool initialization
        final GenericKeyedObjectPool.Config config = new GenericKeyedObjectPool.Config();
        config.minIdle = 0;
        config.maxActive = this.configurationService.getContainerConfiguration().getTCPSenders();
        config.testOnBorrow = true;
        config.minEvictableIdleTimeMillis = this.configurationService.getContainerConfiguration()
                .getTCPSenderEvictableDelay();
        config.timeBetweenEvictionRunsMillis = this.configurationService
                .getContainerConfiguration().getTCPSenderEvictorDelay();

        this.clientConnections = new GenericKeyedObjectPool(new NioClientAgentPoolObjectFactory(
                this.topologyService, this.selectorAgent, this.configurationService
                        .getContainerConfiguration().getTCPConnectionTimeout(), this.log.getLogger()),
                config);

        this.log.end();
    }

    /**
     * Method called when the fractal component is stopped
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        this.log.start();

        // Close the connection pool and disconnect all clients
        this.clientConnections.close();

        this.log.end();
    }

}
