/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.util.stream.queue;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;


/**
 * <p>This class implements an input stream in which the data are read from a byte
 * array queue.</p>
 * <p>
 * Closing a <tt>QueueInputStream</tt> has no effect. For performance
 * reasons, the methods in this class can be called after the stream has been
 * closed without generating an <tt>IOException</tt>.</p>
 * <p>
 * <code>QueueInputStream</code> does not support <code>mark</code> and
 * <code>reset</code> methods.</p>
 * 
 * @author Christophe DENEUX - Capgemini Sud
 */
public class QueueInputStream extends InputStream {

    private final QueueOutputStream llos;

    /**
     * Index of the read beginning reading of the first entry in the queue. It is
     * used to optimze the read algorithm in case of partial reads.
     */
    private int bufferReadPos = 0;

    public QueueInputStream(final QueueOutputStream llos) {
        this.llos = llos;
    }

    @Override
    public int available() throws IOException {

        int available = 0;

        final Iterator<byte[]> iterator = this.llos.bufferList.iterator();

        while (iterator.hasNext()) {
            available += iterator.next().length;
        }

        // Already read bytes (by a previous call to read()) of the first entry does not have to be counted
        return available - this.bufferReadPos;
    }

    @Override
    public boolean markSupported() {
        return false;
    }

    @Override
    public int read() throws IOException {
        byte[] buffer = new byte[1];
        int nbByteRead = this.read(buffer);
        if (nbByteRead != 1) {
            throw new IOException("No byte read.");
        }
        return buffer[0];
    }

    @Override
    public int read(final byte[] cbuf) throws IOException {
        return this.read(cbuf, 0, cbuf.length);
    }

    @Override
    public int read(final byte[] cbuf, final int off, final int len) throws IOException {
        return this.localRead(cbuf, off, len, false);
    }

    @Override
    public long skip(long n) throws IOException, IllegalArgumentException {
        if (n > Long.MAX_VALUE) {
            throw new IllegalArgumentException("Only value lesser " + Integer.MAX_VALUE
                    + "are accepted.");
        }
        return this.localRead(null, 0, (int) n, true);
    }

    protected int localRead(final byte[] cbuf, final int off, final int len, final boolean skip)
            throws IOException {
        int remainingLen = len;
        int offset = off;

        byte[] buffer;
        while ((buffer = this.llos.bufferList.peek()) != null
				&& remainingLen != 0) {
            // Already read bytes (by a previous call to read()) of the first
            // entry does not have to be counted
            int availableBytesToReadInBuffer = buffer.length - this.bufferReadPos;
            if (remainingLen >= availableBytesToReadInBuffer) {
                // Full copy needed
                if (!skip) {
                    System.arraycopy(buffer, this.bufferReadPos, cbuf, offset, availableBytesToReadInBuffer);
                }

                // We remove the head of the buffer queue.
                this.llos.bufferList.remove();
                this.bufferReadPos = 0;

                remainingLen -= availableBytesToReadInBuffer;
                if (remainingLen == 0) {
                    // No more bytes must be read
                    return len;
                }
                offset += availableBytesToReadInBuffer;

            } else {
                // Partial copy needed because the provided buffer is smaller
                // than the first list entry
                if (!skip) {
                    System.arraycopy(buffer, this.bufferReadPos, cbuf, offset, remainingLen);
                }

                this.bufferReadPos += remainingLen;
                
                // D'ont remove the head of the buffer queue because some bytes
				// are remaining in the head buffer.

                return len;
            }
        }

        // All bytes available in different entries of the buffer queue have
		// been read. No more bytes are available in the provided buffer, we
		// return the number byte read.
        return offset - off;
    }

}
