/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.extension;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.objectweb.fractal.api.Component;
import org.ow2.petals.microkernel.api.extension.PetalsExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.NotSatisfiedDependenciesException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link ExtensionsManagerImpl}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ExtensionsManagerImplTest {

    /**
     * Name of the method {@link ExtensionsManagerImpl#sortExtensionControllers}
     */
    private final static String METHOD_NAME_SORT_EXTENSION_CONTROLLERS = "sortExtensionControllers";

    /**
     * Name of the method {@link ExtensionsManagerImpl#isAllDependenciesLoaded}
     */
    private final static String METHOD_NAME_IS_ALL_DEPENDENCIES_LOADED = "isAllDependenciesLoaded";

    /**
     * Name of the attribute {@link ExtensionsManagerImpl#extensionControllers}
     */
    private final static String ATTRIBUTE_NAME_EXTENSION_CONTROLLERS = "extensionControllers";

    private class ExtensionController implements PetalsExtensionController {

        private String name;

        private String[] dependencies;

        private boolean isActivated;

        public ExtensionController(final String name, final String[] dependencies) {
            this(name, dependencies, false);
        }

        public ExtensionController(final String name, final String[] dependencies,
                final boolean isActivated) {
            super();
            this.name = name;
            this.dependencies = dependencies;
            this.isActivated = isActivated;
        }

        @Override
        public Component createFractalComponent(Component extensionComposite)
                throws CreationExtensionException {
            return null;
        }

        @Override
        public void removeFractalComponent() {
            // NOP
        }

        @Override
        public String getExtensionName() {
            return this.name;
        }

        @Override
        public boolean isActivated(Properties properties) {
            return this.isActivated;
        }

        @Override
        public String[] getDependencies() {
            return this.dependencies;
        }

    }

    @Rule
    public ExpectedException expectedEx = ExpectedException.none();

    /**
     * Check the processing of
     * {@link ExtensionsManagerImpl#sortExtensionControllers} against a simple
     * list of extensions already sorted.
     * 
     */
    @Test
    public void testSortExtensionControllers_00() throws InvocationTargetException {
        
        final List<PetalsExtensionController> extensionControllers = new ArrayList<PetalsExtensionController>();
        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        extensionControllers.add(new ExtensionController(unitTest1, null));
        extensionControllers.add(new ExtensionController(unitTest2, new String[] { unitTest1 } ));
        
        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        final List<PetalsExtensionController> sortedExtensionController = (List<PetalsExtensionController>) ReflectionHelper
                .invokePrivateMethod(extMgr,
                        ExtensionsManagerImplTest.METHOD_NAME_SORT_EXTENSION_CONTROLLERS,
                        new Object[] { extensionControllers }, new Class[] { List.class });
        assertNotNull("Null result", sortedExtensionController);
        assertEquals("Unexpected size", 2, sortedExtensionController.size());
        assertEquals("Unexpected first element", unitTest1, sortedExtensionController.get(0)
                .getExtensionName());
        assertEquals("Unexpected 2nd element", unitTest2, sortedExtensionController.get(1)
                .getExtensionName());
    }

    /**
     * Check the processing of
     * {@link ExtensionsManagerImpl#sortExtensionControllers} against a simple
     * list of extensions not sorted.
     * 
     */
    @Test
    public void testSortExtensionControllers_01() throws InvocationTargetException {

        final List<PetalsExtensionController> extensionControllers = new ArrayList<PetalsExtensionController>();
        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        extensionControllers.add(new ExtensionController(unitTest2, new String[] { unitTest1 }));
        extensionControllers.add(new ExtensionController(unitTest1, null));

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        final List<PetalsExtensionController> sortedExtensionController = (List<PetalsExtensionController>) ReflectionHelper
                .invokePrivateMethod(extMgr,
                        ExtensionsManagerImplTest.METHOD_NAME_SORT_EXTENSION_CONTROLLERS,
                        new Object[] { extensionControllers }, new Class[] { List.class });
        assertNotNull("Null result", sortedExtensionController);
        assertEquals("Unexpected size", 2, sortedExtensionController.size());
        assertEquals("Unexpected first element", unitTest1, sortedExtensionController.get(0)
                .getExtensionName());
        assertEquals("Unexpected 2nd element", unitTest2, sortedExtensionController.get(1)
                .getExtensionName());
    }

    /**
     * Check the processing of
     * {@link ExtensionsManagerImpl#sortExtensionControllers} against a list of
     * extensions not sorted.
     * 
     */
    @Test
    public void testSortExtensionControllers_02() throws InvocationTargetException {

        final List<PetalsExtensionController> extensionControllers = new ArrayList<PetalsExtensionController>();
        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        final String unitTest3 = "Unit Test 3";
        extensionControllers.add(new ExtensionController(unitTest2, new String[] { unitTest1 }));
        extensionControllers.add(new ExtensionController(unitTest1, null));
        extensionControllers.add(new ExtensionController(unitTest3, new String[] { unitTest1,
                unitTest2 }));

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        final List<PetalsExtensionController> sortedExtensionController = (List<PetalsExtensionController>) ReflectionHelper
                .invokePrivateMethod(extMgr,
                        ExtensionsManagerImplTest.METHOD_NAME_SORT_EXTENSION_CONTROLLERS,
                        new Object[] { extensionControllers }, new Class[] { List.class });
        assertNotNull("Null result", sortedExtensionController);
        assertEquals("Unexpected size", 3, sortedExtensionController.size());
        assertEquals("Unexpected first element", unitTest1, sortedExtensionController.get(0)
                .getExtensionName());
        assertEquals("Unexpected 2nd element", unitTest2, sortedExtensionController.get(1)
                .getExtensionName());
        assertEquals("Unexpected 3rd element", unitTest3, sortedExtensionController.get(2)
                .getExtensionName());
    }

    /**
     * Check the processing of
     * {@link ExtensionsManagerImpl#sortExtensionControllers} against a list of
     * extensions not sorted with a not satisfied dependency.
     * 
     */
    @Test
    public void testSortExtensionControllers_03() throws Throwable {

        this.expectedEx.expect(NotSatisfiedDependenciesException.class);
        // TODO: Add a check on the content of
        // NotSatisfiedDependenciesException.extensionControllers

        final List<PetalsExtensionController> extensionControllers = new ArrayList<PetalsExtensionController>();
        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        final String unitTest3 = "Unit Test 3";
        final String unitTest4 = "Unit Test 4";
        final String unitTest5 = "Unit Test 5";
        extensionControllers.add(new ExtensionController(unitTest2, new String[] { unitTest1 }));
        extensionControllers.add(new ExtensionController(unitTest1, null));
        extensionControllers.add(new ExtensionController(unitTest3, new String[] { unitTest1,
                unitTest2 }));
        extensionControllers.add(new ExtensionController(unitTest5, new String[] { unitTest4 }));

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        List<PetalsExtensionController> sortedExtensionController;
        try {
            sortedExtensionController = (List<PetalsExtensionController>) ReflectionHelper
                    .invokePrivateMethod(extMgr,
                            ExtensionsManagerImplTest.METHOD_NAME_SORT_EXTENSION_CONTROLLERS,
                            new Object[] { extensionControllers }, new Class[] { List.class });
        } catch (final InvocationTargetException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Check the processing of
     * {@link ExtensionsManagerImpl#isAllDependenciesLoaded} against a extension
     * having no dependency.
     * </p>
     * <p>
     * Expected result: <code>true</code>
     * </p>
     * 
     */
    @Test
    public void testIsAllDependenciesLoaded_00() throws InvocationTargetException {

        final String unitTest1 = "Unit Test 1";
        final PetalsExtensionController pec1 = new ExtensionController(unitTest1, null, true);

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        final Boolean isAllDependenciesLoaded = (Boolean) ReflectionHelper.invokePrivateMethod(
                extMgr, ExtensionsManagerImplTest.METHOD_NAME_IS_ALL_DEPENDENCIES_LOADED,
                new Object[] { pec1 }, new Class[] { PetalsExtensionController.class });
        assertNotNull("Null result", isAllDependenciesLoaded);
        assertEquals("The extension should be activated", true,
                isAllDependenciesLoaded.booleanValue());
    }

    /**
     * <p>
     * Check the processing of
     * {@link ExtensionsManagerImpl#isAllDependenciesLoaded} against a extension
     * having a dependency not activated.
     * </p>
     * <p>
     * Expected result: <code>false</code>
     * </p>
     * 
     */
    @Test
    public void testIsAllDependenciesLoaded_01() throws InvocationTargetException {

        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        final PetalsExtensionController pec2 = new ExtensionController(unitTest2,
                new String[] { unitTest1 }, true);

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();

        // No extension controller is loaded when creating the extension
        // manager, so the extension controller of 'unitTest1' is not loaded.

        final Boolean isAllDependenciesLoaded = (Boolean) ReflectionHelper.invokePrivateMethod(
                extMgr, ExtensionsManagerImplTest.METHOD_NAME_IS_ALL_DEPENDENCIES_LOADED,
                new Object[] { pec2 }, new Class[] { PetalsExtensionController.class });
        assertNotNull("Null result", isAllDependenciesLoaded);
        assertEquals("The extension should not be activated", false,
                isAllDependenciesLoaded.booleanValue());
    }

    /**
     * <p>
     * Check the processing of
     * {@link ExtensionsManagerImpl#isAllDependenciesLoaded} against a extension
     * having a dependency not activated and another activated.
     * </p>
     * <p>
     * Expected result: <code>false</code>
     * </p>
     * 
     */
    @Test
    public void testIsAllDependenciesLoaded_02() throws InvocationTargetException {

        final String unitTest1 = "Unit Test 1";
        final String unitTest2 = "Unit Test 2";
        final PetalsExtensionController pec2 = new ExtensionController(unitTest2, new String[] {},
                true);
        final String unitTest3 = "Unit Test 3";
        final PetalsExtensionController pec3 = new ExtensionController(unitTest3, new String[] {
                unitTest1, unitTest2 }, true);

        final ExtensionsManagerImpl extMgr = new ExtensionsManagerImpl();
        final Map<PetalsExtensionController, Component> extensionControllers = (Map<PetalsExtensionController, Component>) ReflectionHelper
                .getPrivateFieldValue(ExtensionsManagerImpl.class, extMgr,
                        ATTRIBUTE_NAME_EXTENSION_CONTROLLERS);
        // The extension 'unitTest1' is not loaded so not activated
        extensionControllers.put(pec2, null);

        final Boolean isAllDependenciesLoaded = (Boolean) ReflectionHelper.invokePrivateMethod(
                extMgr, ExtensionsManagerImplTest.METHOD_NAME_IS_ALL_DEPENDENCIES_LOADED,
                new Object[] { pec3 }, new Class[] { PetalsExtensionController.class });
        assertNotNull("Null result", isAllDependenciesLoaded);
        assertEquals("The extension should not be activated", false,
                isAllDependenciesLoaded.booleanValue());
    }

}
