/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.undeploy;

import java.util.logging.Logger;

import junit.framework.TestCase;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Test;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.util.AbstractMavenTest;
import org.ow2.petals.systemstate.generated.ServiceAssembly;

/**
 * Test class for the task {@link RemoveSAStateHolderTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class RemoveSAStateHolderTaskTest extends AbstractMavenTest {

    final static private String SA_NAME = "myServiceAssemblyName";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            RemoveSAStateHolderTaskTest.class.getName()));

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSAStateHolderTask_000() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteServiceAssemblyStateHolder(SA_NAME)).andAnswer(
                new IAnswer<ServiceAssembly>() {

                    @Override
                    public ServiceAssembly answer() throws Throwable {
                        final ServiceAssembly serviceAssembly = new ServiceAssembly();
                        serviceAssembly.setName(SA_NAME);
                        return serviceAssembly;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final Context ctx = new Context();
        ctx.setEntityName(SA_NAME);
        final Task task = new RemoveSAStateHolderTask(log, systemStateSvc, null);

        // Process the task.
        task.execute(ctx);
        // Check returned value of the task
        TestCase.assertEquals(SA_NAME, ctx.getServiceAssemblyStateHolder().getName());

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task twice for the same service assembly.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponentStateHolderTask_001() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteServiceAssemblyStateHolder(SA_NAME)).andAnswer(
                new IAnswer<ServiceAssembly>() {

                    @Override
                    public ServiceAssembly answer() throws Throwable {
                        final ServiceAssembly serviceAssembly = new ServiceAssembly();
                        serviceAssembly.setName(SA_NAME);
                        return serviceAssembly;
                    }
                });
        EasyMock.expect(systemStateSvc.deleteServiceAssemblyStateHolder(SA_NAME)).andReturn(null);
        EasyMock.replay(systemStateSvc);

        // First, process the task for the 1st installation of the service
        // assembly.
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(SA_NAME);
            final Task task1 = new RemoveSAStateHolderTask(log, systemStateSvc, null);

            task1.execute(ctx1);

            // Check returned value of the task
            TestCase.assertEquals(SA_NAME, ctx1.getServiceAssemblyStateHolder().getName());
        }

        // Second, process the task for the 2nd installation of the service
        // assembly.
        {
            final Context ctx2 = new Context();
            ctx2.setEntityName(SA_NAME);
            final Task task2 = new RemoveSAStateHolderTask(log, systemStateSvc, null);

            task2.execute(ctx2);

            // Check returned value of the task
            TestCase.assertNull("Returned service assembly state holder not null",
                    ctx2.getServiceAssemblyStateHolder());
        }

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

}
