/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import java.util.logging.Logger;

import junit.framework.TestCase;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Test;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.installation.uninstall.component.RemoveComponentStateHolderTask;
import org.ow2.petals.microkernel.util.AbstractMavenTest;
import org.ow2.petals.systemstate.generated.Component;

/**
 * Test class for the task {@link RemoveComponentStateHolderTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class RemoveComponentStateHolderTaskTest extends AbstractMavenTest {

    final static private String COMPONENT_NAME = "myComponentName";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            RemoveComponentStateHolderTaskTest.class.getName()));

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponentStateHolderTask_000() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME)).andAnswer(
                new IAnswer<Component>() {

                    @Override
                    public Component answer() throws Throwable {
                        final Component component = new Component();
                        component.setName(COMPONENT_NAME);
                        return component;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        final Task task = new RemoveComponentStateHolderTask(log, systemStateSvc, null);

        // Process the task.
        task.execute(ctx);
        // Check returned value of the task
        TestCase.assertEquals(COMPONENT_NAME, ctx.getComponentStateHolder().getName());

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task twice for the same component.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponentStateHolderTask_001() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME)).andAnswer(
                new IAnswer<Component>() {

                    @Override
                    public Component answer() throws Throwable {
                        final Component component = new Component();
                        component.setName(COMPONENT_NAME);
                        return component;
                    }
                });
        EasyMock.expect(systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME)).andReturn(null);
        EasyMock.replay(systemStateSvc);

        // First, process the task for the 1st installation of the component.
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(COMPONENT_NAME);
            final Task task1 = new RemoveComponentStateHolderTask(log, systemStateSvc, null);

            task1.execute(ctx1);

            // Check returned value of the task
            TestCase.assertEquals(COMPONENT_NAME, ctx1.getComponentStateHolder().getName());
        }

        // Second, process the task for the 2nd installation of the component.
        {
            final Context ctx2 = new Context();
            ctx2.setEntityName(COMPONENT_NAME);
            final Task task2 = new RemoveComponentStateHolderTask(log, systemStateSvc, null);

            task2.execute(ctx2);

            // Check returned value of the task
            TestCase.assertNull("Returned component state holder not null",
                    ctx2.getComponentStateHolder());
        }

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

}
