/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import static org.junit.Assert.assertNotNull;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

/**
 * @author ddesjardins - EBM WebSourcing
 */
public class ComponentClassLoaderTest {

    /**
     * ComponentClassLoader to test
     */
    private ComponentClassLoader componentClassLoader;

    /**
     * Shared library class loader
     */
    private SharedLibraryClassLoader sharedLibraryClassLoader;

    @Before
    public void setUp() throws IOException, URISyntaxException {
        SharedLibrariesClassLoader sharedLibrariesClassLoader = new SharedLibrariesClassLoader(this
                .getClass().getClassLoader());
        List<URL> baseUrls = new ArrayList<URL>();
        List<String> classpathLocations = new ArrayList<String>();
        classpathLocations.add("petals-core-test.jar");

        final URL oneResourceURL = Thread.currentThread().getContextClassLoader()
                .getResource("resourceBoth.txt");
        final File oneResourceFile = new File(oneResourceURL.toURI());
        final File testClassesDir = oneResourceFile.getParentFile();
        final File targetDir = testClassesDir.getParentFile();
        baseUrls.add(testClassesDir.toURI().toURL());
        baseUrls.add(targetDir.toURI().toURL());
        sharedLibraryClassLoader = new SharedLibraryClassLoader(baseUrls.toArray(new URL[0]),
                classpathLocations);
        sharedLibraryClassLoader.setParentFirst(false);
        sharedLibrariesClassLoader.addSharedLibraryClassLoader("1", sharedLibraryClassLoader);

        baseUrls = new ArrayList<URL>();
        classpathLocations = new ArrayList<String>();
        classpathLocations.add("petals-common-test.jar");

        baseUrls.add(testClassesDir.toURI().toURL());
        List<String> slNames = new ArrayList<String>();
        slNames.add("1");
        componentClassLoader = new ComponentClassLoader(baseUrls.toArray(new URL[0]),
                classpathLocations, sharedLibrariesClassLoader, slNames);
    }

    /**
     * Display the classpath
     */
    @Test
    public void testGetClasspath() {
        String compoCP = "testGetClasspath - component classpath "
                + componentClassLoader.getClasspath();
        compoCP.length();
        String slCP = "testGetClasspath - shared library classpath "
                + sharedLibraryClassLoader.getClasspath();
        slCP.length();
    }

    /**
     * Test to get a resource from itself The resource is in the component
     * classpath and in the shared library classpath
     */
    @Test
    public void testGetResourceInSelfAndParentSelfFirst() {
        componentClassLoader.setParentFirst(false);
        URL url = componentClassLoader.getResource("resourceBoth.txt");
        assertNotNull("testGetResourceInSelfAndParentSelfFirst - resource not found", url);
    }

    /**
     * Test to get a resource from its parent The resource is in the component
     * classpath and in the shared library classpath
     */
    @Test
    public void testGetResourceInSelfAndParentParentFirst() {

        componentClassLoader.setParentFirst(true);
        URL url = componentClassLoader.getResource("resourceBoth.txt");
        assertNotNull("testGetResourceInSelfAndParentParentFirst - resource not found", url);
    }

    /**
     * Test to get a resource as stream from itself The resource is in the
     * component classpath and in the shared library classpath
     * 
     * @throws IOException
     */
    @Test
    public void testGetResourceAsStreamInSelfAndParentSelfFirst() throws IOException {

        componentClassLoader.setParentFirst(false);
        InputStream inputStream = componentClassLoader.getResourceAsStream("resourceBoth.txt");
        assertNotNull("testGetResourceAsStreamInSelfAndParentSelfFirst - resource not found",
                inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);

    }

    /**
     * Test to get a resource as stream from its parent The resource is in the
     * component classpath and in the shared library classpath
     * 
     * @throws IOException
     * 
     */
    @Test
    public void testGetResourceAsStreamInSelfAndParentParentFirst() throws IOException {

        componentClassLoader.setParentFirst(true);
        InputStream inputStream = componentClassLoader.getResourceAsStream("resourceBoth.txt");
        assertNotNull("testGetResourceAsStreamInSelfAndParentParentFirst - resource not found",
                inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);
    }

    /**
     * Test to get a resource as stream from its self The resource is in the
     * shared library classpath
     * 
     * @throws IOException
     */
    @Test
    public void testGetResourceAsStreamInParentSelfFirst() throws IOException {

        componentClassLoader.setParentFirst(false);
        InputStream inputStream = componentClassLoader.getResourceAsStream("resourceCore.txt");
        assertNotNull("testGetResourceAsStreamInParentSelfFirst - resource not found", inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);
    }

    /**
     * Test to get a resource as stream from its parent The resource is in the
     * component classpath
     * 
     * @throws IOException
     */
    @Test
    public void testGetResourceAsStreamInSelfParentFirst() throws IOException {

        componentClassLoader.setParentFirst(true);
        InputStream inputStream = componentClassLoader.getResourceAsStream("resourceContainer.txt");
        assertNotNull("testGetResourceAsStreamInSelfParentFirst - resource not found", inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);
    }

    /**
     * Test to load a class from its parent
     */
    @Test
    public void testLoadClassParentFirst() throws ClassNotFoundException {

        componentClassLoader.setParentFirst(true);
        final Class<?> clazz = componentClassLoader.loadClass(
                "org.objectweb.petals.ServiceContext", false);
        assertNotNull("testLoadClassParentFirst - class not found", clazz);
    }

    /**
     * Test to load a class from its self
     */
    @Test
    public void testLoadClassSelfFirst() throws ClassNotFoundException {

        componentClassLoader.setParentFirst(false);
        final Class<?> clazz = componentClassLoader.loadClass(
                "org.objectweb.petals.PetalsException", false);
        assertNotNull("testLoadClassSelfFirst - class not found", clazz);
    }

    /**
     * Test to load a class from its parent The class is in both classpath
     * (parent and self)
     */
    @Test
    public void testLoadClassInSelfAndParentParentFirst() throws InstantiationException,
            IllegalAccessException, ClassNotFoundException {

        componentClassLoader.setParentFirst(true);
        final Class<?> clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.classloader.TestClassloader", false);
        assertNotNull("testLoadClassInSelfAndParentParentFirst - class not found", clazz);
    }

    /**
     * Test to load a class from its self The class is in both classpath (parent
     * and self)
     */
    @Test
    public void testLoadClassInSelfAndParentSelfFirst() throws InstantiationException,
            IllegalAccessException, ClassNotFoundException {

        componentClassLoader.setParentFirst(false);
        final Class<?> clazz = componentClassLoader.loadClass(
                    "org.objectweb.petals.classloader.TestClassloader", false);
        assertNotNull("testLoadClassInSelfAndParentSelfFirst - class not found", clazz);
    }

}
