/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.repository;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;

import org.apache.commons.io.FileUtils;
import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.management.JbiArchiveType;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.system.repository.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.ServiceAssemblyAlreadyExistsException;
import org.ow2.petals.microkernel.api.system.repository.exception.SharedLibraryAlreadyExistsException;
import org.ow2.petals.microkernel.util.AbstractMavenTest;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Test class for the file-based repository service.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class RepositoryServiceImplTest extends AbstractMavenTest {

    private final static String COMPONENT_NAME = "petals-se-helloworld";

    private final static String SA_NAME = "petals-se-helloworld";

    private final static String SL_NAME = "petals-se-helloworld";

    private final static String SL_VERSION = "1.0.0";

    private final static String REPOSITORY_DIRNAME = "repository";

    private File archiveSourceDir;

    @Before
    public void setUp() throws Exception {
        super.setUp();

        this.archiveSourceDir = new File(this.testResourcesDir, "repository-service");
    }

    private RepositoryService createRepositoryService(final String repositoryDirectory)
            throws Exception {
        final ConfigurationService configurationServiceMock = EasyMock
                .createMock(ConfigurationService.class);
        EasyMock.expect(configurationServiceMock.getContainerConfiguration()).andAnswer(
                new IAnswer<ContainerConfiguration>() {

                    @Override
                    public ContainerConfiguration answer() throws Throwable {
                        final ContainerConfiguration contCfg = new ContainerConfiguration();
                        contCfg.setRepositoryDirectoryPath(repositoryDirectory);
                        return contCfg;
                    }
                });
        EasyMock.replay(configurationServiceMock);

        final RepositoryService repositorySvc = new RepositoryServiceImpl();

        // Inject dependencies
        ReflectionHelper.setPrivateField(repositorySvc, "configurationService",
                configurationServiceMock);

        // Start the system state implementation
        final Method methodStart = repositorySvc.getClass().getDeclaredMethod("start");
        methodStart.setAccessible(true);
        methodStart.invoke(repositorySvc);

        return repositorySvc;

    }

    /**
     * Add a component in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddComponent_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddComponent_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File componentDir = new File(testHomeDir, "component");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, COMPONENT_NAME), componentDir);

        try {
            final File installationRoot = repositorySvc.addComponent(COMPONENT_NAME, componentDir);
            RepositoryServiceImplTest.assertComponentInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final ComponentAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    private static void assertComponentInstallationRoot(final File installationRoot,
            final File repositoryHomeDir) {

        assertNotNull(installationRoot);
        assertTrue(installationRoot.exists());
        assertTrue(installationRoot.getAbsolutePath().startsWith(
                repositoryHomeDir.getAbsolutePath()));
        assertTrue(installationRoot.getAbsolutePath().endsWith("install"));
        assertTrue(new File(installationRoot.getParentFile(), "work").exists());
        assertTrue(new File(installationRoot, "META-INF").exists());
        assertTrue(new File(new File(installationRoot, "META-INF"), "jbi.xml").exists());
        assertTrue(new File(installationRoot, "petals-se-helloworld.jar").exists());
        assertTrue(installationRoot.getParentFile().getParent().endsWith("components"));

    }

    /**
     * Add a component twice in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddComponent_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddComponent_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());
        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File componentDir = new File(testHomeDir, "component");
        // First component installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, COMPONENT_NAME), componentDir);

            final File installationRoot = repositorySvc.addComponent(COMPONENT_NAME, componentDir);
            RepositoryServiceImplTest.assertComponentInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final ComponentAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

        // Second component installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, COMPONENT_NAME), componentDir);
            repositorySvc.addComponent(COMPONENT_NAME, componentDir);
            fail("The exception " + ComponentAlreadyExistsException.class.getName()
                    + " is not raised");
        } catch (final ComponentAlreadyExistsException e) {
            // Expected exception
            assertEquals(COMPONENT_NAME, e.getComponentName());
            assertTrue(e.getMessage().contains(COMPONENT_NAME));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Add a component in a repository that has been deleted
     * 
     * @throws Exception
     */
    @Test
    public void testAddComponent_003() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddComponent_003");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File componentDir = new File(testHomeDir, "component");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, COMPONENT_NAME), componentDir);

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
            repositorySvc.addComponent(COMPONENT_NAME, componentDir);
            fail("The exception " + IOException.class.getName() + " is not raised");
        } catch (final ComponentAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

    /**
     * Add a shared library in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddSharedLibrary_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddSharedLibrary_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File slDir = new File(testHomeDir, "sl");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SL_NAME), slDir);

        try {
            final File installationRoot = repositorySvc
                    .addSharedLibrary(SL_NAME, SL_VERSION, slDir);
            RepositoryServiceImplTest.assertSharedLibraryInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final SharedLibraryAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    private static void assertSharedLibraryInstallationRoot(final File installationRoot,
            final File repositoryHomeDir) {

        assertNotNull(installationRoot);
        assertTrue(installationRoot.exists());
        assertTrue(installationRoot.getAbsolutePath().startsWith(
                repositoryHomeDir.getAbsolutePath()));
        assertTrue(installationRoot.getAbsolutePath().endsWith("install"));
        assertTrue(new File(installationRoot.getParentFile(), "work").exists());
        assertTrue(new File(installationRoot, "META-INF").exists());
        assertTrue(new File(new File(installationRoot, "META-INF"), "jbi.xml").exists());
        assertTrue(new File(installationRoot, "petals-se-helloworld.jar").exists());
        assertTrue(installationRoot.getParentFile().getParent()
                .endsWith("shared-libraries"));

    }

    /**
     * Add a shared library twice in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddSharedLibrary_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddSharedLibrary_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());
        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File slDir = new File(testHomeDir, "sl");
        // First shared library installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, SL_NAME), slDir);
            final File installationRoot = repositorySvc
                    .addSharedLibrary(SL_NAME, SL_VERSION, slDir);
            RepositoryServiceImplTest.assertSharedLibraryInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final SharedLibraryAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

        // Second component installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, SL_NAME), slDir);
            repositorySvc.addSharedLibrary(SL_NAME, SL_VERSION, slDir);
            fail("The exception " + SharedLibraryAlreadyExistsException.class.getName()
                    + " is not raised");
        } catch (final SharedLibraryAlreadyExistsException e) {
            // Expected exception
            assertEquals(SL_NAME, e.getSharedLibraryName());
            assertEquals(SL_VERSION, e.getVersion());
            assertTrue(e.getMessage().contains(SL_NAME));
            assertTrue(e.getMessage().contains(SL_VERSION));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Add a shared library in a repository that has been deleted.
     * 
     * @throws Exception
     */
    @Test
    public void testAddSharedLibrary_003() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddSharedLibrary_003");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File slDir = new File(testHomeDir, "sl");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SL_NAME), slDir);

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
           repositorySvc.addSharedLibrary(SL_NAME, SL_VERSION, slDir);
            fail("The exception " + IOException.class.getName() + " is not raised");
        } catch (final SharedLibraryAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

    /**
     * Add a service assembly in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddServiceAssembly_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddServiceAssembly_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File saDir = new File(testHomeDir, "sa");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SA_NAME), saDir);

        try {
            final File installationRoot = repositorySvc.addServiceAssembly(SA_NAME, saDir);
            RepositoryServiceImplTest.assertServiceAssemblyInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final ServiceAssemblyAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    private static void assertServiceAssemblyInstallationRoot(final File installationRoot,
            final File repositoryHomeDir) {

        assertNotNull(installationRoot);
        assertTrue(installationRoot.exists());
        assertTrue(installationRoot.getAbsolutePath().startsWith(
                repositoryHomeDir.getAbsolutePath()));
        assertTrue(installationRoot.getAbsolutePath().endsWith("install"));
        assertTrue(new File(installationRoot.getParentFile(), "work").exists());
        assertTrue(new File(installationRoot, "META-INF").exists());
        assertTrue(new File(new File(installationRoot, "META-INF"), "jbi.xml").exists());
        assertTrue(new File(installationRoot, "petals-se-helloworld.jar").exists());
        assertTrue(installationRoot.getParentFile().getParent()
                .endsWith("service-assemblies"));

    }

    /**
     * Add a service assembly twice in an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testAddServiceAssembly_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddServiceAssembly_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());
        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File saDir = new File(testHomeDir, "sa");
        // First shared library installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, SA_NAME), saDir);
            final File installationRoot = repositorySvc.addServiceAssembly(SA_NAME, saDir);
            RepositoryServiceImplTest.assertServiceAssemblyInstallationRoot(installationRoot,
                    repositoryHomeDir);

        } catch (final ServiceAssemblyAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

        // Second component installation
        try {
            FileUtils.copyDirectory(new File(this.archiveSourceDir, SA_NAME), saDir);
            repositorySvc.addServiceAssembly(SA_NAME, saDir);
            fail("The exception " + ServiceAssemblyAlreadyExistsException.class.getName()
                    + " is not raised");
        } catch (final ServiceAssemblyAlreadyExistsException e) {
            // Expected exception
            assertEquals(SA_NAME, e.getServiceAssemblyName());
            assertTrue(e.getMessage().contains(SA_NAME));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Add a service assembly in a repository that has been deleted.
     * 
     * @throws Exception
     */
    @Test
    public void testAddServiceAssembly_003() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testAddServiceAssembly_003");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        final File saDir = new File(testHomeDir, "sa");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SA_NAME), saDir);

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
            repositorySvc.addServiceAssembly(SA_NAME, saDir);
            fail("The exception " + IOException.class.getName() + " is not raised");
        } catch (final ServiceAssemblyAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

    /**
     * Remove a component from an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponent_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveComponent_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        try {
            repositorySvc.removeComponent(COMPONENT_NAME);
            fail("The exception " + FileNotFoundException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(COMPONENT_NAME));
            assertTrue(e.getMessage().contains(JbiArchiveType.COMPONENT_TYPE.toString()));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a component from an repository containing the component
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponent_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveComponent_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // First, we install the component into the repository
        final File componentDir = new File(testHomeDir, "component");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, COMPONENT_NAME), componentDir);

        try {
            final File installationRoot = repositorySvc.addComponent(COMPONENT_NAME, componentDir);
            RepositoryServiceImplTest.assertComponentInstallationRoot(installationRoot,
                    repositoryHomeDir);

            // Second, we remove the component from the repository
            repositorySvc.removeComponent(COMPONENT_NAME);
            assertEquals("Remaining component in the repository", 0, installationRoot
                    .getParentFile().getParentFile().list().length);

        } catch (final ComponentAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a component from an unexisting repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveComponent_002() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveComponent_002");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
            repositorySvc.removeComponent(COMPONENT_NAME);
            fail("The exception " + IOException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

    /**
     * Remove a shared library from an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSharedLibrary_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveSharedLibrary_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        try {
            repositorySvc.removeSharedLibrary(SL_NAME, SL_VERSION);
            fail("The exception " + FileNotFoundException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(SL_NAME));
            assertTrue(e.getMessage().contains(SL_VERSION));
            assertTrue(e.getMessage().contains(JbiArchiveType.SL_TYPE.toString()));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a shared library from an repository containing the component
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSharedLibrary_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveSharedLibrary_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // First, we install the shared library into the repository
        final File slDir = new File(testHomeDir, "sl");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SL_NAME), slDir);

        try {
            final File installationRoot = repositorySvc
                    .addSharedLibrary(SL_NAME, SL_VERSION, slDir);
            RepositoryServiceImplTest.assertSharedLibraryInstallationRoot(installationRoot,
                    repositoryHomeDir);

            // Second, we remove the shared library from the repository
            repositorySvc.removeSharedLibrary(SL_NAME, SL_VERSION);
            assertEquals("Remaining shared library in the repository", 0, installationRoot
                    .getParentFile().getParentFile().list().length);

        } catch (final SharedLibraryAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a shared library from an unexisting repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSharedLibrary_002() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveSharedLibrary_002");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
            repositorySvc.removeSharedLibrary(SL_NAME, SL_VERSION);
            fail("The exception " + IOException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

    /**
     * Remove a service assembly from an empty repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveServiceAssembly_000() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveServiceAssembly_000");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        try {
            repositorySvc.removeServiceAssembly(SA_NAME);
            fail("The exception " + FileNotFoundException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(SA_NAME));
            assertTrue(e.getMessage().contains(JbiArchiveType.SA_TYPE.toString()));
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a service assembly from an repository containing the component
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveServiceAssembly_001() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveServiceAssembly_001");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // First, we install the service assembly into the repository
        final File saDir = new File(testHomeDir, "sA");
        FileUtils.copyDirectory(new File(this.archiveSourceDir, SA_NAME), saDir);

        try {
            final File installationRoot = repositorySvc.addServiceAssembly(SA_NAME, saDir);
            RepositoryServiceImplTest.assertServiceAssemblyInstallationRoot(installationRoot,
                    repositoryHomeDir);

            // Second, we remove the service assembly from the repository
            repositorySvc.removeServiceAssembly(SA_NAME);
            assertEquals("Remaining service assembly in the repository", 0,
                    installationRoot.getParentFile().getParentFile().list().length);

        } catch (final ServiceAssemblyAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Remove a service assembly from an unexisting repository
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveServiceAssembly_002() throws Exception {

        final File testHomeDir = new File(this.targetDir, "testRemoveServiceAssembly_002");
        if (testHomeDir.exists()) {
            try {
                FileUtils.deleteDirectory(testHomeDir);
            } catch (final IOException e) {
                fail("Unable to clean previous test execution.");
            }
        }
        final File repositoryHomeDir = new File(testHomeDir, REPOSITORY_DIRNAME);
        assertTrue(repositoryHomeDir.mkdirs());

        final RepositoryService repositorySvc = this.createRepositoryService(repositoryHomeDir
                .getAbsolutePath());

        // We delete the repository folder
        try {
            FileUtils.deleteDirectory(repositoryHomeDir);
        } catch (final IOException e) {
            fail("Unbale to delete the repository folder.");
        }

        try {
            repositorySvc.removeServiceAssembly(SA_NAME);
            fail("The exception " + IOException.class.getName() + " is not raised");

        } catch (final FileNotFoundException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final IOException e) {
            // Expected exception
            assertTrue(e.getMessage().contains(repositoryHomeDir.getAbsolutePath()));
        }
    }

}