/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.systemstate;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.util.List;

import javax.jbi.management.LifeCycleMBean;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.system.systemstate.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.system.systemstate.exception.ServiceAssemblyAlreadyExistsException;
import org.ow2.petals.microkernel.system.systemstate.exception.SharedLibraryAlreadyExistsException;
import org.ow2.petals.microkernel.util.AbstractMavenTest;
import org.ow2.petals.systemstate.SystemStateBuilder;
import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.systemstate.generated.Components;
import org.ow2.petals.systemstate.generated.ServiceAssemblies;
import org.ow2.petals.systemstate.generated.ServiceAssembly;
import org.ow2.petals.systemstate.generated.SharedLibraries;
import org.ow2.petals.systemstate.generated.SharedLibrary;
import org.ow2.petals.systemstate.generated.SystemState;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Test class for the file-based system state service.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class SystemStateServiceImplTest extends AbstractMavenTest {

    private final static String COMPONENT_NAME = "myComponentName";

    private final static String SA_NAME = "myServiceAssemblyName";

    private final static String SL_NAME = "mySharedLibrarieName";

    private final static String SL_VERSION = "1.0.0";

    private final static String ARCHIVE_NAME = "archive.zip";

    private final static String SYSTEMSTATE_FILE = "system-state.xml";

    private SystemStateService createSystemStateService(final File repositoryDirectory)
            throws Exception {
        final RepositoryService repositoryServiceMock = EasyMock
                .createMock(RepositoryService.class);
        EasyMock.expect(repositoryServiceMock.getRepositoryDirectory()).andReturn(
                repositoryDirectory);
        EasyMock.replay(repositoryServiceMock);

        final SystemStateService systemStateSvc = new SystemStateServiceImpl();

        // Inject dependencies
        ReflectionHelper
                .setPrivateField(systemStateSvc, "repositoryService", repositoryServiceMock);

        // Start the system state implementation
        final Method methodStart = systemStateSvc.getClass().getDeclaredMethod("start");
        methodStart.setAccessible(true);
        methodStart.invoke(systemStateSvc);

        return systemStateSvc;

    }

    /**
     * Test adding a component (not already registered) into the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateComponentStateHolder_000");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File componentHomeDir = new File(repositoryHomeDir, COMPONENT_NAME);
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            systemStateSvc.createComponentStateHolder(COMPONENT_NAME, componentHomeDir.toURI()
                    .toURL(), ARCHIVE_NAME);

            final List<Component> components = systemStateSvc.getComponentStateHolders();
            assertEquals(1, components.size());
            final Component component = components.get(0);
            assertEquals(COMPONENT_NAME, component.getName());
            assertEquals(ARCHIVE_NAME, component.getArchiveName());
            assertEquals(Installer.UNINSTALLED, component.getInstallState());
            assertEquals(componentHomeDir.toURI().toURL().toString(),
                    component.getInstallURL());
            assertEquals(LifeCycleMBean.UNKNOWN, component.getLifecycleState());

        } catch (final ComponentAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test adding a component twice into the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateComponentStateHolder_001");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            // First registration of the component
            try {
                final File componentHomeDir1 = new File(repositoryHomeDir, COMPONENT_NAME);
                systemStateSvc.createComponentStateHolder(COMPONENT_NAME, componentHomeDir1.toURI()
                        .toURL(), ARCHIVE_NAME);

                final List<Component> components = systemStateSvc.getComponentStateHolders();
                assertEquals(1, components.size());
                final Component component = components.get(0);
                assertEquals(COMPONENT_NAME, component.getName());
                assertEquals(ARCHIVE_NAME, component.getArchiveName());
                assertEquals(Installer.UNINSTALLED, component.getInstallState());
                assertEquals(componentHomeDir1.toURI().toURL().toString(),
                        component.getInstallURL());
                assertEquals(LifeCycleMBean.UNKNOWN, component.getLifecycleState());

            } catch (final ComponentAlreadyExistsException e) {
                fail("Unexpected exception: " + e.getClass().getName());
            }

            // Second registration of the component
            {
                final File componentHomeDir2 = new File(repositoryHomeDir, COMPONENT_NAME);
                try {
                    systemStateSvc.createComponentStateHolder(COMPONENT_NAME, componentHomeDir2
                            .toURI().toURL(), ARCHIVE_NAME);
                    fail("The exception " + ComponentAlreadyExistsException.class.getName()
                            + " is not raised.");
                } catch (final ComponentAlreadyExistsException e) {
                    // Expected exception

                    // Check system state content
                    final List<Component> components = systemStateSvc.getComponentStateHolders();
                    assertEquals(1, components.size());
                    final Component component = components.get(0);
                    assertEquals(COMPONENT_NAME, component.getName());
                    assertEquals(ARCHIVE_NAME, component.getArchiveName());
                    assertEquals(Installer.UNINSTALLED, component.getInstallState());
                    assertEquals(componentHomeDir2.toURI().toURL().toString(),
                            component.getInstallURL());
                    assertEquals(LifeCycleMBean.UNKNOWN, component.getLifecycleState());
                }

            }

        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Test adding a service assembly (not already registered) into the system
     * state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateServiceAssemblyStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateServiceAssemblyStateHolder_000");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File saHomeDir = new File(repositoryHomeDir, SA_NAME);
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            systemStateSvc.createServiceAssemblyStateHolder(SA_NAME, saHomeDir.toURI().toURL(),
                    ARCHIVE_NAME);

            final List<ServiceAssembly> serviceAssemblies = systemStateSvc
                    .getServiceAssemblyStateHolders();
            assertEquals(1, serviceAssemblies.size());
            final ServiceAssembly serviceAssembly = serviceAssemblies.get(0);
            assertEquals(SA_NAME, serviceAssembly.getName());
            assertEquals(ARCHIVE_NAME, serviceAssembly.getArchiveName());
            assertEquals(saHomeDir.toURI().toURL().toString(),
                    serviceAssembly.getInstallURL());
            assertEquals(LifeCycleMBean.UNKNOWN, serviceAssembly.getLifecycleState());

        } catch (final ServiceAssemblyAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test adding a service assembly twice into the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateServiceAssemblyStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateServiceAssemblyStateHolder_001");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            // First registration of the service assembly
            try {
                final File saHomeDir1 = new File(repositoryHomeDir, SA_NAME);
                systemStateSvc.createServiceAssemblyStateHolder(SA_NAME,
                        saHomeDir1.toURI().toURL(), ARCHIVE_NAME);

                final List<ServiceAssembly> serviceAssemblies = systemStateSvc
                        .getServiceAssemblyStateHolders();
                assertEquals(1, serviceAssemblies.size());
                final ServiceAssembly serviceAssembly = serviceAssemblies.get(0);
                assertEquals(SA_NAME, serviceAssembly.getName());
                assertEquals(ARCHIVE_NAME, serviceAssembly.getArchiveName());
                assertEquals(saHomeDir1.toURI().toURL().toString(),
                        serviceAssembly.getInstallURL());
                assertEquals(LifeCycleMBean.UNKNOWN, serviceAssembly.getLifecycleState());

            } catch (final ServiceAssemblyAlreadyExistsException e) {
                fail("Unexpected exception: " + e.getClass().getName());
            }

            // Second registration of the service assembly
            {
                final File saHomeDir2 = new File(repositoryHomeDir, SA_NAME);
                try {
                    systemStateSvc.createServiceAssemblyStateHolder(SA_NAME, saHomeDir2.toURI()
                            .toURL(), ARCHIVE_NAME);
                    fail("The exception " + ServiceAssemblyAlreadyExistsException.class.getName()
                            + " is not raised.");
                } catch (final ServiceAssemblyAlreadyExistsException e) {
                    // Expected exception

                    // Check system state content
                    final List<ServiceAssembly> serviceAssemblies = systemStateSvc
                            .getServiceAssemblyStateHolders();
                    assertEquals(1, serviceAssemblies.size());
                    final ServiceAssembly serviceAssembly = serviceAssemblies.get(0);
                    assertEquals(SA_NAME, serviceAssembly.getName());
                    assertEquals(ARCHIVE_NAME, serviceAssembly.getArchiveName());
                    assertEquals(saHomeDir2.toURI().toURL().toString(),
                            serviceAssembly.getInstallURL());
                    assertEquals(LifeCycleMBean.UNKNOWN,
                            serviceAssembly.getLifecycleState());
                }
            }

        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Test adding a shared library (not already registered) into the system
     * state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateSharedLibraryStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateSharedLibraryStateHolder_000");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File slHomeDir = new File(repositoryHomeDir, SL_NAME);
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, slHomeDir.toURI()
                    .toURL(), ARCHIVE_NAME);

            final List<SharedLibrary> sharedLibraries = systemStateSvc
                    .getSharedLibraryStateHolders();
            assertEquals(1, sharedLibraries.size());
            final SharedLibrary sharedLibrary = sharedLibraries.get(0);
            assertEquals(SL_NAME, sharedLibrary.getName());
            assertEquals(SL_VERSION, sharedLibrary.getVersion());
            assertEquals(ARCHIVE_NAME, sharedLibrary.getArchiveName());
            assertEquals(slHomeDir.toURI().toURL().toString(),
                    sharedLibrary.getInstallURL());

        } catch (final SharedLibraryAlreadyExistsException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test adding a shared library twice into the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateSharedLibraryStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testCreateSharedLibraryStateHolder_001");
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            // First registration of the shared library
            try {
                final File slHomeDir1 = new File(repositoryHomeDir, SL_NAME);
                systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, slHomeDir1
                        .toURI().toURL(), ARCHIVE_NAME);

                final List<SharedLibrary> sharedLibraries = systemStateSvc
                        .getSharedLibraryStateHolders();
                assertEquals(1, sharedLibraries.size());
                final SharedLibrary sharedLibrary = sharedLibraries.get(0);
                assertEquals(SL_NAME, sharedLibrary.getName());
                assertEquals(SL_VERSION, sharedLibrary.getVersion());
                assertEquals(ARCHIVE_NAME, sharedLibrary.getArchiveName());
                assertEquals(slHomeDir1.toURI().toURL().toString(),
                        sharedLibrary.getInstallURL());

            } catch (final SharedLibraryAlreadyExistsException e) {
                fail("Unexpected exception: " + e.getClass().getName());
            }

            // Second registration of the shared library
            {
                final File slHomeDir2 = new File(repositoryHomeDir, SL_NAME);
                try {
                    systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, slHomeDir2
                            .toURI().toURL(), ARCHIVE_NAME);
                    fail("The exception " + SharedLibraryAlreadyExistsException.class.getName()
                            + " is not raised.");
                } catch (final SharedLibraryAlreadyExistsException e) {
                    // Expected exception

                    // Check system state content
                    final List<SharedLibrary> sharedLibraries = systemStateSvc
                            .getSharedLibraryStateHolders();
                    assertEquals(1, sharedLibraries.size());
                    final SharedLibrary sharedLibrary = sharedLibraries.get(0);
                    assertEquals(SL_NAME, sharedLibrary.getName());
                    assertEquals(SL_VERSION, sharedLibrary.getVersion());
                    assertEquals(ARCHIVE_NAME, sharedLibrary.getArchiveName());
                    assertEquals(slHomeDir2.toURI().toURL().toString(),
                            sharedLibrary.getInstallURL());
                }
            }

        } catch (final MalformedURLException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }
    }

    /**
     * Test deletion of an existing component from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteComponentStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteComponentStateHolder_000");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File componentHomeDir = new File(repositoryHomeDir, COMPONENT_NAME);

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            final Component component = new Component();
            component.setName(COMPONENT_NAME);
            component.setArchiveName(ARCHIVE_NAME);
            component.setInstallState(Installer.UNINSTALLED);
            component.setInstallURL(componentHomeDir.toURI().toURL().toString());
            component.setLifecycleState(LifeCycleMBean.UNKNOWN);
            final Components components = new Components();
            components.getComponent().add(component);
            systemState.setComponents(components);
            systemState.setServiceAssemblies(new ServiceAssemblies());
            systemState.setSharedLibraries(new SharedLibraries());
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final Component component = systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME);

            assertEquals(COMPONENT_NAME, component.getName());
            assertEquals(ARCHIVE_NAME, component.getArchiveName());
            assertEquals(Installer.UNINSTALLED, component.getInstallState());
            assertEquals(componentHomeDir.toURI().toURL().toString(),
                    component.getInstallURL());
            assertEquals(LifeCycleMBean.UNKNOWN, component.getLifecycleState());

            final List<Component> components = systemStateSvc.getComponentStateHolders();
            assertEquals(0, components.size());

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test deletion of an unexisting component from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteComponentStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteComponentStateHolder_001");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            systemState.setComponents(new Components());
            systemState.setServiceAssemblies(new ServiceAssemblies());
            systemState.setSharedLibraries(new SharedLibraries());
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final Component component = systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME);

            assertNull(component);

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test deletion of an existing service assembly from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteServiceAssemblyStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteServiceAssemblyStateHolder_000");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File saHomeDir = new File(repositoryHomeDir, SA_NAME);

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            final ServiceAssembly serviceAssembly = new ServiceAssembly();
            serviceAssembly.setName(SA_NAME);
            serviceAssembly.setArchiveName(ARCHIVE_NAME);
            serviceAssembly.setInstallURL(saHomeDir.toURI().toURL().toString());
            serviceAssembly.setLifecycleState(LifeCycleMBean.UNKNOWN);
            final ServiceAssemblies serviceAssemblies = new ServiceAssemblies();
            serviceAssemblies.getServiceAssembly().add(serviceAssembly);
            systemState.setComponents(new Components());
            systemState.setServiceAssemblies(serviceAssemblies);
            systemState.setSharedLibraries(new SharedLibraries());
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final ServiceAssembly serviceAssembly = systemStateSvc
                    .deleteServiceAssemblyStateHolder(SA_NAME);

            assertEquals(SA_NAME, serviceAssembly.getName());
            assertEquals(ARCHIVE_NAME, serviceAssembly.getArchiveName());
            assertEquals(saHomeDir.toURI().toURL().toString(),
                    serviceAssembly.getInstallURL());
            assertEquals(LifeCycleMBean.UNKNOWN, serviceAssembly.getLifecycleState());

            final List<ServiceAssembly> serviceAssemblies = systemStateSvc
                    .getServiceAssemblyStateHolders();
            assertEquals(0, serviceAssemblies.size());

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test deletion of an unexisting service assembly from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteServiceAssemblyStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteServiceAssemblyStateHolder_001");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            systemState.setComponents(new Components());
            systemState.setServiceAssemblies(new ServiceAssemblies());
            systemState.setSharedLibraries(new SharedLibraries());
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final ServiceAssembly serviceAssembly = systemStateSvc
                    .deleteServiceAssemblyStateHolder(SA_NAME);

            assertNull(serviceAssembly);

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test deletion of an existing shared library from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteSharedLibraryStateHolder_000() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteSharedLibraryStateHolder_000");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }
        final File slHomeDir = new File(repositoryHomeDir, SL_NAME);

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            final SharedLibrary sharedLibrary = new SharedLibrary();
            sharedLibrary.setName(SL_NAME);
            sharedLibrary.setVersion(SL_VERSION);
            sharedLibrary.setArchiveName(ARCHIVE_NAME);
            sharedLibrary.setInstallURL(slHomeDir.toURI().toURL().toString());
            final SharedLibraries sharedLibraries = new SharedLibraries();
            sharedLibraries.getSharedLibrary().add(sharedLibrary);
            systemState.setComponents(new Components());
            systemState.setServiceAssemblies(new ServiceAssemblies());
            systemState.setSharedLibraries(sharedLibraries);
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final SharedLibrary sharedLibrary = systemStateSvc.deleteSharedLibraryStateHolder(
                    SL_NAME, SL_VERSION);

            assertEquals(SL_NAME, sharedLibrary.getName());
            assertEquals(SL_VERSION, sharedLibrary.getVersion());
            assertEquals(ARCHIVE_NAME, sharedLibrary.getArchiveName());
            assertEquals(slHomeDir.toURI().toURL().toString(),
                    sharedLibrary.getInstallURL());

            final List<SharedLibrary> sharedLibraries = systemStateSvc
                    .getSharedLibraryStateHolders();
            assertEquals(0, sharedLibraries.size());

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }

    /**
     * Test deletion of an unexisting shared library from the system state
     * 
     * @throws Exception
     */
    @Test    
    public void testDeleteSharedLibraryStateHolder_001() throws Exception {
        final File repositoryHomeDir = new File(this.targetDir,
                "testDeleteSharedLibraryStateHolder_001");
        repositoryHomeDir.mkdirs();
        final File systemStateFile = new File(repositoryHomeDir, SYSTEMSTATE_FILE);
        if (systemStateFile.exists()) {
            assertTrue(systemStateFile.delete());
        }

        // Initialize the system state
        {
            final SystemState systemState = new SystemState();
            systemState.setComponents(new Components());
            systemState.setServiceAssemblies(new ServiceAssemblies());
            systemState.setSharedLibraries(new SharedLibraries());
            SystemStateBuilder.buildXmlFileSystemState(systemState, systemStateFile);
        }
        assertTrue(systemStateFile.exists());

        final SystemStateService systemStateSvc = this.createSystemStateService(repositoryHomeDir);

        try {
            final SharedLibrary sharedLibrary = systemStateSvc.deleteSharedLibraryStateHolder(
                    SL_NAME, SL_VERSION);

            assertNull(sharedLibrary);

        } catch (final SystemStateException e) {
            fail("Unexpected exception: " + e.getClass().getName());
        }

    }
}
